@@ -102,6 +102,9 @@ return {
      --------------------------------------------------------------------------
      e_Args_Not_Strings    = [==[Syntax error in file: %{fn}
  with command: %{cmdName}, one or more arguments are not strings.
+]==], --
+     e_Args_Not_Table    = [==[Syntax error in file: %{fn}
+ with function: %{func}, is not a table.
 ]==], --
      e_Args_Not_Strings_short = "command: %{cmdName}, one or more arguments are not strings.",
      e_Avail_No_MPATH      = "module %{name} is not possible. MODULEPATH is not set or not set with valid paths.\n",
@@ -110,6 +113,8 @@ return {
      e_BrokenCacheFn       = "Spider cache fn: \"%{fn}\" appears broken",
      e_BrokenQ             = "Internal error: broken module Q\n",
      e_Conflict            = "Cannot load module \"%{name}\" because these module(s) are loaded:\n   %{module_list}\n",
+     e_Conflict_Downstream = "Cannot load module \"%{userName}\" because this module set a conflict: \"%{fullNameUpstream}\"\n",
+     e_Dofile_not_supported = "The dofile() function is not supported.  Use require() or loadfile() or loadstring()",
      e_Execute_Msg         = [==[Syntax error in file: %{fn}
 with command: "execute".
 The syntax is:
@@ -135,6 +140,13 @@ Please check the spelling or version number. Also try "module spider ..."
 
 Also make sure that all modulefiles written in TCL start with the string #%Module
 ]==],
+     e_Failed_depends_any     = [==[The depends_on_any function failed because it could not find any of the following modules : %{module_list}
+
+Please check the spelling or version number. Also try "module spider ..."
+
+Also make sure that all modulefiles written in TCL start with the string #%Module
+]==],
+
      e_Family_Conflict     = [==[You can only have one %{name} module loaded at a time.
 You already have %{oldName} loaded.
 To correct the situation, please execute the following command:
@@ -143,14 +155,18 @@ To correct the situation, please execute the following command:
 
 Please submit a consulting ticket if you require additional assistance.
 ]==],
+     e_Forbidden           = "Access to %{fullName} is denied\n",
+     e_Forbidden_w_msg     = "Access to %{fullName} is denied\n%{literal_msg}",
      e_Illegal_Load        = [==[The following module(s) are illegal: %{module_list}
 Lmod does not support modulefiles that start with two or more underscores
 ]==],
      e_Illegal_option      = [==[Option: "%{v}" is unknown.
   Try module --help for usage.
 ]==],
-     e_Inf_Loop            = "Infinite Load Loop detected for module: \"%{fullName}\" file: \"%{file}\"",
+     e_Inf_Loop            = "A load storm (possibly an infinite loop) detected for module: \"%{fullName}\" file: \"%{file}\". It was loaded more than %{count} times.\n",
      e_LocationT_Srch      = "Error in LocationT:search().",
+     e_Missing_Action      = "Missing action internal error. sn=\"%{sn}\", msg=\"${msg}\"",
+     e_Malformed_time      = "Before or after time is malformed. Please use YYYY-MM-DD or YYYY-MM-DDTHH:MM instead of \"%{tStr}\"\n",
      e_Missing_Value       = "%{func}(\"%{name}\") is not valid; a value is required.",
      e_MT_corrupt          = [==[The module table stored in the environment is corrupt.
 please execute the command \" clearMT\" and reload your modules.
@@ -161,7 +177,7 @@ please execute the command \" clearMT\" and reload your modules.
    $ module swap %{oldFullName} %{newFullName}
 
 Alternatively, you can set the environment variable LMOD_DISABLE_SAME_NAME_AUTOSWAP to "no" to re-enable same name autoswapping.
-]==], 
+]==],  -- 
      e_No_Hashsum          = "Unable to find HashSum program (sha1sum, shasum, md5sum or md5).",
      e_No_Matching_Mods    = "No matching modules found.\n",
      e_No_Mod_Entry        = "%{routine}: Did not find module entry: \"%{name}\". This should not happen!\n",
@@ -169,6 +185,7 @@ Alternatively, you can set the environment variable LMOD_DISABLE_SAME_NAME_AUTOS
      e_No_PropT_Entry      = "%{routine}: system property table has no %{location} for: \"%{name}\". \nCheck spelling and case of name.\n",
      e_No_UUID             = "uuidgen is not available, fallback failed too",
      e_No_ValidT_Entry     = "%{routine}: The validT table for %{name} has no entry for: \"%{value}\". Make sure that all keys in displayT have a matching key in validT. \nCheck spelling and case of name.\n",
+     e_Newer_posix_reqd    = "The site's luaposix is too old to use before or after modifiers to hide modules.  Please upgrade your sites luaposix.\n",
      e_Prereq              = "Cannot load module \"%{name}\" without these module(s) loaded:\n   %{module_list}\n",
      e_Prereq_Any          = "Cannot load module \"%{name}\". At least one of these module(s) must be loaded:\n   %{module_list}\n",
      e_RequireFullName     = [==[Module "%{sn}" must be loaded with the version specified, e.g. "module load %{fullName}". Use
@@ -183,12 +200,16 @@ See https://lmod.readthedocs.io/en/latest/260_sh_to_modulefile.html for details.
 ]==],
      e_Sh_convertSh2MF     = "convertSh2MF script failed to produce 7 blocks\n",
      e_Spdr_Timeout        = "Spider search timed out.\n",
+     e_SU_defaults         = "Internal error in setting SU defaults\n",
      e_Swap_Failed         = "Swap failed: \"%{name}\" is not loaded.\n",
      e_Unable_2_Load       = "Unable to load module because of error when evaluating modulefile: %{name}\n     %{fn}: %{message}\n     Please check the modulefile and especially if there is a line number specified in the above message",
      e_Unable_2_Load_short = "%{message}",
      e_Unable_2_parse      = "Unable to parse: \"%{path}\". Aborting!\n",
      e_Unable_2_rename     = "Unable to rename %{from} to %{to}, error message: %{errMsg}",
      e_Unknown_Coll        = "User module collection: \"%{collection}\" does not exist.\n  Try \"module savelist\" for possible choices.\n",
+     e_Unknown_key         = "This is an unknown key: \"%{key}\" for the %{func} function",
+     e_Unknown_v_type      = "This is an unknown value type: \"%{tkind}\".  The value type should be: \"%{kind}\" for key: \"%{key}\" for the %{func} function",
+     e_Unknown_value       = "This is an unknown value: \"%{value} \". for key: \"%{key}\" in the %{func} function",
      e_coll_corrupt        = "The module collection file is corrupt. Please remove: %{fn}\n",
      e_dbT_sn_fail         = "dbT[sn] failed for sn: %{sn}\n",
      e_missing_table       = "sandbox_registration: The argument passed is: \"%{kind}\". It should be a table.",
@@ -204,10 +225,11 @@ See https://lmod.readthedocs.io/en/latest/260_sh_to_modulefile.html for details.
      m_Depend_Mods         = "\n    You will need to load all module(s) on any one of the lines below before the \"%{fullName}\" module is available to load.\n",
      m_Description         = "    Description:\n%{descript}\n\n",
      m_Direct_Load         = "\n    This module can be loaded directly: module load %{fullName}\n",
-     m_Extensions_head     = "This is a list of module extensions \"module --nx avail ...\" to not show.\n",
+     m_Extensions_head     = "This is a list of module extensions. Use \"module --nx avail ...\" to not show extensions.",
      m_Extensions_tail     = "\nThese extensions cannot be loaded directly, use \"module spider extension_name\" for more information.\n",
      m_Family_Swap         = "\nLmod is automatically replacing \"%{oldFullName}\" with \"%{newFullName}\".\n",
      m_For_System          = ", for system: \"%{sname}\"",
+     m_Hidden_loaded       = "\nOne or more modules are hidden from list. To see all do \"module --show_hidden list\"",
      m_Inactive_Modules    = "\nInactive Modules:\n",
      m_IsNVV               = [==[
 Module defaults are chosen based on Find First Rules due to Name/Version/Version modules found in the module tree.
@@ -229,6 +251,7 @@ To search the contents of modules for matching words execute:
 
   $   module keyword %{s}
 ]==], --
+     m_Module_Msgs_close   = "%{border}",
      m_Other_matches       = "\n     Other possible modules matches:\n        %{bb}\n",
      m_Other_possible      = [==[
      Other possible modules matches:
@@ -305,11 +328,14 @@ to see if the module(s) are available across all compilers and MPI implementatio
 %{border}
 The following dependent module(s) are not currently loaded: %{missing}
 %{border}
-]==],
+]==], 
      w_MPATH_Coll          = "The system MODULEPATH has changed: please rebuild your saved collection.\n",
      w_Mods_Not_Loaded     = "The following modules were not loaded: %{module_list}\n\n",
+     w_Nearly_Forbidden    = "Access will be denied to this module starting %{after}\n",
+     w_Nearly_Forbidden_w_msg = "Access will be denied to this module starting %{after}\n%{literal_msg}",
      w_No_Coll             = "No collection named \"%{collection}\" found.",
      w_No_dot_Coll         = "It is illegal to have a `.' in a collection name.  Please choose another name for: \"%{name}\".",
+     w_Possible_Bad_Dir    = "Adding \"%{dir}\" to $MODULEPATH. Did you mean: \"module %{dir} use\"?",
      w_SYS_DFLT_EMPTY      = [==[
 The system default contains no modules
   (env var: LMOD_SYSTEM_DEFAULT_MODULES is empty)
@@ -325,19 +351,18 @@ MODULEPATH directory: "%{mpath}" has too many non-modulefiles (%{regularFn}). Pl
 ]==],
      w_Undef_MPATH         = "MODULEPATH is undefined.\n",
      w_Unknown_Hook        = "Unknown hook: %{name}\n",
+     w_Unknown_Hook_Action = "Unknown hook action: %{action}\n",
 
 
      --------------------------------------------------------------------------
      -- Usage Message
      --------------------------------------------------------------------------
      usage_cmdline         = "module [options] sub-command [args ...]",
-     help_title            = "Help sub-commands:\n" ..
-                             "------------------",
+     help_title            = "Help sub-commands:\n\n",
      help1                 = "prints this message",
      help2                 = "print help message from module(s)",
 
-     load_title            = "Loading/Unloading sub-commands:\n" ..
-                             "-------------------------------",
+     load_title            = "Loading/Unloading sub-commands:\n\n",
      load1                 = "load module(s)",
      load2                 = "Add module(s), do not complain if not found",
      load3                 = "Remove module(s), do not complain if not found",
@@ -346,8 +371,7 @@ MODULEPATH directory: "%{mpath}" has too many non-modulefiles (%{regularFn}). Pl
      load6                 = "reload aliases from current list of modules.",
      load7                 = "reload all currently loaded modules.",
 
-     list_title            = "Listing / Searching sub-commands:\n" ..
-                             "---------------------------------",
+     list_title            = "Listing / Searching sub-commands:\n\n",
      list1                 = "List loaded modules",
      list2                 = "List loaded modules that match the pattern",
      list3                 = "List available modules",
@@ -362,15 +386,13 @@ MODULEPATH directory: "%{mpath}" has too many non-modulefiles (%{regularFn}). Pl
      ov1                   = "List all available modules by short names with number of versions",
      ov2                   = "List available modules by short names with number of versions that contain \"string\"",
 
-     srch_title            = "Searching with Lmod:\n" ..
-                             "--------------------",
+     srch_title            = "Searching with Lmod:\n\n",
      srch0                 = "  All searching (spider, list, avail, keyword) support regular expressions:",
      srch1                 = "Finds all the modules that start with `p' or `P'",
      srch2                 = "Finds all modules that have \"mpi\" in their name.",
      srch3                 = "Finds all modules that end with \"mpi\" in their name.",
 
-     collctn_title         = "Handling a collection of modules:\n"..
-                             "--------------------------------",
+     collctn_title         = "Handling a collection of modules:\n\n",
      collctn1              = "Save the current list of modules to a user defined \"default\" collection.",
      collctn2              = "Save the current list of modules to \"name\" collection.",
      collctn3              = "The same as \"restore system\"",
@@ -381,15 +403,13 @@ MODULEPATH directory: "%{mpath}" has too many non-modulefiles (%{regularFn}). Pl
      collctn8              = "Describe the contents of a module collection.",
      collctn9              = "Disable (i.e. remove) a collection.",
 
-     depr_title            = "Deprecated commands:\n" ..
-                             "--------------------",
+     depr_title            = "Deprecated commands:\n\n",
      depr1                 = "load name collection of modules or user's \"default\" if no name given.",
      depr2                 = "===> Use \"restore\" instead  <====",
      depr3                 = "Save current list of modules to name if given, otherwise save as the default list for you the user.",
      depr4                 = "===> Use \"save\" instead. <====",
 
-     misc_title            = "Miscellaneous sub-commands:\n" ..
-                             "---------------------------",
+     misc_title            = "Miscellaneous sub-commands:\n\n",
      misc1                 = "show the commands in the module file.",
      misc2                 = "Prepend or Append path to MODULEPATH.",
      misc3                 = "remove path from MODULEPATH.",
@@ -398,8 +418,7 @@ MODULEPATH directory: "%{mpath}" has too many non-modulefiles (%{regularFn}). Pl
      misc_isAvail          = "return a true status if module can be loaded",
 
 
-     env_title             = "Important Environment Variables:\n" ..
-                             "--------------------------------",
+     env_title             = "Important Environment Variables:\n\n",
      env1                  = "If defined to be \"YES\" then Lmod prints properties and warning in color.",
      web_sites             = "Lmod Web Sites",
      rpt_bug               = "  To report a bug please read ",
@@ -409,52 +428,59 @@ MODULEPATH directory: "%{mpath}" has too many non-modulefiles (%{regularFn}). Pl
      --------------------------------------------------------------------------
      -- module help strings
      --------------------------------------------------------------------------
-     StickyM   = "Module is Sticky, requires --force to unload or purge",
-     LoadedM   = "Module is loaded",
-     ExplM     = "Experimental",
-     TstM      = "Testing",
-     ObsM      = "Obsolete",
+     StickyM        = "Module is Sticky, requires --force to unload or purge",
+     LoadedM        = "Module is loaded",
+     ExplM          = "Experimental",
+     TstM           = "Testing",
+     ObsM           = "Obsolete",
 
-     help_hlp  = "This help message",
-     style_hlp = "Site controlled avail style: %{styleA} (default: %{default})",
-     rt_hlp    = "Lmod regression testing",
-     dbg_hlp   = "Program tracing written to stderr",
-     dbg_hlp2  = "Program tracing written to stderr (where dbglvl is a number 1,2,3)",
-     pin_hlp   = "When doing a restore use specified version, do not follow defaults",
-     avail_hlp = "List default modules only when used with avail",
-     quiet_hlp = "Do not print out warnings",
-     exprt_hlp = "Expert mode",
-     terse_hlp = "Write out in machine readable format for commands: list, avail, spider, savelist",
-     initL_hlp = "loading Lmod for first time in a user shell",
-     location_H = "Just print the file location when using show ",
-     latest_H  = "Load latest (ignore default)",
-     cache_hlp = "Treat the cache file(s) as out-of-date",
-     novice_H  = "Turn off expert and quiet flag",
-     raw_hlp   = "Print modulefile in raw output when used with show",
-     width_hlp = "Use this as max term width",
-     v_hlp     = "Print version info and quit",
-     rexp_hlp  = "use regular expression match",
-     gitV_hlp  = "Dump git version in a machine readable way and quit",
-     dumpV_hlp = "Dump version in a machine readable way and quit",
-     chkSyn_H  = "Checking module command syntax: do not load",
-     config_H  = "Report Lmod Configuration",
-     miniConfig_H  = "Report Lmod Configuration differences",
-     jcnfig_H  = "Report Lmod Configuration in json format",
-     MT_hlp    = "Report Module Table State",
-     timer_hlp = "report run times",
-     force_hlp = "force removal of a sticky module or save an empty collection",
-     redirect_H= "Send the output of list, avail, spider to stdout (not stderr)",
-     nrdirect_H= "Force output of list, avail and spider to stderr",
-     hidden_H  = "Avail and spider will report hidden modules",
-     spdrT_H   = "a timeout for spider",
-     trace_T   = "trace major changes such as loads",
-     nx_T      = "Do not print extensions",
+     MT_hlp         = "Report Module Table State",
+     avail_hlp      = "List default modules only when used with avail",
+     cache_hlp      = "Treat the cache file(s) as out-of-date",
+     chkSyn_H       = "Checking module command syntax: do not load",
+     config_H       = "Report Lmod Configuration",
+     dbg_hlp        = "Program tracing written to stderr",
+     dbg_hlp2       = "Program tracing written to stderr (where dbglvl is a number 1,2,3)",
+     dumpN_hlp      = "Dump the name Lmod in a machine readable way and quit",
+     dumpV_hlp      = "Dump version in a machine readable way and quit",
+     exprt_hlp      = "Expert mode",
+     force_hlp      = "force removal of a sticky module or save an empty collection",
+     gitV_hlp       = "Dump git version in a machine readable way and quit",
+     help_hlp       = "This help message",
+     hidden_H       = "Avail and spider will report hidden modules",
+     initL_hlp      = "loading Lmod for first time in a user shell",
+     jcnfig_H       = "Report Lmod Configuration in json format",
+     latest_H       = "Load latest (ignore default)",
+     location_H     = "Just print the file location when using show ",
+     miniConfig_H   = "Report Lmod Configuration differences",
+     novice_H       = "Turn off expert and quiet flag",
+     nrdirect_H     = "Force output of list, avail and spider to stderr",
+     nx_H           = "Do not print extensions",
+     pin_hlp        = "When doing a restore use specified version, do not follow defaults",
+     pod_H          = "Generate pod format",
+     quiet_hlp      = "Do not print out warnings",
+     raw_hlp        = "Print modulefile in raw output when used with show",
+     redirect_H     = "Send the output of list, avail, spider to stdout (not stderr)",
+     rexp_hlp       = "use regular expression match",
+     rt_hlp         = "Lmod regression testing",
+     spdrT_H        = "a timeout for spider",
+     style_hlp      = "Site controlled avail style: %{styleA} (default: %{default})",
+     terse_hlp      = "Write out in machine readable format for commands: list, avail, spider, savelist",
+     terseShowExt_H = "report extensions when doing a terse avail",
+     timer_hlp      = "report run times",
+     trace_H        = "trace major changes such as loads",
+     v_hlp          = "Print version info and quit",
+     width_hlp      = "Use this as max term width",
 
-     Where     = "\n  Where:\n",
-     Inactive  = "\nInactive Modules",
-     DefaultM  = "Default Module",
-     HiddenM   = "Hidden Module",
-     Extension = "Extension that is provided by another module",
+     Where          = "\n  Where:\n",
+     Inactive       = "\nInactive Modules",
+     DefaultM       = "Default Module",
+     NearlyForbiddenM = "Nearly Forbidden Module",
+     ForbiddenM     = "Forbidden Module",
+     HiddenM        = "Hidden Module",
+     HiddenLoadM    = "Hidden from load Module",
+     Hidden_softM   = "Soft Hidden Module",
+     Extension      = "Extension that is provided by another module",
 
      avail     = [==[If the avail list is too long consider trying:
 
