@@ -15,6 +15,9 @@ local cosmic     = require("Cosmic"):singleton()
 local dbg        = require("Dbg"):dbg()
 local testDir    = "spec/MRC"
 setenv_lmod_version()
+local mrc       = MRC:singleton()
+mrc:set_display_mode("spider")
+cosmic:assign("LMOD_SHOW_HIDDEN","no")
 
 describe("Testing MRC class #MRC.",
          function()
@@ -22,19 +25,21 @@ describe("Testing MRC class #MRC.",
                function()
                   local mrc = MRC:singleton()
                   local ModA={
-                     {kind="module_version",module_name="/15.0.2",      module_versionA={"15.0","15"}},
-                     {kind="module_version",module_name="intel/14.0.3", module_versionA={"14.0"}},
-                     {kind="module_version",module_name="intel/14.0",   module_versionA={"14"}},
-                     {kind="module_version",module_name="intel/14",     module_versionA={"default"}},
+                     {action="module_version",module_name="/15.0.2",      module_versionA={"15.0","15"}},
+                     {action="module_version",module_name="intel/14.0.3", module_versionA={"14.0"}},
+                     {action="module_version",module_name="intel/14.0",   module_versionA={"14"}},
+                     {action="module_version",module_name="intel/14",     module_versionA={"default"}},
                   }
+                  --dbg:activateDebug(1)
                   local defaultV = mrc:parseModA_for_moduleA("intel", "/path/to/nowhere", ModA)
+                  dbg.print{"defaultV: ",defaultV,"\n"}
                   assert.are.equal("intel/14.0.3" , defaultV)
 
                   ModA={
-                     {kind="module_version", module_name="/15.0.2", module_versionA={"15.0","15"}},
-                     {kind="module_version", module_name="/14.0.5", module_versionA={"14.0"}},
-                     {kind="module_version", module_name="/14.0",   module_versionA={"14"}},
-                     {kind="module_version", module_name="/14",     module_versionA={"default"}},
+                     {action="module_version", module_name="/15.0.2", module_versionA={"15.0","15"}},
+                     {action="module_version", module_name="/14.0.5", module_versionA={"14.0"}},
+                     {action="module_version", module_name="/14.0",   module_versionA={"14"}},
+                     {action="module_version", module_name="/14",     module_versionA={"default"}},
                   }
                   defaultV = mrc:parseModA_for_moduleA("acme", "/path/to/nowhere", ModA)
                   assert.are.equal("acme/14.0.5" , defaultV)
@@ -66,7 +71,7 @@ describe("Testing MRC class #MRC.",
                   _G.mcp             = _G.MainControl.build("load")
                   _G.MCP             = _G.MainControl.build("load")
                   posix.setenv("HOME",base, true)
-                  cosmic:assign("LMOD_MODULERCFILE",pathJoin(projDir,testDir,"dot.modulerc.lua"))
+                  cosmic:assign("LMOD_MODULERC",pathJoin(projDir,testDir,"dot.modulerc.lua"))
                   --dbg:activateDebug(1)
                   MRC:__clear()
                   local moduleA = ModuleA:singleton{reset=true, spider_cache=true}
