@@ -15,9 +15,12 @@ local FrameStk   = require("FrameStk")
 local dbg        = require("Dbg"):dbg()
 local concatTbl  = table.concat
 local cosmic     = require("Cosmic"):singleton()
+local mrc        = require("MRC"):singleton()
 local getenv     = os.getenv
 local testDir    = "spec/ModuleA"
 setenv_lmod_version()
+cosmic:assign("LMOD_SHOW_HIDDEN","no")
+mrc:set_display_mode("spider")
 describe("Testing ModuleA Class #ModuleA.",
          function()
             it("Build moduleA from mf",
@@ -82,7 +85,7 @@ describe("Testing ModuleA Class #ModuleA.",
                                           ["luaExt"] = 4,
                                           ["mpath"] = "%ProjDir%/spec/ModuleA/mf",
                                           ["pV"] = "000000032/000000001.*zfinal",
-                                          ["wV"] = "000000032/s00000001.*zfinal",
+                                          ["wV"] = "000000032/000000001.*zfinal",
                                        },
                                        ["bio/bowtie/32/2.0"]  = {
                                           ["Version"] = "32/2.0",
@@ -156,15 +159,6 @@ describe("Testing ModuleA Class #ModuleA.",
                               defaultT = {},
                               dirT = {},
                               fileT = {
-                                 ["boost/.version.1.46.0"]  = {
-                                    ["Version"] = ".version.1.46.0",
-                                    ["canonical"] = ".version.1.46.0",
-                                    ["dot_version"] = 1,
-                                    ["fn"] = "%ProjDir%/spec/ModuleA/mf/boost/.version.1.46.0",
-                                    ["mpath"] = "%ProjDir%/spec/ModuleA/mf",
-                                    ["pV"] = "*version.000000001.000000046.*zfinal",
-                                    ["wV"] = "*version.000000001.000000046.*zfinal",
-                                 },
                                  ["boost/1.46.0"]  = {
                                     ["Version"] = "1.46.0",
                                     ["canonical"] = "1.46.0",
@@ -186,7 +180,6 @@ describe("Testing ModuleA Class #ModuleA.",
                   posix.setenv("MODULEPATH",mpath,true)
                   local maxdepth = mpath .. ":2;"
                   cosmic:assign("LMOD_MAXDEPTH",maxdepth)
-                  cosmic:assign("LMOD_MODULERCFILE",pathJoin(projDir,testDir,".modulerc"))
                   _G.mcp             = _G.MainControl.build("load")
                   _G.MCP             = _G.MainControl.build("load")
                   --dbg:activateDebug(1)
@@ -232,46 +225,71 @@ describe("Testing ModuleA Class #ModuleA.",
                      {
                         A = {
                            {
-                              ["fn"] = "%ProjDir%/spec/ModuleA/mf/bio/bowtie/32/1.0.lua",
-                              ["fullName"] = "bio/bowtie/32/1.0",
-                              ["pV"] = "bio/bowtie/000000032/000000001.*zfinal",
-                              ["sn"] = "bio/bowtie",
+                              fn = "%ProjDir%/spec/ModuleA/mf/bio/bowtie/32/1.0.lua",
+                              fullName = "bio/bowtie/32/1.0",
+                              moduleKindT = {
+                                 hidden_loaded = false,
+                                 kind = "normal",
+                              },
+                              pV = "bio/bowtie/000000032/000000001.*zfinal",
+                              sn = "bio/bowtie",
                            },
                            {
-                              ["fn"] = "%ProjDir%/spec/ModuleA/mf/bio/bowtie/32/2.0.lua",
-                              ["fullName"] = "bio/bowtie/32/2.0",
-                              ["pV"] = "bio/bowtie/000000032/000000002.*zfinal",
-                              ["sn"] = "bio/bowtie",
+                              fn = "%ProjDir%/spec/ModuleA/mf/bio/bowtie/32/2.0.lua",
+                              fullName = "bio/bowtie/32/2.0",
+                              moduleKindT = {
+                                 hidden_loaded = false,
+                                 kind = "normal",
+                              },
+                              pV = "bio/bowtie/000000032/000000002.*zfinal",
+                              sn = "bio/bowtie",
                            },
                            {
-                              ["fn"] = "%ProjDir%/spec/ModuleA/mf/bio/bowtie/64/2.0.lua",
-                              ["fullName"] = "bio/bowtie/64/2.0",
-                              ["pV"] = "bio/bowtie/000000064/000000002.*zfinal",
-                              ["sn"] = "bio/bowtie",
+                              fn = "%ProjDir%/spec/ModuleA/mf/bio/bowtie/64/2.0.lua",
+                              fullName = "bio/bowtie/64/2.0",
+                              moduleKindT = {
+                                 hidden_loaded = false,
+                                 kind = "normal",
+                              },
+                              pV = "bio/bowtie/000000064/000000002.*zfinal",
+                              sn = "bio/bowtie",
                            },
                            {
-                              ["fn"] = "%ProjDir%/spec/ModuleA/mf/bio/genomics.lua",
-                              ["fullName"] = "bio/genomics",
-                              ["pV"] = "bio/genomics/M.*zfinal",
-                              ["sn"] = "bio/genomics",
-                              propT = { arch = { ["mic"] = 1} }
+                              fn = "%ProjDir%/spec/ModuleA/mf/bio/genomics.lua",
+                              fullName = "bio/genomics",
+                              moduleKindT = {
+                                 hidden_loaded = false,
+                                 kind = "normal",
+                              },
+                              pV = "bio/genomics/M.*zfinal",
+                              propT = {
+                                 arch = {
+                                    mic = 1,
+                                 },
+                              },
+                              sn = "bio/genomics",
                            },
                            {
-                              ["fn"] = "%ProjDir%/spec/ModuleA/mf/boost/1.46.0.lua",
-                              ["fullName"] = "boost/1.46.0",
-                              ["pV"] = "boost/000000001.000000046.*zfinal",
-                              ["sn"] = "boost",
+                              fn = "%ProjDir%/spec/ModuleA/mf/boost/1.46.0.lua",
+                              fullName = "boost/1.46.0",
+                              moduleKindT = {
+                                 hidden_loaded = false,
+                                 kind = "normal",
+                              },
+                              pV = "boost/000000001.000000046.*zfinal",
+                              sn = "boost",
                            },
                         },
-                        ["mpath"] = "%ProjDir%/spec/ModuleA/mf",
+                        mpath = "%ProjDir%/spec/ModuleA/mf",
                      },
                   }
 
                   local _availA = {}
                   sanizatizeTbl(rplmntA, availA, _availA)
+                  --dbg:activateDebug(1)
+                  dbg.printT("gold_availA",gold_availA)
+                  dbg.printT("_availA",    _availA)
                   assert.are.same(gold_availA, _availA)
-                  --print(serializeTbl{indent=true, name="availA",value = _availA})
-                  --print(serializeTbl{indent=true, name="gold_availA",value = gold_availA})
                end)
             it("Test of meta module and regular modules with the same name",
                function()
