@@ -39,29 +39,29 @@ require("TermWidth")
 require("string_utils")
 require("loadModuleFile")
 
-local Banner       = require("Banner")
-local BeautifulTbl = require("BeautifulTbl")
-local Cache        = require("Cache")
-local ColumnTable  = require("ColumnTable")
-local FrameStk     = require("FrameStk")
-local M            = {}
-local MRC          = require("MRC")
-local MName        = require("MName")
-local MT           = require("MT")
-local ModuleA      = require("ModuleA")
-local Spider       = require("Spider")
-local Var          = require("Var")
-local concatTbl    = table.concat
-local cosmic       = require("Cosmic"):singleton()
-local dbg          = require("Dbg"):dbg()
-local hook         = require("Hook")
-local i18n         = require("i18n")
-local remove       = table.remove
-local sort         = table.sort
-local q_load       = 0
-local s_same       = true
+local Banner        = require("Banner")
+local BeautifulTbl  = require("BeautifulTbl")
+local Cache         = require("Cache")
+local ColumnTable   = require("ColumnTable")
+local FrameStk      = require("FrameStk")
+local M             = {}
+local MRC           = require("MRC")
+local MName         = require("MName")
+local MT            = require("MT")
+local ModuleA       = require("ModuleA")
+local Spider        = require("Spider")
+local Var           = require("Var")
+local concatTbl     = table.concat
+local cosmic        = require("Cosmic"):singleton()
+local dbg           = require("Dbg"):dbg()
+local hook          = require("Hook")
+local i18n          = require("i18n")
+local remove        = table.remove
+local sort          = table.sort
+local q_load        = 0
+local s_same        = true
 
-local mpath_avail  = cosmic:value("LMOD_MPATH_AVAIL")
+local A             = ShowResultsA
 
 ------------------------------------------------------------------------
 -- a private ctor that is used to construct a singleton.
@@ -86,7 +86,6 @@ function M.singleton(self, safe)
    if (not s_hub) then
       s_hub = l_new(self, safe)
    end
-   dbg.print{"s_hub: ",tostring(s_hub), ", safe: ",s_hub.__safe,"\n"}
    dbg.fini("Hub:singleton")
    return s_hub
 end
@@ -112,10 +111,13 @@ function M.access(self, ...)
    local shellNm   = shell:name()
    local help      = (_G.help ~= dbg.quiet) and "-h" or nil
    local A         = ShowResultsA
+   local mrc       = MRC:singleton()
    local result, t
 
+   mrc:set_display_mode("all")
+
    local argA = pack(...)
-   if (optionTbl.location) then
+   if (optionTbl.location or optionTbl.terse) then
       local userName = argA[1]
       local mname    = mt:have(userName,"any") and MName:new("mt",userName)
                                                or  MName:new("load",userName)
@@ -142,7 +144,7 @@ function M.access(self, ...)
             local mList = concatTbl(mt:list("both","active"),":")
             frameStk:push(mname)
             loadModuleFile{file=fn,help=help, shell=shellNm, mList = mList,
-                           reportErr=true}
+                           reportErr=true, forbiddenT = mname:forbiddenT()}
             frameStk:pop()
             A[#A+1] = "\n"
          end
@@ -156,7 +158,6 @@ function M.access(self, ...)
 
 
    if (#a > 0) then
-      setWarningFlag()
       LmodWarning{msg="w_Failed_2_Find",quote_comma_list=concatTbl(a,"\", \""),
                              module_list=concatTbl(a," ")}
    end
@@ -218,7 +219,7 @@ local function l_registerUnloaded(fullName, fn)
 end
 
 function M.inheritModule(self)
-   dbg.start{"Hub:inherit()"}
+   dbg.start{"Hub:inheritModule()"}
    local shellNm    = _G.Shell:name()
    local frameStk   = FrameStk:singleton()
    local myFn       = frameStk:fn()
@@ -247,16 +248,27 @@ function M.inheritModule(self)
       local mt    = frameStk:mt()
       local mList = concatTbl(mt:list("both","active"),":")
       frameStk:push(mname)
-      loadModuleFile{file=mname:fn(),mList = mList, shell=shellNm, reportErr=true}
+
+      if (mode() == "show") then
+         A[#A+1] = "--> "
+         A[#A+1] = fnI
+         A[#A+1] = "\n\n"
+      end
+
+      loadModuleFile{file=mname:fn(),mList = mList, shell=shellNm, reportErr=true,
+                     forbiddenT = mname:forbiddenT()}
       frameStk:pop()
    end
 
+   if (mode() == "show") then
+      A[#A+1] = "\n"
+   end
    if (mode() == "load") then
       local mt = frameStk:mt()
       mt:pushInheritFn(sn, mname)
    end
 
-   dbg.fini("Hub:inherit")
+   dbg.fini("Hub:inheritModule")
 end
 
 local s_stk = {}
@@ -292,12 +304,13 @@ function M.load(self, mA)
 
    local disable_same_name_autoswap = cosmic:value("LMOD_DISABLE_SAME_NAME_AUTOSWAP")
 
-   local optionTbl = optionTbl()
-   local tracing   = cosmic:value("LMOD_TRACING")
-   local frameStk  = FrameStk:singleton()
-   local shell     = _G.Shell
-   local shellNm   = shell and shell:name() or "bash"
-   local a         = true
+   local dsConflicts = cosmic:value("LMOD_DOWNSTREAM_CONFLICTS")
+   local optionTbl   = optionTbl()
+   local tracing     = cosmic:value("LMOD_TRACING")
+   local frameStk    = FrameStk:singleton()
+   local shell       = _G.Shell
+   local shellNm     = shell and shell:name() or "bash"
+   local a           = true
    local mt
 
 
@@ -305,11 +318,11 @@ function M.load(self, mA)
       repeat
          local mname      = mA[i]
          local userName   = mname:userName()
-
-         dbg.print{"Hub:load i: ",i,", userName: ",userName,"\n",}
-
          mt               = frameStk:mt()
+
          local sn         = mname:sn()
+         dbg.print{"Hub:load i: ",i,", userName: ",userName,", sn: ",sn,"\n",}
+
          if ((sn == nil) and ((i > 1) or (frameStk:stackDepth() > 0))) then
             dbg.print{"Pushing ",mname:userName()," on moduleQ\n"}
             dbg.print{"i: ",i,", stackDepth: ", frameStk:stackDepth(),"\n"}
@@ -336,7 +349,7 @@ function M.load(self, mA)
                         ")" }
          end
 
-         dbg.print{"Hub:load i: ",i," sn: ",sn," fn: ",fn,"\n"}
+         dbg.print{"Hub:load i: ",i,", sn: ",sn,", fullName: ",fullName,", fn: ",fn,"\n"}
 
          if (mt:have(sn,"active")) then
             local version    = mname:version()
@@ -364,7 +377,7 @@ function M.load(self, mA)
          elseif (not fn and not frameStk:empty()) then
             local msg = "Executing this command requires loading \"" .. userName .. "\" which failed"..
                " while processing the following module(s):\n\n"
-            msg = buildMsg(TermWidth(), msg)
+            msg = buildMsg(TermWidth(), {n = 1,msg})
             if (haveWarnings()) then
                stackTraceBackA[#stackTraceBackA+1] = moduleStackTraceBack(msg)
             end
@@ -374,7 +387,19 @@ function M.load(self, mA)
             frameStk:push(mname)
             mt = frameStk:mt()
             mt:add(mname,"pending")
-            local status = loadModuleFile{file = fn, shell = shellNm, mList = mList, reportErr = true}
+            dbg.print{"dsConflicts: ",dsConflicts,"\n"}
+            if (dsConflicts == "yes") then
+               local snUpstream = mt:haveDSConflict(mname)
+               if (snUpstream) then
+                  local fullNameUpstream = mt:fullName(snUpstream)
+                  LmodError{msg="e_Conflict_Downstream", fullNameUpstream = fullNameUpstream,
+                            userName=userName}
+               end
+            end
+
+
+            local status = loadModuleFile{file = fn, shell = shellNm, mList = mList,
+                                          reportErr = true, forbiddenT = mname:forbiddenT()}
             mt = frameStk:mt()
 
             -- A modulefile could the same named module over top of the current modulefile
@@ -391,7 +416,7 @@ function M.load(self, mA)
          end
          mt = frameStk:mt()
          if (not mt:have(sn,"active")) then
-            dbg.print{"failed to load ",mname:show(),"\n"}
+            dbg.print{"failed to load ",mname:show(),", sn: ", sn,"\n"}
             mcp:missing_module(userName, mname:show())
             a = false
          end
@@ -460,6 +485,27 @@ function M.load(self, mA)
    return a
 end
 
+local function l_missingFn_action(actionA)
+   dbg.start{"l_missingFn_action(actionA)"}
+   local frameStk = FrameStk:singleton()
+   local sn       = frameStk:sn()
+   local msg      = ""
+   local status   = true
+   if (next(actionA) == nil) then
+      dbg.fini("l_missingFn_action with empty actionA")
+      return status
+   end
+   local whole  = concatTbl(actionA,"\n")
+   dbg.print{"whole: ",whole,"\n"}
+   status, msg = sandbox_run(whole)
+   if (not status) then
+      LmodError{msg="e_Missing_Action", name = sn, message = msg}
+   end
+   dbg.fini("l_missingFn_action")
+   return status
+end
+
+
 
 --------------------------------------------------------------------------
 -- Unload modulefile(s) via the module names.
@@ -511,9 +557,15 @@ function M.unload(self,mA)
          end
 
          mt:setStatus(sn,"pending")
-         local mList  = concatTbl(mt:list("both","active"),":")
-	 local status = loadModuleFile{file=fn, mList=mList, shell=shellNm, reportErr=false}
-         dbg.print{"status from loadModulefile: ",status,"\n"}
+         local status
+         if (not isFile(fn)) then
+            status = l_missingFn_action(mt:get_actionA(sn))
+         else
+            local mList  = concatTbl(mt:list("both","active"),":")
+            status = loadModuleFile{file=fn, mList=mList, shell=shellNm, reportErr=false,
+                                    forbiddenT = {}}
+            dbg.print{"status from loadModulefile: ",status,"\n"}
+         end
          if (status) then
             mt = frameStk:mt()
             mt:remove(sn)
@@ -687,7 +739,7 @@ function M.refresh()
          frameStk:push(MName:new("mt",userName))
          dbg.print{"loading: ",sn,", userName: ",myModuleUsrName(),", fn: ", fn,"\n"}
          loadModuleFile{file = fn, shell = shellNm, mList = mList,
-                        reportErr=true}
+                        reportErr=true, forbiddenT = {}}
          frameStk:pop()
       end
    end
@@ -719,9 +771,9 @@ function M.dependencyCk()
       local fn       = mt:fn(sn)
       if (isFile(fn)) then
          frameStk:push(MName:new("mt",sn))
-         dbg.print{"loading: ",sn," fn: ", fn,"\n"}
+         dbg.print{"DepCk loading: ",sn," fn: ", fn,"\n"}
          loadModuleFile{file = fn, shell = shellNm, mList = mList,
-                        reportErr=true}
+                        reportErr=true, forbiddenT = {}}
          frameStk:pop()
       end
    end
@@ -801,6 +853,7 @@ function M.safeToUpdate()
 end
 
 local function l_availEntry(defaultOnly, label, searchA, defaultT, entry)
+   local mpath_avail = cosmic:value("LMOD_MPATH_AVAIL")
    if (defaultOnly) then
       local fn    = entry.fn
       if (not defaultT[fn]) then
@@ -812,6 +865,7 @@ local function l_availEntry(defaultOnly, label, searchA, defaultT, entry)
    local fullName = entry.fullName
    local sn       = entry.sn
    local fn       = entry.fn
+   local provideA = entry.provides
    if (searchA.n > 0) then
       found = false
       for i = 1, searchA.n do
@@ -827,7 +881,7 @@ local function l_availEntry(defaultOnly, label, searchA, defaultT, entry)
       end
    end
    if (found) then
-      return sn, fullName, fn
+      return sn, fullName, fn, provideA
    end
    return nil, nil
 end
@@ -918,7 +972,10 @@ function M.overview(self,argA)
    local mt          = FrameStk:singleton():mt()
    local mpathA      = mt:modulePathA()
    local availStyle  = optionTbl.availStyle
-   
+   local mrc         = MRC:singleton()
+
+   mrc:set_display_mode("avail")
+
    local numDirs = 0
    for i = 1,#mpathA do
       local mpath = mpathA[i]
@@ -936,7 +993,8 @@ function M.overview(self,argA)
       return a
    end
 
-   local mrc         = MRC:singleton()
+   
+
    local use_cache   = false
    local moduleA     = ModuleA:singleton{spider_cache=use_cache}
    local availA      = moduleA:build_availA()
@@ -946,9 +1004,8 @@ function M.overview(self,argA)
    local searchA     = argA
    local showSN      = true
    local defaultOnly = false
-   local alias2modT  = mrc:getAlias2ModT(mpathA)
    local banner      = Banner:singleton()
-   
+
    if (not optionTbl.regexp and argA and next(argA) ~= nil) then
       searchA = {}
       for i = 1, argA.n do
@@ -964,7 +1021,9 @@ function M.overview(self,argA)
    end
 
    availA = regroup_avail_blocks(availStyle, availA)
-   self:terse_avail(mpathA, availA, alias2modT, searchA, showSN, defaultOnly, defaultT, aa)
+   local showModuleExt = false
+
+   self:terse_avail(mpathA, availA, searchA, showSN, defaultOnly, defaultT, showModuleExt, aa)
 
    local label    = ""
    local a        = {}
@@ -987,7 +1046,7 @@ function M.overview(self,argA)
 
    ---------------------------------------------------------------
    -- This local function stores the current sn and count into the
-   -- b array and if the current entry is true then define the next 
+   -- b array and if the current entry is true then define the next
    -- sn to be entry (minus the trailing slash) and zero count.
    local function register_sn_count_in_b(entry)
       if (sn and count > 0) then
@@ -1004,9 +1063,9 @@ function M.overview(self,argA)
    end
 
    for i = 1,#aa do
-      local entry = aa[i]:sub(1,-2) --> strip trailing newline
-      repeat 
-         dbg.print{"RTM: entry: ",entry,"\n"}
+      local entry = aa[i]:gsub("%s+$",""):gsub(" *<.*","") --> strip trailing newline and any decorations
+      repeat
+         dbg.print{"RTM: entry: \"",entry,"\"\n"}
          if (entry:find("%(@")) then
             break
          end
@@ -1045,13 +1104,61 @@ function M.overview(self,argA)
    return a
 end
 
-function M.terse_avail(self, mpathA, availA, alias2modT, searchA, showSN, defaultOnly, defaultT, a)
+function M.buildExtA(self, searchA, mpathA, providedByT, extA)
+    dbg.start{"Hub:buildExtA()"}
+
+    local mpathT = {}
+    for i = 1, #mpathA do
+       mpathT[mpathA[i]] = true
+    end
+
+    dbg.printT("mpathT",mpathT)
+    dbg.printT("providedByT",providedByT)
+
+    for k,v in pairsByKeys(providedByT) do
+       local found = false
+       if (searchA.n > 0) then
+          for i = 1, searchA.n do
+             for kk,vv in pairs(v) do
+                local s        = searchA[i]
+                for i = 1,#vv do
+                   local vvv = vv[i]
+                   if (kk:find(s) and mpathT[vvv.mpath] and (not vvv.hidden)) then
+                      found = true
+                      break
+                   end
+                end
+                if (found) then break end
+             end
+             if (found) then break end
+          end
+       else
+          for kk,vv in pairs(v) do
+             for i = 1,#vv do
+                local vvv = vv[i]
+                if (mpathT[vvv.mpath] and (not vvv.hidden)) then
+                   found = true
+                   break
+                end
+             end
+             if (found) then break end
+          end
+       end
+       if (found) then
+          extA[#extA + 1] = {"    " .. colorize("blue",k),"(E)"}
+       end
+    end
+
+    dbg.fini("Hub:buildExtA()")
+end
+
+function M.terse_avail(self, mpathA, availA, searchA, showSN, defaultOnly, defaultT, showModuleExt, a)
    dbg.start{"Hub:terse_avail()"}
    local mrc         = MRC:singleton()
    local optionTbl   = optionTbl()
 
    if (searchA.n > 0) then
-      for k, v in pairsByKeys(alias2modT) do
+      for k, v in mrc:pairsForMRC_aliases(mpathA) do
          local fullName = mrc:resolve(mpathA, v)
          for i = 1, searchA.n do
             local s = searchA[i]
@@ -1061,34 +1168,44 @@ function M.terse_avail(self, mpathA, availA, alias2modT, searchA, showSN, defaul
          end
       end
    else
-      for k, v in pairsByKeys(alias2modT) do
+      for k, v in mrc:pairsForMRC_aliases(mpathA) do
          local fullName = mrc:resolve(mpathA, v)
          a[#a+1] = k.."(@" .. fullName ..")\n"
       end
    end
 
 
+   dbg.printT("availA",availA)
+
    for j = 1,#availA do
       local A      = availA[j].A
-      local label  = availA[j].mpath
+      local mpath  = availA[j].mpath
+      local label  = mpath
       local aa     = {}
       local prtSnT = {}  -- Mark if we have printed the sn?
-      
+
       for i = 1,#A do
-         local sn, fullName, fn = l_availEntry(defaultOnly, label, searchA, defaultT, A[i])
+         local sn, fullName, fn, provideA = l_availEntry(defaultOnly, label, searchA, defaultT, A[i])
+         local entry  = A[i]
          if (sn) then
             if (not prtSnT[sn] and sn ~= fullName and showSN) then
                prtSnT[sn] = true
                aa[#aa+1]  = sn .. "/\n"
             end
-            local aliasA = mrc:getFull2AliasesT(mpathA, fullName)
+            local aliasA = mrc:search_mapT("full2aliasesT", mpath, fullName)
             if (aliasA) then
                for i = 1,#aliasA do
                   local fullName = mrc:resolve(mpathA, aliasA[i])
                   aa[#aa+1]  = aliasA[i] .. "(@".. fullName ..")\n"
                end
             end
-            aa[#aa+1]     = fullName .. "\n"
+            aa[#aa+1]  = decorateModule(fullName, entry, entry.forbiddenT) .. "\n"
+
+            if (showModuleExt and provideA and next(provideA) ~= nil ) then
+               for k = 1,#provideA do
+                  aa[#aa+1] = "#    " .. provideA[k] .. "\n"
+               end
+            end
          end
       end
       if (next(aa) ~= nil) then
@@ -1098,7 +1215,6 @@ function M.terse_avail(self, mpathA, availA, alias2modT, searchA, showSN, defaul
          end
       end
    end
-
    dbg.fini("Hub:terse_avail")
    return a
 end
@@ -1111,6 +1227,9 @@ function M.avail(self, argA)
    local mt          = FrameStk:singleton():mt()
    local mpathA      = mt:modulePathA()
    local availStyle  = optionTbl.availStyle
+   local mrc         = MRC:singleton()
+
+   mrc:set_display_mode("avail")
 
    local numDirs = 0
    for i = 1,#mpathA do
@@ -1133,16 +1252,18 @@ function M.avail(self, argA)
    local use_cache     = (not optionTbl.terse) or (cosmic:value("LMOD_CACHED_LOADS") ~= "no")
    local moduleA       = ModuleA:singleton{spider_cache=use_cache}
    local isNVV         = moduleA:isNVV()
-   local mrc           = MRC:singleton()
    local availA        = moduleA:build_availA()
    local twidth        = TermWidth()
    local cwidth        = optionTbl.rt and LMOD_COLUMN_TABLE_WIDTH or twidth
    local defaultT      = moduleA:defaultT()
    local searchA       = argA
    local defaultOnly   = optionTbl.defaultOnly
-   local alias2modT    = mrc:getAlias2ModT(mpathA)
    local showSN        = not defaultOnly
 
+
+
+   dbg.printT("availA",availA)
+
    if (showSN) then
       showSN = argA.n == 0
    end
@@ -1166,10 +1287,11 @@ function M.avail(self, argA)
       searchA.n = argA.n
    end
 
-   if (optionTbl.terse) then
+   if (optionTbl.terse or optionTbl.terseShowExtensions) then
       --------------------------------------------------
       -- Terse output
-      self:terse_avail(mpathA, availA, alias2modT, searchA, showSN, defaultOnly, defaultT, a)
+      self:terse_avail(mpathA, availA, searchA, showSN, defaultOnly, 
+                       defaultT, optionTbl.terseShowExtensions, a)
 
       dbg.fini("Hub:avail")
       return a
@@ -1184,52 +1306,51 @@ function M.avail(self, argA)
    local na       = "N/A"
    local pna      = "("..na..")"
 
-   if (next(alias2modT) ~= nil) then
-      local fndAlias = false
-      local b        = {}
-      local bb       = {}
-      if (searchA.n > 0) then
-         for k, v in pairsByKeys(alias2modT) do
-            local fullName = mrc:resolve(mpathA,v)
-            for i = 1, searchA.n do
-               local s = searchA[i]
-               if (fullName:find(s)) then
-                  local mname    = MName:new("load",k)
-                  fullName = mname:fullName() or pna
-                  if (fullName == pna) then
-                     legendT[na] = i18n("m_Global_Alias_na")
-                  end
-                  fndAlias = true
-                  b[#b+1]  = { "   " .. k, "->", fullName}
-                  break
+   local fndAlias = false
+   local b        = {}
+   local bb       = {}
+   if (searchA.n > 0) then
+      for k, v in mrc:pairsForMRC_aliases(mpathA) do
+         local fullName = mrc:resolve(mpathA,v)
+         for i = 1, searchA.n do
+            local s = searchA[i]
+            if (fullName:find(s)) then
+               local mname    = MName:new("load",k)
+               fullName = mname:fullName() or pna
+               if (fullName == pna) then
+                  legendT[na] = i18n("m_Global_Alias_na")
                end
+               fndAlias = true
+               b[#b+1]  = { "   " .. k, "->", fullName}
+               break
             end
          end
-      else
-         for k, v in pairsByKeys(alias2modT) do
-            local mname    = MName:new("load",k)
-            local fullName = mname:fullName() or pna
-            if (fullName == pna) then
-               legendT[na] = i18n("m_Global_Alias_na")
-            end
-            fndAlias = true
-            b[#b+1]  = { "   " .. k, "->", fullName}
+      end
+   else
+      for k, v in mrc:pairsForMRC_aliases(mpathA) do
+         local mname    = MName:new("load",k)
+         local fullName = mname:fullName() or pna
+         if (fullName == pna) then
+            legendT[na] = i18n("m_Global_Alias_na")
          end
+         fndAlias = true
+         b[#b+1]  = { "   " .. k, "->", fullName}
       end
-      if (fndAlias) then
-         local ct = ColumnTable:new{tbl=b, gap=1, len=length, width = cwidth}
-         a[#a+1]  = "\n"
-         a[#a+1] = banner:bannerStr("Global Aliases")
-         a[#a+1] = "\n"
-         a[#a+1]  = ct:build_tbl()
-         a[#a+1] = "\n"
-      end
+   end
+   if (fndAlias) then
+      local ct = ColumnTable:new{tbl=b, gap=1, len=length, width = cwidth}
+      a[#a+1]  = "\n"
+      a[#a+1] = banner:bannerStr("Global Aliases")
+      a[#a+1] = "\n"
+      a[#a+1]  = ct:build_tbl()
+      a[#a+1] = "\n"
    end
 
 
    for k = 1,#availA do
       local A = availA[k].A
-      local label = availA[k].mpath
+      local mpath = availA[k].mpath
+      local label = mpath
       if (next(A) ~= nil) then
          local b = {}
          for j = 1,#A do
@@ -1247,14 +1368,16 @@ function M.avail(self, argA)
                   entry.propT["status"] = {active = 1}
                end
                local c = {}
-               local resultA = colorizePropA("short", {sn=sn, fullName=fullName, fn=fn}, mrc, entry.propT, legendT)
+               local resultA = colorizePropA("short", mt,
+                                             {sn=sn, fullName=fullName, fn=fn, mpath = mpath },
+                                             mrc, entry.propT, legendT, entry.forbiddenT)
                c[#c+1] = '  '
                for i = 1,#resultA do
                   c[#c+1] = resultA[i]
                end
 
                local propStr = c[3] or ""
-               local verMapStr = mrc:getMod2VersionT(mpathA, fullName)
+               local verMapStr = mrc:search_mapT("mod2versionT", mpath, fullName)
                if (verMapStr) then
                   legendT["Aliases"] = i18n("aliasMsg",{})
                   if (dflt == Default) then
@@ -1295,58 +1418,17 @@ function M.avail(self, argA)
    local cache                  = Cache:singleton{buildCache=true}
    local spiderT,dbT,
          mpathMapT, providedByT = cache:build()
-   
+
    dbg.printT("providedByT", providedByT)
 
-
-
    if (extensions and providedByT and next(providedByT) ~= nil) then
-      local mpathT = {}
-      for i = 1, #mpathA do
-         mpathT[mpathA[i]] = true
-      end
+      local extA = {}
+      self:buildExtA(searchA, mpathA, providedByT, extA)
 
-      dbg.printT("mpathT",mpathT)
-      dbg.printT("providedByT",providedByT)
-
-      local b = {}
-      for k,v in pairsByKeys(providedByT) do
-         local found = false
-         if (searchA.n > 0) then
-            for i = 1, searchA.n do
-               for kk,vv in pairs(v) do
-                  local s        = searchA[i]
-                  for i = 1,#vv do
-                     local vvv = vv[i]
-                     if (kk:find(s) and mpathT[vvv.mpath] and (not vvv.hidden)) then
-                        found = true
-                        break
-                     end
-                  end
-                  if (found) then break end
-               end
-               if (found) then break end
-            end
-         else
-            for kk,vv in pairs(v) do
-               for i = 1,#vv do
-                  local vvv = vv[i]
-                  if (mpathT[vvv.mpath] and (not vvv.hidden)) then
-                     found = true
-                     break
-                  end
-               end
-               if (found) then break end
-            end
-         end
-         if (found) then
-            b[#b + 1] = {"    " .. colorize("blue",k),"(E)"}
-         end
-      end
-      if (next(b) ~= nil) then
+      if (next(extA) ~= nil) then
          legendT['E'] = i18n("Extension")
-         numFound = numFound + #b
-         local ct = ColumnTable:new{tbl=b, gap=1, len = length, width = cwidth}
+         numFound = numFound + #extA
+         local ct = ColumnTable:new{tbl=extA, gap=1, len = length, width = cwidth}
          a[#a+1] = "\n"
          a[#a+1] = banner:bannerStr(i18n("m_Extensions_head"))
          a[#a+1] = "\n"
@@ -1385,7 +1467,4 @@ function M.avail(self, argA)
    return a
 end
 
-
-
-
 return M
