@@ -49,8 +49,6 @@ local dbg         = require("Dbg"):dbg()
 local sort        = table.sort
 local s_findT     = false
 
-local exact_match = cosmic:value("LMOD_EXACT_MATCH")
-
 function M.className(self)
    return self.my_name
 end
@@ -63,10 +61,14 @@ local function l_lessthan_equal(a,b)
    return a <= b
 end
 
+local s_rangeFuncT = { ["<="] = {func = l_lessthan_equal, name = "<="},
+                       ["<"]  = {func = l_lessthan,       name = "<"},
+                     }
 
 
 function M.new(self, sType, name, action, is, ie)
-   --dbg.start{"Mname:new(",sType,")"}
+   dbg.start{"Mname:new(",sType,", name: ",name,", action: ", action,")"}
+   local exact_match = cosmic:value("LMOD_EXACT_MATCH")
 
    if (not s_findT) then
       local Match   = require("MN_Match")
@@ -79,6 +81,7 @@ function M.new(self, sType, name, action, is, ie)
          latest  = Latest,
          between = Between,
          atleast = Between,
+         atmost  = Between,
       }
    end
 
@@ -90,33 +93,39 @@ function M.new(self, sType, name, action, is, ie)
 
    local o = s_findT[action]:create()
 
-   is             = is or false
-   ie             = ie or false
-   o.__sn         = false
-   o.__version    = false
-   o.__fn         = false
-   o.__versionStr = false
-   o.__dependsOn  = false
-   o.__ref_count  = nil
-   o.__sType      = sType
-   o.__wV         = false
-   o.__waterMark  = "MName"
-   o.__action     = action
-   o.__range_fnA  = { l_lessthan_equal, l_lessthan_equal }
-   o.__show_range = { is, ie}
+   is                  = is or false
+   ie                  = ie or false
+   o.__isOrig          = is
+   o.__ieOrig          = ie
+   o.__sn              = false
+   o.__version         = false
+   o.__fn              = false
+   o.__versionStr      = false
+   o.__dependsOn       = false
+   o.__moduleKindT     = nil
+   o.__ref_count       = nil
+   o.__depends_on_anyA = nil
+   o.__sType           = sType
+   o.__wV              = false
+   o.__waterMark       = "MName"
+   o.__action          = action
+   o.__range_fnA       = { s_rangeFuncT["<="], s_rangeFuncT["<="]}
+   o.__show_range      = { is, ie}
    if (is and (is:sub(1,1) == "<" or is:sub(-1) == "<")) then
-      o.__range_fnA[1]  = l_lessthan
+      o.__range_fnA[1]  = s_rangeFuncT["<"]
       is = is:gsub("<","")
+   elseif (is and (is:sub(1,1) == ">" or is:sub(-1) == ">")) then
+      o.__range_fnA[1]  = s_rangeFuncT["<"]
+      is = is:gsub(">","")
    end
    if (ie and (ie:sub(1,1) == "<" or ie:sub(-1) == "<")) then
-      o.__range_fnA[2]  = l_lessthan
+      o.__range_fnA[2]  = s_rangeFuncT["<"]
       ie = ie:gsub("<","")
    end
-   o.__is         = is 
-   o.__ie         = ie 
+   o.__is         = is
+   o.__ie         = ie
    o.__have_range = is or ie
    o.__range      = { o.__is and parseVersion(o.__is) or " ", o.__ie and parseVersion(o.__ie) or "~" }
-   o.__actionNm   = action
 
    if (sType == "entryT") then
       local t      = name
@@ -133,11 +142,23 @@ function M.new(self, sType, name, action, is, ie)
       o.__userName   = (name or ""):trim():gsub("/+$",""):gsub("%.lua$","")
    end
 
-   --dbg.fini("MName:new")
+   dbg.fini("MName:new")
    return o
 end
 
 
+
+--------------------------------------------------------------------------
+-- This l_overRide_sType() function is here to convert
+--    mcp:conflict(MName:buildA("mt",...))
+--    where the list of userNames might include between("A","1.0","2.0")
+--    Functions like between assume "load".  Where as conflict, prereq etc
+--    need "mt".
+
+local function l_overRide_sType(mname, sTypeIn)
+   mname.__sType = sTypeIn
+end
+
 --------------------------------------------------------------------------
 -- Return an array of MName objects
 -- @param self A MName object
@@ -152,6 +173,7 @@ function M.buildA(self,sType, ...)
       if (type(v) == "string" ) then
          a[#a + 1] = self:new(sType, v:trim())
       elseif (type(v) == "table") then
+         l_overRide_sType(v,sType)
          a[#a + 1] = v
       end
    end
@@ -159,7 +181,7 @@ function M.buildA(self,sType, ...)
 end
 
 local function l_lazyEval(self)
-   --dbg.start{"l_lazyEval(",self.__userName,")"}
+   dbg.start{"l_lazyEval(",self.__userName,")"}
 
    local sType   = self.__sType
    if (sType == "mt") then
@@ -168,14 +190,15 @@ local function l_lazyEval(self)
       local sn       = mt:lookup_w_userName(self.__userName)
       --dbg.print{"sn: ",sn,"\n"}
       if (sn) then
-         self.__sn         = sn
-         self.__fn         = mt:fn(sn)
-         self.__version    = mt:version(sn)
-         self.__stackDepth = mt:stackDepth(sn)
-         self.__wV         = mt:wV(sn)
-         self.__ref_count  = mt:get_ref_count(sn)
+         self.__sn              = sn
+         self.__fn              = mt:fn(sn)
+         self.__version         = mt:version(sn)
+         self.__stackDepth      = mt:stackDepth(sn)
+         self.__wV              = mt:wV(sn)
+         self.__ref_count       = mt:get_ref_count(sn)
+         self.__depends_on_anyA = mt:get_depends_on_anyA(sn)
       end
-      --dbg.fini("l_lazyEval via mt")
+      dbg.fini("l_lazyEval via mt")
       return
    end
 
@@ -192,7 +215,7 @@ local function l_lazyEval(self)
          self.__wV       = t.wV
       end
 
-      --dbg.fini("l_lazyEval via inherit")
+      dbg.fini("l_lazyEval via inherit")
       return
    end
 
@@ -200,9 +223,17 @@ local function l_lazyEval(self)
    local mrc                   = MRC:singleton()
    local frameStk              = FrameStk:singleton()
    local mt                    = frameStk:mt()
+   local origUserName          = self:userName()
    local userName              = mrc:resolve(mt:modulePathA(), self:userName())
    local sn, versionStr, fileA = moduleA:search(userName)
-   --dbg.print{"l_lazyEval: userName: ",userName, ", sn: ",sn,", versionStr: ",versionStr,"\n"}
+   dbg.printT("fileA",fileA)
+   dbg.print{"l_lazyEval: orig: ",origUserName,", userName: ",userName, ", sn: ",sn,", versionStr: ",versionStr,"\n"}
+   mrc:applyWeights(sn, fileA)
+
+   if (origUserName ~= userName) then
+      self.__origUserName = origUserName
+   end
+   dbg.print{"l_lazyEval: self.__origUserName: ",self.__origUserName,"\n"}
 
    self.__userName   = userName
    self.__sn         = sn
@@ -210,7 +241,7 @@ local function l_lazyEval(self)
    self.__stackDepth = self.__stackDepth or frameStk:stackDepth()
 
    if (not sn) then
-      --dbg.fini("l_lazyEval via no sn")
+      dbg.fini("l_lazyEval via no sn")
       return
    end
 
@@ -219,28 +250,43 @@ local function l_lazyEval(self)
    local fn
    local wV
    local found
-   --dbg.printT("fileA",fileA)
-   --dbg.print{"#stepA: ",#stepA,"\n"}
-   --dbg.print{"userName: ",self.__userName,"\n"}
-   --dbg.print{"sn: ",self.__sn,"\n"}
+   local mpath
+   local moduleKindT
+   dbg.printT("fileA",fileA)
+   dbg.print{"#stepA: ",#stepA,"\n"}
+   dbg.print{"userName: ",self.__userName,"\n"}
+   dbg.print{"sn: ",self.__sn,"\n"}
 
 
    for i = 1, #stepA do
       local func = stepA[i]
-      found, fn, version, wV = func(self, fileA)
+      found, fn, version, wV, moduleKindT, mpath = func(self, fileA)
       if (found) then
-         self.__fn      = fn
-         self.__version = version
-         self.__wV      = wV
-         if (self.__actionNm == "latest" or self.__sn ~= self.__userName) then
+         self.__fn          = fn
+         self.__version     = version
+         self.__wV          = wV
+         self.__moduleKindT = moduleKindT
+         self.__mpath       = mpath
+         if (self.__action == "latest" or self.__sn ~= self.__userName) then
             self.__userName = build_fullName(self.__sn, version)
          end
          break
       end
    end
-   --dbg.print{"l_lazyEval: sn: ",self.__sn, ", version: ",self.__version, ", fn: ",self.__fn,", wV: ",self.__wV,", userName: ",self.__userName,"\n"}
-   --dbg.print{"fn: ",self.__fn,"\n"}
-   --dbg.fini("l_lazyEval")
+
+   ---------------------------------------------------------------
+   -- If found then check to see if this MName object is forbidden
+   
+   if (found) then
+      self.__forbiddenT = mrc:isForbidden{fullName=build_fullName(self.__sn, version),
+                                          sn = self.__sn, fn = self.__fn,
+                                          mpath = self.__mpath}
+   end
+
+   local tt = self.__moduleKindT or {}
+   dbg.print{"l_lazyEval: sn: ",self.__sn, ", version: ",self.__version, ", fn: ",self.__fn,", wV: ",self.__wV,", userName: ",self.__userName,"\n"}
+   dbg.print{"fn: ",self.__fn,", kind: ",tt.kind,"\n"}
+   dbg.fini("l_lazyEval")
 end
 
 
@@ -251,11 +297,35 @@ function M.valid(self)
    return self.__fn
 end
 
+function M.moduleKindT(self)
+   if (not self.__moduleKindT) then
+      return nil
+   end
+   local moduleKindT   = self.__moduleKindT
+   local hidden_loaded = moduleKindT.hidden_loaded or nil
+   local t = nil
+   if (moduleKindT.kind ~= "normal") then
+      t = {kind = moduleKindT.kind, hidden_loaded = hidden_loaded} 
+   end
+   return t
+end
+
+function M.isVisible(self)
+   if (not self.__moduleKindT) then
+      return true
+   end
+   local moduleKindT   = self.__moduleKindT
+   return moduleKindT.kind == "normal"
+end
 
 function M.userName(self)
    return self.__userName
 end
 
+function M.origUserName(self)
+   return self.__origUserName
+end
+
 function M.sn(self)
    if (not self.__sn) then
       --dbg.start{"Mname:sn()"}
@@ -304,6 +374,17 @@ function M.set_ref_count(self, count)
    self.__ref_count = count
 end
 
+function M.set_depends_on_anyA(self, depends_on_anyA)
+   self.__depends_on_anyA = depends_on_anyA
+end
+
+function M.get_depends_on_anyA(self, sn)
+   if (not self.__sn) then
+      l_lazyEval(self)
+   end
+   return self.__depends_on_anyA
+end
+
 function M.ref_count(self)
    if (not self.__sn) then
       l_lazyEval(self)
@@ -372,6 +453,53 @@ end
 -- the whole module tree is treated as NVV.
 
 
+local function l_find_exact_match(self, must_have_version, fileA)
+   --dbg.start{"MName l_find_exact_match(must_have_version:,",must_have_version,",fileA)"}
+   local versionStr  = self.__versionStr
+   local mrc         = MRC:singleton()
+   local fn          = false
+   local version     = false
+   local pV          = " "  -- this is less than the lowest possible weight
+   local wV          = false
+   local moduleKindT = {}
+   local found       = false
+   local mpath       = false
+   if (must_have_version and not versionStr) then
+      return found, fn, version
+   end
+
+   for j = 1, #fileA do
+      local blockA = fileA[j]
+      for i = 1, #blockA do
+         local entry   = blockA[i]
+         --dbg.print{"entry.fullName: ",entry.fullName,", entry.fn: ",entry.fn,"\n"}
+         if (entry.version == versionStr and entry.pV > pV ) then
+            local resultT = mrc:isVisible{fullName=entry.fullName,sn=entry.sn,fn=entry.fn, mpath=entry.mpath,
+                                          visibleT = {soft = true, hidden = true}}
+            --dbg.printT("resultT",resultT)
+            if (resultT.isVisible)  then
+               pV          = entry.pV
+               wV          = entry.wV
+               fn          = entry.fn
+               mpath       = entry.mpath
+               version     = entry.version or false
+               moduleKindT = resultT.moduleKindT
+               found       = true
+               self.__range = { pV, pV }
+               break
+            end
+         end
+      end
+      if (found) then break end
+   end
+ 
+   --dbg.print{"found: ",found,", fn: ",fn,", version: ", version,", wV: ",wV,
+   --         ", kind: ",moduleKindT.kind,"\n"}
+   --dbg.fini("MName l_find_exact_match")
+   return found, fn, version, wV, moduleKindT, mpath
+end
+   
+
 ------------------------------------------------------------------------
 -- M.find_exact_match() is more difficult because there are possibly
 -- more than one marked default:
@@ -379,105 +507,73 @@ end
 --    2) The system admins can have a modulerc file. (weighted by 's')
 --    3) The user can have a ~/.modulerc file.       (weighted by 'u')
 
-
 function M.find_exact_match(self, fileA)
    --dbg.start{"MName:find_exact_match(fileA)"}
-   local versionStr = self.__versionStr
-   local fn         = false
-   local version    = false
-   local pV         = " "  -- this is less than the lowest possible weight
-   local wV         = false
-   local found      = false
-   if (not versionStr) then
-      --dbg.print{"found: ",found,", fn: ",fn,", version: ", version,"\n"}
-      --dbg.fini("MName:find_exact_match")
-      return found, fn, version
-   end
-      
-   for i = 1, #fileA do
-      local a = fileA[i]
-      for j = 1, #a do
-         local entry = a[j]
-         if (entry.version == versionStr and entry.pV > pV ) then
-            pV      = entry.pV
-            wV      = entry.wV
-            fn      = entry.fn
-            version = entry.version or false
-            found   = true
-            self.__range = { pV, pV }
-            break
-         end
-      end
-   end
-
-   ---dbg.print{"found: ",found,", fn: ",fn,", version: ", version,"\n"}
+   local must_have_version = true
+   local found, fn, version, wV, moduleKindT, mpath  = l_find_exact_match(self, must_have_version, fileA)
    --dbg.fini("MName:find_exact_match")
-   return found, fn, version, wV
+   return found, fn, version, wV, moduleKindT, mpath
 end
+------------------------------------------------------------------------
+-- This routine is almost the same as M.find_exact_match
+-- But this routine is here to find moduleName w/o versions
+-- (namely meta modules).  But modules with version have a
+-- higher priority over meta modules.
 
 function M.find_exact_match_meta_module(self, fileA)
    --dbg.start{"MName:find_exact_match_meta_module(fileA)"}
-   local versionStr = self.__versionStr
-   local fn         = false
-   local version    = false
-   local pV         = " "  -- this is less than the lowest possible weight
-   local wV         = false
-   local found      = false
-   for i = 1, #fileA do
-      local a = fileA[i]
-      for j = 1, #a do
-         local entry = a[j]
-         if (entry.version == versionStr and entry.pV > pV ) then
-            pV      = entry.pV
-            wV      = entry.wV
-            fn      = entry.fn
-            version = entry.version or false
-            found   = true
-            self.__range = { pV, pV }
-            break
-         end
-      end
-   end
-
-   --dbg.print{"found: ",found,", fn: ",fn,", version: ", version,"\n"}
+   local must_have_version = false
+   local found, fn, version, wV, moduleKindT, mpath = l_find_exact_match(self, must_have_version, fileA)
    --dbg.fini("MName:find_exact_match_meta_module")
-   return found, fn, version, wV
+   return found, fn, version, wV, moduleKindT, mpath
 end
 
-
 local function l_find_highest_by_key(self, key, fileA)
-   --dbg.start{"MName:find_by_key(key:\"",key,"\",fileA)"}
-   local mrc     = MRC:singleton()
-   local a       = fileA[1] or {}
-   local weight  = " "  -- this is less than the lower possible weight.
-   local idx     = nil
-   local fn      = false
-   local found   = false
-   local version = false
-   local pV      = false
-   local wV      = false
+   --dbg.start{"MName: l_find_highest_by_key(key:\"",key,"\",fileA)"}
+   local mrc         = MRC:singleton()
+   local weight      = " "  -- this is less than the lower possible weight.
+   local idx         = nil
+   local fn          = false
+   local moduleKindT = {}
+   local found       = false
+   local version     = false
+   local pV          = false
+   local wV          = false
+   local mpath       = false
+   fileA             = fileA or {}
+   local blockA
 
-   for j = 1,#a do
-      local entry = a[j]
-      local v     = entry[key]
-      if (mrc:isVisible{fullName=entry.fullName,sn=entry.sn,fn=entry.fn} or isMarked(v)) then
+   for j = 1,#fileA do
+       blockA = fileA[j]
+
+      for i = 1,#blockA do
+         local entry   = blockA[i]
+         local v       = entry[key]
          if (v > weight) then
-            idx    = j
-            weight = v
-            pV     = entry.pV
-            wV     = entry.wV
+            local resultT = mrc:isVisible{fullName=entry.fullName,sn=entry.sn,fn=entry.fn, mpath=entry.mpath,
+                                       visibleT = {soft = true}}
+            if (isMarked(v) or resultT.isVisible) then
+               idx         = i
+               weight      = v
+               pV          = entry.pV
+               wV          = entry.wV
+               mpath       = entry.mpath
+               moduleKindT = resultT.moduleKindT
+            end
          end
       end
+      if (idx) then break end
    end
    if (idx) then
-      fn           = a[idx].fn
-      version      = a[idx].version or false
+      fn           = blockA[idx].fn
+      version      = blockA[idx].version or false
       found        = true
       self.__range = { pV, pV }
    end
-   --dbg.print{"found: ",found,", fn: ",fn,", version: ", version,", wV: ",wV,"\n"}
-   --dbg.fini("MName:find_by_key")
-   return found, fn, version, wV
+   --dbg.print{"found: ",found,", fn: ",fn,", version: ", version,", wV: ",wV,
+   --          ", kind: ",moduleKindT.kind,"\n"}
+   --dbg.fini("MName: l_find_highest_by_key")
+   return found, fn, version, wV, moduleKindT, mpath
 end
 
 ------------------------------------------------------------------------
@@ -499,32 +595,34 @@ function M.find_between(self, fileA)
            return x.pV < y.pV
            end)
 
-   local fn         = false
-   local version    = false
-   local lowerBound = self.__range[1]
-   local upperBound = self.__range[2]
-   local lowerFn    = self.__range_fnA[1]
-   local upperFn    = self.__range_fnA[2]
+   local mrc         = MRC:singleton()
+   local fn          = false
+   local version     = false
+   local lowerBound  = self.__range[1]
+   local upperBound  = self.__range[2]
+   local lowerFn     = self.__range_fnA[1].func
+   local upperFn     = self.__range_fnA[2].func
 
-   local pV         = lowerBound
-   local wV         = " "  -- this is less than the lower possible weight.
-
-   --dbg.print{"lower: ",pV,"\n"}
-   --dbg.print{"upper: ",upperBound,"\n"}
-   --dbg.print{"wV:    \"",wV,"\"\n\n"}
-
-   local idx        = nil
-   local found      = false
+   local pV          = lowerBound
+   local wV          = " "  -- this is less than the lower possible weight.
+   local mpath       = false
+   local kind        = nil
+   local idx         = nil
+   local found       = false
+   local moduleKindT = {}
    for j = 1,#a do
       local entry = a[j]
       local v     = entry.pV
-      --dbg.print{"pV: ",pV,", v: ",v,", upper: \"",upperBound,"\"\n"}
-      --dbg.print{"pV <= v: ",pV <= v, ", v <= upperBound: ",v <= upperBound,", entry.wV > wV: ",entry.wV > wV,"\n"}
-
       if (lowerFn(pV,v) and upperFn(v,upperBound) and entry.wV > wV) then
-         idx = j
-         pV  = v
-         wV  = entry.wV
+         local resultT = mrc:isVisible{fullName=entry.fullName,sn=entry.sn,fn=entry.fn, mpath=entry.mpath,
+                                       visibleT = {soft = true}}
+         if (isMarked(v) or resultT.isVisible ) then
+            idx         = j
+            pV          = v
+            wV          = entry.wV
+            mpath       = entry.mpath
+            moduleKindT = resultT.moduleKindT
+         end
       end
    end
    if (idx) then
@@ -536,11 +634,37 @@ function M.find_between(self, fileA)
       end
    end
    --dbg.fini("MName:find_between")
-   return found, fn, version, wV
+   return found, fn, version, wV, moduleKindT, mpath
 end
 
-function M.find_inherit_match(self,fileA)
-   local a = fileA[1] or {}
+local function l_rangeCk(self, version, result_if_found, result_if_not_found)
+   --dbg.start{"l_rangeCk(self, version: ",version,", result_if_found: ",result_if_found,", result_if_not_found: ",result_if_not_found,")"}
+   local have_range = false
+   local result     = result_if_not_found
+   if (not self.__have_range) then
+      --dbg.print{"no range\n"}
+      --dbg.fini("l_rangeCk")
+      return have_range, result
+   end
+
+   have_range       = true
+   local lowerBound = self.__range[1]
+   local upperBound = self.__range[2]
+   local lowerFn    = self.__range_fnA[1]
+   local upperFn    = self.__range_fnA[2]
+   local pV         = parseVersion(version)
+   --dbg.print{"lowerBound: ",lowerBound,"\n"}
+   --dbg.print{"upperBound: ",upperBound,"\n"}
+   --dbg.print{"pV:         ",pV,"\n"}
+   --dbg.print{"lowerFn: ",lowerFn.name,", lowerFn.func(lowerBound, pV): ",lowerFn.func(lowerBound, pV),"\n"}
+   --dbg.print{"upperFn: ",upperFn.name,", upperFn.func(pV, upperBound): ",upperFn.func(pV, upperBound),"\n"}
+
+   if (lowerFn.func(lowerBound, pV) and upperFn.func(pV, upperBound)) then
+      result = result_if_found
+   end
+
+   --dbg.fini("l_rangeCk")
+   return have_range, result
 end
 
 function M.isloaded(self)
@@ -550,11 +674,15 @@ function M.isloaded(self)
    local sn        = self:sn()
    local status    = mt:status(sn)
    local sn_status = ((status == "active") or (status == "pending"))
-   if (sn_status and self.__have_range) then
-      local pV = parseVersion(mt:version(sn))
-      if ((self.__range[1] <= pV) and (pV <= self.__range[2])) then
-         return sn_status
-      end
+   if (not sn_status) then
+      --dbg.fini("MName:isloaded")
+      return sn_status
+   end
+
+   local have_range, result = l_rangeCk(self, mt:version(sn), sn_status, false)
+   if (have_range) then
+      --dbg.fini("MName:isloaded")
+      return result
    end
 
    local userName  = self:userName()
@@ -585,7 +713,7 @@ function M.isPending(self)
 end
 
 function M.defaultKind(self)
-   local kindT = { 
+   local kindT = {
       ["^"] = "marked",
       s     = "system",
       u     = "user",
@@ -613,13 +741,11 @@ function M.prereq(self)
       return userName
    end
 
-   if (self.__have_range) then
-      local pV = parseVersion(mt:version(sn))
-      if ((self.__range[1] <= pV) and (pV <= self.__range[2])) then
-         return false
-      end
+   local have_range, result = l_rangeCk(self, mt:version(sn), false, userName)
+   if (have_range) then
+      return result
    end
-      
+
    if (userName == sn or userName == fullName) then
       -- The userName matched the either the sn or fullName
       -- stored in the MT
@@ -635,6 +761,60 @@ function M.prereq(self)
    return userName
 end
 
+function M.conflictCk(self, mt)
+   dbg.start{"MName:conflictCk(mt)"}
+   local userName = false
+   local sn       = self:sn()
+   if (not (sn and mt:have(sn,"active"))) then
+      dbg.print{"1) userName: ",userName,"\n"}
+      dbg.fini("MName:conflictCk")
+      return userName
+   end
+
+   local have_range, result = l_rangeCk(self, mt:version(sn), mt:fullName(sn), false)
+   if (have_range) then
+      dbg.fini("MName:conflictCk")
+      return result
+   end
+
+   local self_userName = self:userName()
+   --dbg.print{"self_userName: ",self_userName,", sn: ",sn,", userName: ",userName,"\n"}
+   if (self_userName == sn or extractVersion(self_userName, sn) == mt:version(sn)) then
+      userName = self_userName
+   end
+   --dbg.print{"3) userName: ",userName,"\n"}
+   dbg.fini("MName:conflictCk")
+   return userName
+end
+
+function M.downstreamConflictCk(self, mnameIn)
+   local snIn = mnameIn:sn()
+   dbg.start{"MName:downstreamConflictCk(snIn:", snIn,")"}
+
+   dbg.print{"self:userName(): ", self:userName(),"\n"}
+
+   local have_range, result = l_rangeCk(self, mnameIn:version(), mnameIn:userName(), false)
+   if (have_range) then
+      dbg.print{"2 result: ",result,"\n"}
+      dbg.fini( "MName:downstreamConflictCk")
+      return result
+   end
+
+   result = false
+   --dbg.print{"self: ",self,"\n"}
+   --dbg.print{"mnameIn: ",mnameIn,"\n"}
+   if (self:userName() == snIn or extractVersion(self:userName(), snIn) == mnameIn:version()) then
+      result = snIn
+   end
+
+   dbg.fini( "MName:downstreamConflictCk")
+   return result
+end
+
+function M.forbiddenT(self)
+   return self.__forbiddenT 
+end
+
 function M.set_depends_on_flag(self, value)
    if (type(value) == "number") then
       self.__dependsOn = value > 0
@@ -658,6 +838,11 @@ function M.reset(self)
    self.__stackDepth = nil
 end
 
+function M.actionName(self)
+   return self.__action
+end
+
+
 --------------------------------------------------------------------------
 -- Return the string of the user name of the module.
 -- @param self A MName object
@@ -665,4 +850,16 @@ function M.show(self)
    return '"' .. self:userName() .. '"'
 end
 
+function M.print(self)
+   local t = { sType    = self.__sType,
+               userName = self.__userName,
+               action   = self.__action,
+               is       = self.__isOrig,
+               ie       = self.__ieOrig,
+   }
+   return t
+end
+
+
+
 return M
