@@ -87,13 +87,14 @@ local function l_new(self, s, restoreFn)
    dbg.start{"MT l_new(s,restoreFn:",restoreFn,")"}
    local o         = {}
 
-   o.c_rebuildTime   = false
-   o.c_shortTime     = false
-   o.mT              = {}
-   o.MTversion       = l_mt_version()
-   o.family          = {}
-   o.mpathA          = {}
-   o.depthT          = {}
+   o.c_rebuildTime = false
+   o.c_shortTime   = false
+   o.mT            = {}
+   o.MTversion     = l_mt_version()
+   o.family        = {}
+   o.mpathA        = {}
+   o.depthT        = {}
+   o.__conflictT   = {}
    o.__stickyA     = {}
    o.__loadT       = {}
    o.__changeMPATH = false
@@ -151,6 +152,25 @@ local function l_new(self, s, restoreFn)
       for k,v in pairs(_ModuleTable_) do
          o[k] = v
       end
+      ------------------------------------------------------------------
+      -- Convert the list of MName constructors in to an array of mnames
+      -- to hold the conflicts.
+      if (o.conflictT and  next(o.conflictT) ~= nil) then
+         local MName = require("MName")
+         local cT = {}
+         local tt = o.conflictT
+
+         for sn, vv in pairs(tt) do
+            local a = {}
+            for i = 1,#vv do
+               local t = tt[sn][i]
+               a[i] = MName:new(t.sType, t.userName, t.action, t.is, t.ie)
+            end
+            cT[sn] = a
+         end
+         o.__conflictT = cT
+      end
+      o.conflictT   = nil
    end
 
    -- remove any mcmdT connected to a mT entry
@@ -188,7 +208,9 @@ function M.singleton(self, t)
    if (not s_mt) then
       dbg.start{"MT:singleton()"}
       s_mt        = l_new(self, getMT())
-      dbg.printT("s_mt",s_mt)
+      if (dbg.active()) then
+         dbg.print{"s_mt: ",s_mt:serializeTbl("pretty") }
+      end
       dbg.fini("MT:singleton")
    end
    return s_mt
@@ -256,15 +278,19 @@ function M.add(self, mname, status, loadOrder)
       ref_count = 0
    end
    mT[sn] = {
-      fullName   = mname:fullName(),
-      fn         = mname:fn(),
-      userName   = mname:userName(),
-      stackDepth = mname:stackDepth(),
-      ref_count  = ref_count,
-      status     = status,
-      loadOrder  = loadOrder,
-      propT      = {},
-      wV         = mname:wV() or false,
+      fullName        = mname:fullName(),
+      fn              = mname:fn(),
+      userName        = mname:userName(),
+      stackDepth      = mname:stackDepth(),
+      origUserName    = mname:origUserName(),
+      moduleKindT     = mname:moduleKindT{},
+      forbiddenT      = mname:forbiddenT{},
+      ref_count       = ref_count,
+      depends_on_anyA = mname:get_depends_on_anyA(),
+      status          = status,
+      loadOrder       = loadOrder,
+      propT           = {},
+      wV              = mname:wV() or false,
    }
    if (status ~= "inactive" and old_status ~= "inactive") then
       self:safely_incr_ref_count(mname)
@@ -277,8 +303,12 @@ end
 -- collection is not found.
 function M.reportContents(self, t)
    dbg.start{"mt:reportContents(",t.fn,")"}
-   local f = io.open(t.fn,"r")
    local a       = {}
+   if (not t.fn) then
+      dbg.fini("mt:reportContents")
+      return a
+   end
+   local f = io.open(t.fn,"r")
    if (not f) then
       dbg.fini("mt:reportContents")
       return a
@@ -314,6 +344,24 @@ function M.reset_MPATH_change_flag(self)
 end
 
 
+function M.add_actionA(self, sn, action, value)
+   local entry = self.mT[sn]
+   if (entry ~= nil) then
+      local a = entry.actionA or {}
+      a[#a+1]    = action .. "(\"MODULEPATH\",\"" .. value .."\")"
+      entry.actionA = a
+   end
+end
+
+function M.get_actionA(self, sn)
+   local entry   = self.mT[sn]
+   local actionA = {}
+   if (entry ~= nil) then
+      actionA = entry.actionA or {}
+   end
+   return actionA
+end
+
 function M.add_sh2mf_cmds(self, sn, script, mcmdA)
    local entry = self.mT[sn]
    if (entry ~= nil) then
@@ -364,9 +412,20 @@ function M.status(self, sn)
    return nil
 end
 
-function M.exists(self, sn)
+function M.exists(self, sn, fullName)
    local entry = self.mT[sn]
-   return (entry ~= nil)
+   if (entry == nil) then
+      return false
+   end
+   return (fullName == nil) or (entry.status ~= "inactive"  and entry.fullName == fullName)
+end
+
+function M.moduleKindT(self, sn)
+   local entry = self.mT[sn]
+   if (entry ~= nil) then
+      return entry.moduleKindT
+   end
+   return nil
 end
 
 --------------------------------------------------------------------------
@@ -396,6 +455,8 @@ end
 
 function M.serializeTbl(self, state)
    local make_pretty = (state == "pretty")
+
+
    local mt     = deepcopy(self)
    local rTest  = optionTbl().rt
    if (rTest) then
@@ -404,6 +465,21 @@ function M.serializeTbl(self, state)
    end
    l_setLoadOrder(mt)
 
+   if (next(self.__conflictT) ~= nil) then
+      local cT = mt.__conflictT
+      local tt = {}
+
+      for sn,vv in pairs(cT) do
+         local a = {}
+         for i=1,#vv do
+            local mname = vv[i]
+            a[i] = mname:print()
+         end
+         tt[sn] = a
+      end
+      mt.conflictT = tt
+   end
+         
    if (make_pretty)  then
       local mT = mt.mT
       for sn, v in pairs(mT) do
@@ -474,6 +550,8 @@ function M.list(self, kind, status)
    local a    = {}
    local b    = {}
 
+   dbg.print{"MT:list(kind: ",kind,", status:",status,")\n"}
+
    if (kind == "short" or kind == "sn") then
       for k, v in pairs(mT) do
          if ((status == "any" or status == v.status) and
@@ -487,7 +565,12 @@ function M.list(self, kind, status)
              (v.status ~= "pending")) then
             local obj = { sn = k, fullName = v.fullName, userName = v.userName,
                           name = v[kind], fn = v.fn, loadOrder = v.loadOrder,
-                          stackDepth = v.stackDepth, ref_count = v.ref_count}
+                          stackDepth = v.stackDepth, ref_count = v.ref_count,
+                          depends_on_anyA = v.depends_on_anyA, displayName = v.fullName,
+                          origUserName = v.origUserName or false,
+                          moduleKindT = v.moduleKindT or {},
+                          forbiddenT  = v.forbiddenT or {},
+                        }
             a, b = l_build_AB(a, b, v.loadOrder, v[kind], obj )
          end
       end
@@ -496,9 +579,14 @@ function M.list(self, kind, status)
          if ((status == "any" or status == v.status) and
              (v.status ~= "pending") and
              (v.stackDepth == 0)) then
-            local obj = { sn = k, fullName = v.fullName, userName = v.userName,
+             local obj = { sn = k, fullName = v.fullName, userName = v.userName,
                           name = v.fullName, fn = v.fn, loadOrder = v.loadOrder,
-                          stackDepth = v.stackDepth, ref_count = v.ref_count}
+                          stackDepth = v.stackDepth, ref_count = v.ref_count,
+                          depends_on_anyA = v.depends_on_anyA, displayName = v.fullName, 
+                          origUserName = v.origUserName or false,
+                          moduleKindT = v.moduleKindT or {},
+                          forbiddenT  = v.forbiddenT or {},
+             }
             a, b = l_build_AB(a, b, v.loadOrder, v.fullName, obj )
          end
       end
@@ -619,23 +707,25 @@ function M.remove_property(self, sn, name, value)
 end
 
 --------------------------------------------------------------------------
--- List the property
+-- List the fullname with possible property
 -- @param self An MT object.
 -- @param idx The index in the list.
 -- @param sn The short name
 -- @param style How to colorize.
 -- @param legendT The legend table.
-function M.list_property(self, idx, sn, style, legendT)
-   dbg.start{"MT:list_property(\"",sn,"\", \"",style,"\")"}
-   local mT    = self.mT
-   local entry = mT[sn]
-   local mrc   = MRC:singleton()
+function M.list_w_property(self, idx, sn, style, legendT)
+   dbg.start{"MT:list_w_property(\"",sn,"\", \"",style,"\")"}
+   local mT          = self.mT
+   local entry       = mT[sn]
+   local mrc         = MRC:singleton()
 
    if (entry == nil) then
-      LmodError{msg="e_No_Mod_Entry", routine = "MT:list_property()", name = sn}
+      LmodError{msg="e_No_Mod_Entry", routine = "MT:list_w_property()", name = sn}
    end
 
-   local resultA = colorizePropA(style, {fullName=entry.fullName,sn=sn,fn=entry.fn}, mrc, entry.propT, legendT)
+   local resultA = colorizePropA(style, self, {fullName=entry.fullName, origUserName=entry.origUserName, sn=sn, fn=entry.fn},
+                                 mrc, entry.propT, legendT, entry.forbiddenT)
+   dbg.print{"resultA: ",resultA[1]," ",resultA[2],"\n"} 
    if (resultA[2]) then
       resultA[2] = "(" .. resultA[2] .. ")"
    end
@@ -643,7 +733,7 @@ function M.list_property(self, idx, sn, style, legendT)
    local cstr    = string.format("%3d)",idx)
 
    table.insert(resultA, 1, cstr)
-   dbg.fini("MT:list_property")
+   dbg.fini("MT:list_w_property")
    return resultA
 end
 
@@ -808,6 +898,50 @@ function M.get_ref_count(self,sn)
    return entry.ref_count
 end
 
+function M.get_depends_on_anyA(self,sn)
+   local entry = self.mT[sn]
+   if (entry == nil or not entry.ref_count) then
+      return nil
+   end
+   return entry.depends_on_anyA
+end
+
+function M.save_depends_on_any(self, sn, child_sn)
+   local entry = self.mT[sn]
+   assert(entry)
+   local anyA = entry.depends_on_anyA or {}
+   anyA[#anyA + 1] = child_sn
+   entry.depends_on_anyA = anyA
+end
+
+function M.pop_depends_on_any(self, sn)
+   local entry = self.mT[sn]
+   assert(entry)
+   if (not (entry.depends_on_anyA and next(entry.depends_on_anyA) ~= nil)) then
+      return nil
+   end
+   local child_sn = table.remove(entry.depends_on_anyA,1)
+   return child_sn
+end
+
+function M.pop_depends_on_any_ck(self,sn)
+   dbg.start{"MT:pop_depends_on_any_ck(sn: \"",sn,"\")"}
+   local entry = self.mT[sn]
+   assert(entry)
+   if (not (entry.depends_on_anyA and next(entry.depends_on_anyA) ~= nil)) then
+      dbg.fini("MT:pop_depends_on_any_ck with no doA")
+      return nil
+   end
+   if (not entry.__depends_on_any_ckA) then
+      entry.__depends_on_any_ckA = deepcopy(entry.depends_on_anyA)
+   end
+   local child_sn = table.remove(entry.__depends_on_any_ckA,1)
+   dbg.print{"child_sn: ",child_sn,"\n"}
+   dbg.fini("MT:pop_depends_on_any_ck")
+   return child_sn
+end
+
+
 function M.updateMPathA(self, value)
    if (type(value) == "string") then
       local clearDblSlash = true
@@ -1196,8 +1330,8 @@ function M.getMTfromFile(self,tt)
       dbg.print{"sn: ",sn,", hash: ", t[sn], "\n"}
    end
 
-   local force = true
-   Purge(force)
+   
+   mcp:purge{force=true}
 
    local savedBaseMPATH = l_mt.systemBaseMPATH
    dbg.print{"Saved baseMPATH: ",savedBaseMPATH,"\n"}
@@ -1235,10 +1369,12 @@ function M.getMTfromFile(self,tt)
 
    s_mt = false
    __removeEnvMT()
+   
+   local varT = frameStk:varT()
    FrameStk:__clear()
    dbg.print{"Setting MODULEPATH to: ",savedMPATH,"\n"}
    posix_setenv(ModulePath, savedMPATH, true)
-   frameStk = FrameStk:singleton()
+   frameStk = FrameStk:singleton{varT = varT}
    mt       = frameStk:mt()
    mt.systemBaseMPATH = savedBaseMPATH
    dbg.print{"savedBaseMPATH: ",savedBaseMPATH,"\n"}
@@ -1289,11 +1425,13 @@ function M.getMTfromFile(self,tt)
       if (ref_count) then
          ref_count = ref_count - 1
       end
+      mname:set_depends_on_anyA(activeA[i].depends_on_anyA)
       mname:set_depends_on_flag(activeA[i].ref_count)
       mname:set_ref_count(ref_count)
       mname:setStackDepth(activeA[i].stackDepth)
       mA[#mA+1]   = mname
    end
+   dbg.print{"Running MCP.load(mcp, mA)\n"}
    MCP.load(mcp,mA)
    mcp        = mcp_old
    dbg.print{"Setting mcp to ", mcp:name(),"\n"}
@@ -1414,4 +1552,76 @@ function M.resetMPATH2system(self)
    return self.systemBaseMPATH
 end
 
+function M.name_w_possible_alias(self, entry, kind)
+   local moduleName = entry.fullName
+   if (kind ~= "terse") then
+      moduleName    = hook.apply("colorize_fullName", moduleName, entry.sn) or moduleName
+   end
+
+   if (entry.origUserName) then
+      if (kind == "terse") then
+         moduleName = entry.fullName .. "\n" .. entry.origUserName
+      else
+         moduleName = entry.origUserName .. " -> " .. moduleName
+      end
+   end
+   return moduleName
+end
+
+------------------------------------------------------------------------
+-- Register Downstream conflicts
+function M.registerConflicts(self, mname, mA)
+   if (dbg.active()) then
+      local s = mAList(mA)
+      dbg.start{"MT:registerConflicts(sn:", mname:sn(),",mA={"..s.."})"}
+   end
+   local sn = mname:sn()
+   local a  = deepcopy(mA)
+   local A  = self.__conflictT[sn] or {}
+   for i = 1,#a do
+      A[#A+1] = a[i]
+   end
+   self.__conflictT[sn] = A
+   dbg.fini("MT:registerConflicts")
+end
+------------------------------------------------------------------------
+-- Unregister Downstream conflicts
+function M.removeConflicts(self, mname)
+   local sn = mname:sn()
+   if (dbg.active()) then
+      dbg.start{"MT:removeConflicts(sn:", sn,")"}
+   end
+   self.__conflictT[sn] = nil
+   dbg.fini("MT:removeConflicts")
+end
+
+function M.haveDSConflict(self, mnameIn)
+   if (dbg.active()) then
+      local snIn   = mnameIn:sn()
+      dbg.start{"MT:haveDSConflict(sn:", snIn,")"}
+   end
+
+   local cT    = self.__conflictT
+   local MName = require("MName")
+   for sn, vv in pairs(cT) do
+      dbg.print{"upstreamSn: ",sn,"\n"}
+      for i = 1,#vv do
+         local conflict_mname = vv[i]
+         dbg.print{"conflict_mname:userName(): ",conflict_mname:userName(),"\n"}
+         local userName_in_MT = self:lookup_w_userName(conflict_mname:userName())
+         dbg.print{"userName_in_MT: ",userName_in_MT,"\n"}
+         if (self:lookup_w_userName(conflict_mname:userName())) then
+            local snUpstream  = conflict_mname:downstreamConflictCk(mnameIn)
+            if (snUpstream) then
+               return sn
+            end
+         end
+      end
+   end
+
+   dbg.fini("MT:haveDSConflict")
+   return false
+end
+
+
 return M
