@@ -69,6 +69,7 @@ local s_performDepCk   = false
 local s_missDepT       = {}
 local s_missingModuleT = {}
 local s_missingFlg     = false
+local s_purgeFlg       = false
 
 --------------------------------------------------------------------------
 -- Remember the user's requested load array into an internal table.
@@ -202,6 +203,13 @@ function M.MNameType(self)
    return self.my_sType
 end
 
+--------------------------------------------------------------------------
+-- Return the sType for prereq/prereg_any
+-- @param self A MainControl object
+function M.MNamePrereqType(self)
+   return (cosmic:value("MODULES_AUTO_HANDLING") == "yes") and self.my_sType or "mt"
+end
+
 --------------------------------------------------------------------------
 -- Return the tcl_mode.
 -- @param self A MainControl object
@@ -305,7 +313,7 @@ end
 -- @param value the environment variable value.
 -- @param respect If true, then respect the old value.
 function M.setenv(self, name, value, respect)
-   name = name:trim()
+   name = (name or ""):trim()
    dbg.start{"MainControl:setenv(\"",name,"\", \"",value,"\", \"",
               respect,"\")"}
 
@@ -335,7 +343,7 @@ end
 -- Set an environment variable.
 -- This function just sets the name with value in the current env.
 function M.setenv_env(self, name, value, respect)
-   name = name:trim()
+   name = (name or ""):trim()
    dbg.start{"MainControl:setenv_env(\"",name,"\", \"",value,"\", \"",
               respect,"\")"}
    posix.setenv(name, value, true)
@@ -350,7 +358,7 @@ end
 -- @param value the environment variable value.
 -- @param respect If true, then respect the old value.
 function M.unsetenv(self, name, value, respect)
-   name = name:trim()
+   name = (name or ""):trim()
    dbg.start{"MainControl:unsetenv(\"",name,"\", \"",value,"\")"}
 
    l_check_for_valid_name("unsetenv",name)
@@ -386,7 +394,7 @@ end
 -- @param name the environment variable name.
 -- @param value the environment variable value.
 function M.pushenv(self, name, value)
-   name = name:trim()
+   name = (name or ""):trim()
    dbg.start{"MainControl:pushenv(\"",name,"\", \"",value,"\")"}
 
    l_check_for_valid_name("pushenv",name)
@@ -442,7 +450,7 @@ end
 -- @param name the environment variable name.
 -- @param value the environment variable value.
 function M.popenv(self, name, value)
-   name = name:trim()
+   name = (name or ""):trim()
    dbg.start{"MainControl:popenv(\"",name,"\", \"",value,"\")"}
 
    l_check_for_valid_name("popenv",name)
@@ -606,7 +614,7 @@ end
 -- @param name the environment variable name.
 -- @param value the environment variable value.
 function M.set_alias(self, name, value)
-   name = name:trim()
+   name = (name or ""):trim()
    dbg.start{"MainControl:set_alias(\"",name,"\", \"",value,"\")"}
 
    l_check_for_valid_alias_name("set_alias",name)
@@ -628,7 +636,7 @@ end
 -- @param name the environment variable name.
 -- @param value the environment variable value.
 function M.unset_alias(self, name, value)
-   name = name:trim()
+   name = (name or ""):trim()
    dbg.start{"MainControl:unset_alias(\"",name,"\", \"",value,"\")"}
 
    local frameStk = FrameStk:singleton()
@@ -648,7 +656,7 @@ end
 -- @param name the environment variable name.
 -- @param value the environment variable value.
 function M.set_shell_function(self, name, bash_function, csh_function)
-   name = name:trim()
+   name = (name or ""):trim()
    dbg.start{"MainControl:set_shell_function(\"",name,"\", \"",bash_function,"\"",
              "\", \"",csh_function,"\""}
 
@@ -671,7 +679,7 @@ end
 -- @param name the environment variable name.
 -- @param value the environment variable value.
 function M.unset_shell_function(self, name, bash_function, csh_function)
-   name = name:trim()
+   name = (name or ""):trim()
    dbg.start{"MainControl:unset_shell_function(\"",name,"\", \"",bash_function,"\"",
              "\", \"",csh_function,"\""}
 
@@ -769,21 +777,33 @@ function M.myModuleVersion(self)
    return frameStk:version()
 end
 
-local function l_generateMsg(kind, label, ...)
+local function l_generateMsg(kind, label, argA)
    local sA     = {}
    local twidth = TermWidth()
-   local argA   = pack(...)
    if (argA.n == 1 and type(argA[1]) == "table") then
+      local msg = nil
+      local key = nil
       local t   = argA[1]
-      local key = t.msg
-      local msg = i18n(key, t) 
+      if (t.literal_msg) then
+         msg = t.literal_msg
+      else
+         key = t.msg
+         msg = i18n(key, t)
+      end
       if (not msg) then
          msg = "Unknown Error Message with unknown key: \"".. key .. "\""
       end
       msg       = hook.apply("errWarnMsgHook", kind, key, msg, t) or msg
-      sA[#sA+1] = buildMsg(twidth, label, msg)
+      sA[#sA+1] = buildMsg(twidth, pack(label, msg))
    else
-      sA[#sA+1] = buildMsg(twidth, label, ...)
+      local ssA = {}
+      ssA[#ssA+1] = label
+      local n = #argA
+      for i = 1, n do
+         ssA[#ssA+1] = argA[i]
+      end
+      ssA.n = n+1
+      sA[#sA+1] = buildMsg(twidth, ssA)
    end
    return sA
 end
@@ -803,6 +823,7 @@ end
 -- Print msgs to stderr.
 -- @param self A MainControl object.
 function M.message(self, ...)
+   build_i18n_messages()
    if (quiet()) then
       return
    end
@@ -814,9 +835,9 @@ function M.message(self, ...)
       local key = t.msg
       local msg = i18n(key, t) or "Unknown Message"
       msg       = hook.apply("errWarnMsgHook", "lmodmessage", key, msg, t) or msg
-      sA[#sA+1] = buildMsg(twidth, msg)
+      sA[#sA+1] = buildMsg(twidth, {n=1, msg})
    else
-      sA[#sA+1] = buildMsg(twidth, ...)
+      sA[#sA+1] = buildMsg(twidth, pack(...))
    end
    io.stderr:write(concatTbl(sA,""),"\n")
 end
@@ -825,14 +846,15 @@ end
 -- Print msgs, traceback then set warning flag.
 -- @param self A MainControl object.
 function M.warning(self, ...)
+   build_i18n_messages()
    if (not quiet() and  haveWarnings()) then
       local label = colorize("red", i18n("warnTitle",{}))
-      local sA    = l_generateMsg("lmodwarning", label, ...)
+      local argA  = pack(...)
+      local sA    = l_generateMsg("lmodwarning", label, argA)
       sA[#sA+1]   = "\n"
       sA[#sA+1]   = moduleStackTraceBack()
       sA[#sA+1]   = "\n"
       io.stderr:write(concatTbl(sA,""),"\n")
-      setWarningFlag()
    end
 end
 
@@ -840,6 +862,7 @@ end
 -- Print msgs, traceback then exit.
 -- @param self A MainControl object.
 function M.error(self, ...)
+   build_i18n_messages()
    -- Check for user loads that failed.
    if (next(s_missingModuleT) ~= nil) then
       local aa = {}
@@ -853,16 +876,24 @@ function M.error(self, ...)
    end
 
    local label = colorize("red", i18n("errTitle", {}))
-   local sA    = l_generateMsg("lmoderror", label, ...)
+   local argA  = pack(...)
+   local sA    = l_generateMsg("lmoderror", label, argA)
    sA[#sA+1]   = "\n"
 
-   local a = concatTbl(stackTraceBackA,"")
-   if (a:len() > 0) then
-       sA[#sA+1] = a
-       sA[#sA+1] = "\n"
+   local noTraceBack = false
+   if (argA.n == 1 and type(argA[1] == "table")) then
+      noTraceBack = argA[1].noTraceBack
+   end
+
+   if (not noTraceBack) then
+      local a = concatTbl(stackTraceBackA,"")
+      if (a:len() > 0) then
+         sA[#sA+1] = a
+         sA[#sA+1] = "\n"
+      end
+      sA[#sA+1]     = moduleStackTraceBack()
+      sA[#sA+1]     = "\n"
    end
-   sA[#sA+1]     = moduleStackTraceBack()
-   sA[#sA+1]     = "\n"
 
    io.stderr:write(concatTbl(sA,""),"\n")
    LmodErrorExit()
@@ -921,6 +952,38 @@ function M.dependencyCk(self,mA)
    return {}
 end
 
+function M.dependencyCk_any(self, mA)
+   if (dbg.active()) then
+      local s = mAList(mA)
+      dbg.start{"MainControl:dependencyCk_any(mA={"..s.."})"}
+   end
+
+   local frameStk = FrameStk:singleton()
+   local mt       = frameStk:mt()
+   local fullName = frameStk:fullName()
+   local child_sn = mt:pop_depends_on_any_ck(frameStk:sn())
+   if (not child_sn) then
+      return {}
+   end
+
+   for i = 1,#mA do
+      repeat
+         local mname = mA[i]
+         local sn    = mname:sn()
+         if (not sn) then break end
+         if (child_sn ~= sn) then break end
+         if (not mname:isloaded() ) then
+            local a = s_missDepT[mname:userName()] or {}
+            a[#a+1] = fullName
+            s_missDepT[mname:userName()] = a
+         end
+      until true
+   end
+
+   dbg.fini("MainControl:dependencyCk_any")
+   return {}
+end
+
 function M.reportMissingDepModules(self)
    local t = s_missDepT
    if (next(t) ~= nil) then
@@ -949,22 +1012,21 @@ function M.depends_on(self, mA)
       dbg.start{"MainControl:depends_on(mA={"..s.."})"}
    end
 
+   local mB = {}
+   local mt = FrameStk:singleton():mt()
 
-   local mB         = {}
-   local mt         = FrameStk:singleton():mt()
-
-   for i = 1,#mA do
-      local mname = mA[i]
+   for i                                       = 1,#mA do
+      local mname                              = mA[i]
       if (not mname:isloaded()) then
          mname:set_depends_on_flag(true)
-         mB[#mB + 1] = mname
+         mB[#mB + 1]                           = mname
       else
          mt:safely_incr_ref_count(mname)
       end
    end
 
    l_registerUserLoads(mB)
-   local a = self:load(mB)
+   local a                                     = self:load(mB)
 
    self:registerDependencyCk()
 
@@ -972,14 +1034,63 @@ function M.depends_on(self, mA)
    return a
 end
 
+
+
+
+
+-------------------------------------------------------------------
+-- depends_on_any() a list of modules.  This is short hand for:
+--
+--   if (isloaded(any("mod1, mod2, modN"))) then done end
+--   if no module loadable error.
+--
+
+function M.depends_on_any(self, mA) 
+   if (dbg.active()) then
+      local s                                  = mAList(mA)
+      dbg.start{"MainControl:depends_on_any(mA = {"..s.."})"}
+   end
+
+   local mt = FrameStk:singleton():mt()
+   local mB = {}
+
+   for i = 1,#mA do
+      local mname = mA[i]
+      if (mname:isloaded()) then
+         mt:safely_incr_ref_count(mname)
+         mt:save_depends_on_any(myModuleName(), mname:sn())
+         dbg.fini("MainControl:depends_on_any")
+         return {}
+      elseif (mname:sn()) then
+         mB[#mB + 1] = mname
+      end
+   end 
+
+   if (next(mB) == nil) then 
+      local s = mAList(mA)
+      LmodError{msg="e_Failed_depends_any", module_list=s}
+   end
+
+   local mC = {mB[1]}
+   local mname = mC[1]
+   mname:set_depends_on_flag(true)
+   mt:save_depends_on_any(myModuleName(), mname:sn())
+
+   l_registerUserLoads(mC)
+   local a = self:load(mC)
+   
+   self:registerDependencyCk()
+   
+   dbg.fini("MainControl:depends_on_any")
+   return a
+end
+
 -------------------------------------------------------------------
 -- forgo a list of modules.  This is the reverse of depends_on()
 --
 --   if (not isloaded("name")) then load("name") end
 --
--- On unload forgo() unloads the ref count is zero.
-
-
+-- On unload forgo() unloads when the ref count is zero.
 function M.forgo(self,mA)
    local hub = Hub:singleton()
    if (dbg.active()) then
@@ -1007,11 +1118,55 @@ function M.forgo(self,mA)
    return aa
 end
 
+-------------------------------------------------------------------
+-- forgo_any a list of modules.  This is the reverse of depends_on_any()
+--
+-- On unload forgo_any() unloads when the ref count is zero.
+function M.forgo_any(self,mA)
+   local hub = Hub:singleton()
+   if (dbg.active()) then
+      local s = mAList(mA)
+      dbg.start{"MainControl:forgo_any(mA={"..s.."})"}
+   end
 
+   local mt       = FrameStk:singleton():mt()
+   local mB       = {}
+   local child_sn = mt:pop_depends_on_any(myModuleName())
+   if (child_sn == nil) then
+      return {}
+   end
+   local found = false
+   for i = 1,#mA do
+      repeat
+         local mname = mA[i]
+         local sn    = mname:sn()
+         if (not sn) then break end
+         if (child_sn ~= sn) then break end
+         
+         ------------------------------------------------------------
+         -- if here then we have found the only depends_on_any module
+         -- that we "loaded". So decrement ref count and unload when
+         -- the ref_count is zero.
+         -- Finally stop looking over mname's in mA
+
+         local ref_count  = mt:decr_ref_count(sn)
+         if (ref_count and ref_count < 1) then
+            mB[#mB+1] = mname
+         end
+         found = true
+      until true
+      if (found) then break end
+   end
+
+   l_unRegisterUserLoads(mB)
+   local aa     = unload_internal(mB)
+   dbg.fini("MainControl:forgo_any")
+   return aa
+end
 
 -------------------------------------------------------------------
 -- Load a list of modules.  Check to see if the user requested
--- modules were actually loaded.
+-- modules were actually loaded.G
 -- @param self A MainControl object
 -- @param mA A array of MName objects.
 -- @return An array of statuses
@@ -1126,7 +1281,6 @@ end
 -- @param mA A array of MName objects.
 function M.try_load(self, mA)
    dbg.start{"MainControl:try_load(mA)"}
-   --deactivateWarning()
    self:load(mA)
    dbg.fini("MainControl:try_load")
 end
@@ -1202,28 +1356,50 @@ end
 function M.conflict(self, mA)
    dbg.start{"MainControl:conflict(mA)"}
 
-
-   local frameStk  = FrameStk:singleton()
-   local mt        = frameStk:mt()
-   local fullName  = frameStk:fullName()
-   local optionTbl = optionTbl()
-   local a         = {}
+   local frameStk    = FrameStk:singleton()
+   local mt          = frameStk:mt()
+   local fullName    = frameStk:fullName()
+   local optionTbl   = optionTbl()
+   local a           = {}
+   local dsConflicts = cosmic:value("LMOD_DOWNSTREAM_CONFLICTS")
 
    for i = 1, #mA do
       local mname    = mA[i]
-      local sn       = mname:sn()  -- this will return false if there is no module loaded.
-      local userName = mname:userName()
-      if (sn and mt:have(sn,"active") and (userName == sn or extractVersion(userName, sn) == mt:version(sn))) then
+      local userName = mname:conflictCk(mt)
+      if ( userName ) then
          a[#a+1]  = userName
       end
    end
 
+   ------------------------------------------------------------------------
+   -- check for current conflicits
+
    if (#a > 0) then
       LmodError{msg="e_Conflict", name = fullName, module_list = concatTbl(a," ")}
    end
+
+   ------------------------------------------------------------------------
+   -- register downstream conflicts
+   if (dsConflicts == "yes") then
+      mt:registerConflicts(frameStk:mname(), mA)
+   end
+
    dbg.fini("MainControl:conflict")
 end
 
+function M.removeConflict(self, mA)
+   dbg.start{"MainControl:removeConflict(mA)"}
+   local frameStk    = FrameStk:singleton()
+   local mt          = frameStk:mt()
+   local dsConflicts = cosmic:value("LMOD_DOWNSTREAM_CONFLICTS")
+
+   if (dsConflicts == "yes" ) then
+      mt:removeConflicts(frameStk:mname())
+   end
+   
+   dbg.fini("MainControl:removeConflict")
+end
+
 --------------------------------------------------------------------------
 -- Check the prereq from *mA*.
 -- @param self A MainControl object.
@@ -1438,7 +1614,7 @@ function M.reportAdminMsgs()
             io.stderr:write(bt:build_tbl(), "\n")
          end
       end
-      io.stderr:write(border,"\n\n")
+      io.stderr:write(i18n("m_Module_Msgs_close",{border=border}))
    end
    dbg.fini("MainControl:reportAdminMsgs")
 end
@@ -1465,7 +1641,7 @@ function M.add_property(self, name, value)
    local sn        = frameStk:sn()
    local mt        = frameStk:mt()
    l_check_for_valid_name("add_property",name)
-   mt:add_property(sn, name:trim(), value)
+   mt:add_property(sn, (name or ""):trim(), value)
 end
 
 
@@ -1479,9 +1655,43 @@ function M.remove_property(self, name, value)
    local sn        = frameStk:sn()
    local mt        = frameStk:mt()
    l_check_for_valid_name("remove_property",name)
-   mt:remove_property(sn, name:trim(), value)
+   mt:remove_property(sn, (name or ""):trim(), value)
 end
 
+
+function purgeFlg()
+   return s_purgeFlg
+end
+
+
+function M.purge(self,t)
+   local force = false
+   if (type(t) == "table") then
+      force = t.force
+   end
+
+   local frameStk = FrameStk:singleton()
+   local mt       = frameStk:mt()
+   local totalA   = mt:list("short","any") --> "any" does not include "pending"
+
+   if (#totalA < 1) then
+      return
+   end
+
+   local mA = {}
+   for i = #totalA,1,-1 do
+      mA[#mA+1] = MName:new("mt",totalA[i])
+   end
+   s_purgeFlg = true
+   unload_usr_internal(mA, force)
+   s_purgeFlg = false
+
+   dbg.fini("MainControl:Purge")
+end
+   
+
+
+
 --------------------------------------------------------------------------
 -- Return the tcl_mode.
 -- @param self A MainControl object
@@ -1557,11 +1767,12 @@ function M.complete(self, shellName, name, args)
       return
    end
    
-   local varT     = FrameStk:singleton():varT()
-   if (varT[name] == nil) then
-      varT[name] = Var:new(name)
+   local varT = FrameStk:singleton():varT()
+   local n    = wrap_complete(name)
+   if (varT[n] == nil) then
+      varT[n] = Var:new(n)
    end
-   varT[name]:complete(args)
+   varT[n]:complete(args)
    dbg.fini("MainControl:complete")
 end
 
@@ -1571,11 +1782,12 @@ function M.uncomplete(self, shellName, name, args)
       dbg.fini("MainControl:complete")
       return
    end
-   local varT     = FrameStk:singleton():varT()
-   if (varT[name] == nil) then
-      varT[name] = Var:new(name)
+   local varT = FrameStk:singleton():varT()
+   local n    = wrap_complete(name)
+   if (varT[n] == nil) then
+      varT[n] = Var:new(n)
    end
-   varT[name]:uncomplete()
+   varT[n]:uncomplete()
 
    dbg.fini("MainControl:uncomplete")
 end
@@ -1612,14 +1824,13 @@ function M.LmodBreak(self, msg)
       shell:echo(concatTbl(b,""))
    end
 
-
    if (msg and msg ~= "") then
       LmodMessage(msg)
    end
 
    -- Remove this module from the list of user requested modules to load.
-   local sn    = frameStk:sn()
-   local mname = MName:new("mt",sn)
+   local mname = frameStk:mname()
+   dbg.print{"sn: ",mname:sn(),", userName: ",mname:userName(),"\n"}
    l_unRegisterUserLoads{mname}
 
    -- Copy the previous frameStk on top of the current stack
@@ -1629,10 +1840,8 @@ function M.LmodBreak(self, msg)
    dbg.fini("MainControl:LmodBreak")
 end
 
-
-
 function M.userInGroups(self, ...)
-   local grps   = capture("groups")
+   local grps   = capture("groups 2> /dev/null")
    local argA   = pack(...)
    for g in grps:split("[ \n]") do
       for i = 1, argA.n do
@@ -1642,7 +1851,7 @@ function M.userInGroups(self, ...)
          end
       end
    end
-   local userId = capture("id -u")
+   local userId = capture("id -u 2> /dev/null")
    local isRoot = tonumber(userId) == 0
    if (isRoot) then
       return true
