@@ -50,8 +50,6 @@ local dbg         = require("Dbg"):dbg()
 local getenv      = os.getenv
 local s_moduleA   = false
 local sort        = table.sort
-local exact_match = cosmic:value("LMOD_EXACT_MATCH")
-local find_first  = cosmic:value("LMOD_TMOD_FIND_FIRST")
 
 -- print(__FILE__() .. ':' .. __LINE__())
 ----------------------------------------------------------------------
@@ -140,13 +138,13 @@ local function l_GroupIntoModules(self, level, maxdepth, mpath, dirT, T)
 end
 
 local function l_build(self, maxdepthT, dirA)
-   --dbg.start{"ModuleA l_build()"}
+   dbg.start{"ModuleA l_build()"}
    local moduleA = {}
    local sz      = #dirA
    --dbg.print{"#dirA: ",sz,"\n"}
    for i = 1,sz do
       local mpath    = dirA[i].mpath
-      --dbg.print{"mpath: ",mpath,"\n"}
+      dbg.print{"mpath: ",mpath,"\n"}
       local dirT     = dirA[i].dirT
       local T        = {}
       local maxdepth = maxdepthT[mpath] or -1
@@ -154,7 +152,7 @@ local function l_build(self, maxdepthT, dirA)
       l_GroupIntoModules(self, level, maxdepth, mpath, dirT, T)
       moduleA[#moduleA + 1] = {mpath = mpath, T = T}
    end
-   --dbg.fini("ModuleA l_build")
+   dbg.fini("ModuleA l_build")
    return moduleA
 end
 
@@ -188,8 +186,8 @@ local function l_find_vA(name, moduleA)
    if (idx) then
       versionStr = name:sub(idx+1,-1)
    end
-   --dbg.print{"sn: ",sn,", versionStr: ",versionStr,"\n"}
-   --dbg.printT("vA",vA)
+   dbg.print{"sn: ",sn,", versionStr: ",versionStr,"\n"}
+   dbg.printT("l_find_vA: vA",vA)
    return sn, versionStr, vA
 end
 
@@ -230,19 +228,20 @@ local function l_find_vB(sn, versionStr, vA)
          vB[#vB + 1] = v
       end
    end
-   --dbg.printT("vB",vB)
+   dbg.printT("l_find_vB: vB",vB)
    return fullStr, vB
 end
 
 local function l_search(name, moduleA)
-   --dbg.start{"ModuleA l_search(",name,",moduleA)"}
+   dbg.start{"ModuleA l_search(\"",name,"\", moduleA)"}
    -- First find sn and collect all v's into vA
 
    local sn, versionStr, vA = l_find_vA(name, moduleA)
    if (sn == nil) then return nil end
+   dbg.print{"sn: ",sn,", versionStr: ",versionStr,"\n"}
    local fullStr, vB        = l_find_vB(sn, versionStr, vA)
 
-   --dbg.print{"name: ",name,", sn: ",sn,", versionStr: ",versionStr, " fullStr: ",fullStr,"\n"}
+   dbg.print{"name: ",name,", sn: ",sn,", versionStr: ",versionStr, " fullStr: ",fullStr,"\n"}
 
    local extended_default = cosmic:value("LMOD_EXTENDED_DEFAULT")
 
@@ -251,12 +250,13 @@ local function l_search(name, moduleA)
       fileA[i] = {}
       collectFileA(sn, fullStr, extended_default, vB[i], fileA[i])
    end
-   --dbg.fini("ModuleA l_search")
+   dbg.printT("fileA",fileA)
+   dbg.fini("ModuleA l_search")
    return sn, versionStr, fileA
 end
 
 function M.applyWeights(self,fullNameDfltT)
-   --dbg.start{"ModuleA:applyWeights(fullNameDfltT)"}
+   dbg.start{"ModuleA:applyWeights(fullNameDfltT)"}
    for fullName, weight in pairs(fullNameDfltT) do
       repeat
          local sn, versionStr, vA = l_find_vA(fullName, self.__moduleA)
@@ -282,16 +282,18 @@ function M.applyWeights(self,fullNameDfltT)
          end
       until true
    end
-   --dbg.fini("ModuleA:applyWeights")
+   dbg.fini("ModuleA:applyWeights")
 end
 
 
 function M.__find_all_defaults(self)
-   --dbg.start{"ModuleA:__find_all_defaults()"}
+   dbg.start{"ModuleA:__find_all_defaults()"}
    local moduleA     = self.__moduleA
    local defaultT    = self.__defaultT
-   local show_hidden = optionTbl().show_hidden
    local mrc         = MRC:singleton()
+   local show_hidden = MRC:show_hidden()
+
+   dbg.printT("defaultT",defaultT)
 
    local function l_find_all_defaults_helper(level,isNVV, mpath, sn, v)
       local weight, keepLooking, fn, idx
@@ -312,16 +314,23 @@ function M.__find_all_defaults(self)
       end
 
       if (keepLooking) then
-         if (v.file and (show_hidden or mrc:isVisible{fullName=sn, sn=sn, fn=v.file})) then
-            defaultT[sn] = {weight = "999999999.*zfinal", fullName = sn, fn = v.file, count = 1}
-         elseif (next(v.fileT) ~= nil) then
+         if (next(v.fileT) ~= nil) then
             for fullName, vv in pairs(v.fileT) do
-               local vis = mrc:isVisible{fullName=fullName, sn=sn, fn=vv.fn} or isMarked(vv.wV)
+               local wV      = mrc:find_wght_for_fullName(fullName, vv.wV)
+               local resultT = mrc:isVisible{fullName=fullName, sn=sn, fn=vv.fn, visibleT = {soft=true}, mpath=vv.mpath}
+               local vis     = (resultT.isVisible or isMarked(wV))
+
+               dbg.print{"l_find_all_defaults_helper: fullName: ",fullName, ", vis: ",vis,", resultT.kind: ",resultT.kind,"\n"}
+               ------------------------------------------------------------
+               -- When hidden modules are shown (show_hidden=true) then
+               -- the count goes up.  However only visible modules can have
+               -- the (D) marking (i.e. default) unless they are marked defaults.
+
                if (show_hidden or vis) then
-                  count = count + 1
-                  if (vis and (vv.wV > weight)) then
+                  count = count + ((resultT.count) and 1 or 0)
+                  if (vis and (wV > weight) and (resultT.moduleKindT.kind ~= "hidden" or isMarked(wV)))  then
                      found      = true
-                     weight     = vv.wV
+                     weight     = wV
                      ext        = vv.luaExt and ".lua" or ""
                      fn         = pathJoin(mpath, fullName .. ext)
                      myfullName = fullName
@@ -359,28 +368,27 @@ function M.__find_all_defaults(self)
       t[v.fn] = { sn = k, count = v.count, fullName = v.fullName, weight = v.weight }
    end
    self.__defaultT = t
-   --dbg.fini("ModuleA:__find_all_defaults")
+   dbg.fini("ModuleA:__find_all_defaults")
 end
 
 
 function M.build_availA(self)
    dbg.start{"ModuleA:build_availA()"}
-   local show_hidden = optionTbl().show_hidden
    local mrc         = MRC:singleton()
 
    local function l_build_availA_helper(mpath, sn, v, A)
       local icnt = #A
-      if (v.file ) then
-         if (show_hidden or mrc:isVisible{fullName=sn,sn=sn,fn=v.file}) then
-            local metaModuleT = v.metaModuleT or {}
-            A[icnt+1] = { fullName = sn, pV = sn, fn = v.file, sn = sn, propT = metaModuleT.propT}
-         end
-      end
       if (next(v.fileT) ~= nil) then
          for fullName, vv in pairs(v.fileT) do
-            if (show_hidden or mrc:isVisible{fullName=fullName,sn=sn,fn=vv.fn}) then
+            dbg.print{"fullName: ",fullName,"\n"}
+            local resultT = mrc:isVisible{fullName=fullName, sn=sn, fn=vv.fn, mpath=vv.mpath}
+            if (resultT.isVisible) then
                icnt    = icnt + 1
-               A[icnt] = { fullName = fullName, pV = pathJoin(sn,vv.pV), fn = vv.fn, sn = sn, propT = vv.propT}
+               dbg.print{"saving fullName: ",fullName,"\n"}
+               A[icnt] = { fullName = fullName, pV = pathJoin(sn,vv.pV), fn = vv.fn, sn = sn,
+                           propT = vv.propT, provides = vv.provides, moduleKindT = resultT.moduleKindT,
+                           forbiddenT = mrc:isForbidden{fullName=fullName, sn = sn, fn = vv.fn, mpath=vv.mpath},
+                         }
             end
          end
       end
@@ -406,7 +414,7 @@ function M.build_availA(self)
       end
       sort(availA[i].A, cmp)
    end
-   --dbg.fini("ModuleA:build_availA")
+   dbg.fini("ModuleA:build_availA")
    return availA
 end
 
@@ -416,22 +424,7 @@ function M.inherited_search(self, search_fullName, orig_fn)
    local function l_inherited_search_helper(key, count, v)
       --dbg.start{"l_inherited_search_helper(",key,",", count,",v)"}
       local fn = false
-      if (v.file) then
-         if (search_fullName == key) then
-            if (count == 0) then
-               if (v.file == orig_fn) then
-                  fn = v.file
-                  count = 1
-               end
-            elseif (count == 1) then
-               fn = v.file
-               count = 2
-            end
-            --dbg.print{"found matching v.file\n"}
-            --dbg.fini("l_inherited_search_helper")
-            return fn, count
-         end
-      elseif (next(v.fileT) ~= nil) then
+      if (next(v.fileT) ~= nil) then
          local entryT = v.fileT[search_fullName]
          if (entryT) then
             if (count == 0) then
@@ -483,15 +476,20 @@ function M.inherited_search(self, search_fullName, orig_fn)
 end
 
 function M.search(self, name)
+   dbg.start{"ModuleA:search(\"",name,"\")"}
    if (self.__isNVV) then
-      return l_search(name, self.__moduleA)
+      local sn, versionStr, fileA = l_search(name, self.__moduleA)
+      dbg.fini("ModuleA:search")
+      return sn, versionStr, fileA 
    end
 
    if (not self.__locationT) then
       self.__locationT = LocationT:new(self.__moduleA)
    end
 
-   return self.__locationT:search(name)
+   local sn, versionStr, fileA = self.__locationT:search(name)
+   dbg.fini("ModuleA:search")
+   return sn, versionStr, fileA 
 end
 
 local function l_checkforNV(T)
@@ -504,16 +502,17 @@ local function l_checkforNV(T)
 end
 
 local function l_build_from_spiderT(spiderT)
-   --dbg.start{"ModuleA l_build_from_spiderT(spiderT)"}
-   local frameStk = FrameStk:singleton()
-   local mt       = frameStk:mt()
-   local mpathA   = mt:modulePathA()
-   local moduleA  = {}
-   local isNV     = find_first == "no"
+   dbg.start{"ModuleA l_build_from_spiderT(spiderT)"}
+   local find_first = cosmic:value("LMOD_TMOD_FIND_FIRST")
+   local frameStk   = FrameStk:singleton()
+   local mt         = frameStk:mt()
+   local mpathA     = mt:modulePathA()
+   local moduleA    = {}
+   local isNV       = find_first == "no"
    for i = 1, #mpathA do
       local mpath = mpathA[i]
       if (isDir(mpath)) then
-         --dbg.print{"pulling mpath: ",mpath," into moduleA\n"}
+         dbg.print{"pulling mpath: ",mpath," into moduleA\n"}
          local T = spiderT[mpath]
          if (T and next(T) ~= nil) then
             moduleA[#moduleA+1] = { mpath = mpath, T = deepcopy(T) }
@@ -523,7 +522,7 @@ local function l_build_from_spiderT(spiderT)
          end
       end
    end
-   --dbg.fini("ModuleA l_build_from_spiderT")
+   dbg.fini("ModuleA l_build_from_spiderT")
    return moduleA, not isNV
 end
 
@@ -534,7 +533,7 @@ end
 
 function M.update(self, t)
    t                   = t or {}
-   --dbg.start{"ModuleA:update(spider_cache = ",t.spider_cache,")"}
+   dbg.start{"ModuleA:update(spider_cache = ",t.spider_cache,")"}
    local frameStk      = FrameStk:singleton()
    local mt            = frameStk:mt()
    local varT          = frameStk:varT()
@@ -592,13 +591,14 @@ function M.update(self, t)
    self.__locationT = false
    self.__moduleA   = moduleA
    mt:updateMPathA(mpathA)
-   --dbg.fini("ModuleA:update")
+   dbg.fini("ModuleA:update")
 end
 
 
 function M.__new(self, mpathA, maxdepthT, moduleRCT, spiderT)
-   --dbg.start{"ModuleA:__new()"}
-   local o = {}
+   dbg.start{"ModuleA:__new()"}
+   local o          = {}
+   local find_first = cosmic:value("LMOD_TMOD_FIND_FIRST")
    setmetatable(o,self)
 
    local dirTree   = false
@@ -607,30 +607,21 @@ function M.__new(self, mpathA, maxdepthT, moduleRCT, spiderT)
    spiderT         = spiderT or {}
    if (next(spiderT) ~= nil) then
       o.__spiderBuilt        = true
-      --dbg.print{"calling l_build_from_spiderT()\n"}
-      --dbg.printT("spiderT",spiderT)
+      dbg.print{"calling l_build_from_spiderT()\n"}
+      dbg.printT("spiderT",spiderT)
       o.__moduleA, o.__isNVV = l_build_from_spiderT(spiderT)
    else
-      --dbg.print{"calling DirTree:new()\n"}
+      dbg.print{"calling DirTree:new()\n"}
       dirTree         = DirTree:new(mpathA)
       o.__spiderBuilt = false
       o.__moduleA     = l_build(o, maxdepthT, dirTree:dirA())
    end
 
-   --dbg.printT("moduleA:",o.__moduleA)
-   --dbg.print{"isNVV: ",o.__isNVV,"\n"}
-   if (moduleRCT and next(moduleRCT) ~= nil) then
-      --dbg.print{"apply weights\n"}
-      local mrc       = MRC:singleton(moduleRCT)
-      o:applyWeights(mrc:fullNameDfltT())
-   end
-   local mrc       = MRC:singleton()
-   --dbg.printT("mrcMpathT",mrc:mrcMpathT())
    o.__locationT   = false
    o.__defaultT    = {}
    
 
-   --dbg.fini("ModuleA:__new")
+   dbg.fini("ModuleA:__new")
    return o
 end
 
@@ -657,6 +648,7 @@ function M.locationT(self)
 end
 
 function M.defaultT(self)
+   local exact_match = cosmic:value("LMOD_EXACT_MATCH")
    if (exact_match == "yes") then
       return self.__defaultT
    end
@@ -668,10 +660,10 @@ function M.defaultT(self)
 end
 
 function M.singleton(self, t)
-   --dbg.start{"ModuleA:singleton(t)"}
+   dbg.start{"ModuleA:singleton(t)"}
    t = t or {}
    if (t.reset or (s_moduleA and s_moduleA:spiderBuilt())) then
-      --dbg.print{"Wiping out old value of s_moduleA\n"}
+      dbg.print{"Wiping out old value of s_moduleA\n"}
       self:__clear{testing=t.reset}
    end
    if (not s_moduleA) then
@@ -687,14 +679,14 @@ function M.singleton(self, t)
       s_moduleA = self:__new(mt:modulePathA(), mt:maxDepthT(), getModuleRCT(), spiderT)
    end
 
-   --dbg.fini("ModuleA:singleton")
+   dbg.fini("ModuleA:singleton")
    return s_moduleA
 end
 
 
 
 function M.__clear(self, t)
-   --dbg.start{"ModuleA:__clear()"}
+   dbg.start{"ModuleA:__clear()"}
    t = t or {}
    local MT = require("MT")
    s_moduleA = false
@@ -704,7 +696,7 @@ function M.__clear(self, t)
       Cache:__clear()
    end
    MT:__clearMT{testing=true}
-   --dbg.fini("ModuleA:__clear")
+   dbg.fini("ModuleA:__clear")
 end
 
 return M
