@@ -87,14 +87,16 @@ end
 -- place the positional arguments in *optionTbl.pargs*
 -- @param self An Option object.
 -- @param usage The program usage string.
-function M.singleton(self, usage)
+function M.singleton(self, progName, usage, description)
 
    local Optiks = require("Optiks")
-   local cmdlineParser  = Optiks:new{usage   = usage,
-                                     error   = LmodWarning,
-                                     exit    = l_nothing,
-                                     prt     = l_prt,
-                                     envArg  = os.getenv("LMOD_OPTIONS"),
+   local cmdlineParser  = Optiks:new{usage    = usage,
+                                     error    = LmodWarning,
+                                     exit     = l_nothing,
+                                     prt      = l_prt,
+                                     progName = progName,
+                                     envArg   = os.getenv("LMOD_OPTIONS"),
+                                     descript = description,
    }
 
    local styleA       = {}
@@ -160,7 +162,7 @@ function M.singleton(self, usage)
    cmdlineParser:add_option{
       name   = {"--pin_versions"},
       dest   = "pinVersions",
-      action = "store",
+      action = "store_true",
       help   = i18n("pin_hlp"),
    }
 
@@ -262,6 +264,13 @@ function M.singleton(self, usage)
       help   = i18n("dumpV_hlp")
    }
 
+   cmdlineParser:add_option{
+      name   = {"--dumpname"},
+      dest   = "dumpname",
+      action = "store_true",
+      help   = i18n("dumpN_hlp")
+   }
+
    cmdlineParser:add_option{
       name   = {"--check_syntax", "--checkSyntax"},
       dest   = "checkSyntax",
@@ -305,7 +314,7 @@ function M.singleton(self, usage)
    }
 
    cmdlineParser:add_option{
-      name   = {"--force" },
+      name   = {"-f","--force"},
       dest   = "force",
       action = "store_true",
       help   = i18n("force_hlp"),
@@ -326,7 +335,7 @@ function M.singleton(self, usage)
    }
 
    cmdlineParser:add_option{
-      name   = {"--show_hidden" },
+      name   = {"--show_hidden","-A", "--all"},
       dest   = "show_hidden",
       action = "store_true",
       help   = i18n("hidden_H")
@@ -364,6 +373,20 @@ function M.singleton(self, usage)
       default = 0.0
    }
 
+   cmdlineParser:add_option{
+      name   = {"--terse_show_extensions"},
+      dest   = "terseShowExtensions",
+      action = "store_true",
+      help   = i18n("terseShowExt_H"),
+   }
+
+   cmdlineParser:add_option{
+      name   = {"--pod"},
+      dest   = "cmdPod",
+      action = "store_true",
+      help   = i18n("pod_H"),
+   }
+
    local optTbl, pargs = cmdlineParser:parse(arg)
    local optionTbl     = optionTbl()
    optionTbl.pargs     = pargs
@@ -378,6 +401,20 @@ function M.singleton(self, usage)
       optionTbl.cmdHelpMsg   = cmdlineParser:buildHelpMsg()
    end
 
+   optionTbl.pod      = ""
+   if (optionTbl.cmdPod) then
+      local a = {}
+      a[#a+1] = cmdlineParser:buildManPod()
+      a[#a+1] = "\n"
+      a[#a+1] = "=head1 COMMAND OVERVIEW\n"
+      a[#a+1] = Usage()
+      a[#a+1] = "\n"
+      a[#a+1] = "=head1 AUTHOR\n\n"
+      a[#a+1] = "Robert McLay mclay@tacc.utexas.edu"
+
+      optionTbl.pod   = concatTbl(a,"\n") 
+   end
+
    if (optionTbl.trace) then
       cosmic:assign("LMOD_TRACING", "yes")
    end
@@ -412,12 +449,16 @@ function M.singleton(self, usage)
       cosmic:assign("LMOD_PIN_VERSIONS","yes")
    end
    if (optionTbl.ignoreCache) then
-      cosmic:assign("LMOD_IGNORE_CACHE", true)
+      cosmic:assign("LMOD_IGNORE_CACHE", "yes")
    end
 
    if (optionTbl.no_extensions) then
       cosmic:assign("LMOD_AVAIL_EXTENSIONS", "no")
    end
+
+   if (optionTbl.show_hidden) then
+      cosmic:assign("LMOD_SHOW_HIDDEN", "all")
+   end
 end
 
 return M
