@@ -70,7 +70,8 @@ function M.new(self)
    local o = {}
    setmetatable(o,self)
    self.__index = self
-   self.__name  = false
+
+   o.__name  = false
    return o
 end
 
@@ -180,7 +181,8 @@ local function l_loadMe(entryT, moduleStack, iStack, myModuleT, mt, mList, mpath
       tracing_msg{msg, fullName, " (fn: ", fn or "nil", ")"}
    end
 
-   loadModuleFile{file=fn, help=true, shell=shellNm, reportErr=false, mList = mList}
+   loadModuleFile{file=fn, help=true, shell=shellNm, reportErr=false,
+                  mList = mList, forbiddenT = {}}
    hook.apply("load_spider",{fn = fn, modFullName = fullName, sn = sn})
    mt:setStatus(sn, "active")
    reset_env()
@@ -190,10 +192,6 @@ local function l_findModules(mpath, mt, mList, sn, v, moduleT)
    local entryT
    local moduleStack = optionTbl().moduleStack
    local iStack      = #moduleStack
-   if (v.file) then
-      entryT   = { fn = v.file, sn = sn, userName = sn, fullName = sn, version = false}
-      l_loadMe(entryT, moduleStack, iStack, v.metaModuleT, mt, mList, mpath, sn, "Spider Loading:       ")
-   end
    if (next(v.fileT) ~= nil) then
       for fullName, vv in pairs(v.fileT) do
          vv.Version = extractVersion(fullName, sn)
@@ -213,9 +211,6 @@ local function l_findChangeMPATH_modules(mpath, mt, mList, sn, v, moduleT)
    local entryT
    local moduleStack = optionTbl().moduleStack
    local iStack      = #moduleStack
-   if (v.file) then
-      LmodError("Calling l_findChangeMPATH_modules w v.file")
-   end
    if (next(v.fileT) ~= nil) then
       for fullName, vv in pairs(v.fileT) do
          if (vv.changeMPATH == true) then
@@ -251,9 +246,10 @@ function M.searchSpiderDB(self, strA, dbT, providedByT)
       for fn, vv in pairs(vvv) do
          local whatisS  = concatTbl(vv.whatis or {},"\n"):lower()
          local found    = false
+         local help     = vv.help or ""
          for i = 1,strA.n do
             local str = strA[i]
-            if (sn:find(str) or whatisS:find(str)) then
+            if (sn:find(str) or whatisS:find(str) or help:find(str)) then
                found = true
                break
             end
@@ -600,12 +596,18 @@ local dbT_keyA = { 'Description', 'Category', 'URL', 'Version', 'whatis', 'dirA'
                    'family','pathA', 'lpathA', 'propT','help','pV','wV','provides'}
 
 
-function M.buildDbT(self, mpathA, mpathMapT, spiderT, dbT)
+function M.buildDbT(self, mpathMapT, spiderT, dbT)
    dbg.start{"Spider:buildDbT(mpathMapT,spiderT, dbT)"}
    dbg.printT("mpathMapT",mpathMapT)
    local mpathParentT = l_build_mpathParentT(mpathMapT)
    dbg.printT("spiderT",spiderT)
    dbg.printT("mpathParentT",mpathParentT)
+   local mpathA = {}
+   for k, v in pairs(spiderT) do
+      if (k ~= "version") then
+         mpathA[#mpathA + 1] = k
+      end
+   end
    dbg.printT("mpathA",mpathA)
    local keepT        = l_build_keepT(mpathA, mpathParentT, spiderT)
    local parentT      = l_build_parentT(keepT, mpathMapT)
@@ -615,21 +617,7 @@ function M.buildDbT(self, mpathA, mpathMapT, spiderT, dbT)
       return a[1] > b[1]
    end
    local function l_buildDbT_helper(mpath, sn, v, T)
-      if (v.file) then
-         local t = {}
-         for i = 1,#dbT_keyA do
-            local key = dbT_keyA[i]
-            t[key]    = v.metaModuleT[key]
-         end
-         if (parentT[mpath] and next(parentT[mpath]) ~= nil) then
-            dbg.printT("parentAA",parentT[mpath])
-            sort(parentT[mpath], l_cmp)
-         end
-         t.parentAA     = parentT[mpath]
-         t.fullName     = sn
-         t.hidden       = not mrc:isVisible{fullName=sn, sn=sn, fn=v.file}
-         T[v.file]      = t
-      end
+      local kind = false
       if (next(v.fileT) ~= nil) then
          for fullName, vv in pairs(v.fileT) do
             local t = {}
@@ -641,11 +629,13 @@ function M.buildDbT(self, mpathA, mpathMapT, spiderT, dbT)
                dbg.printT("parentAA",parentT[mpath])
                sort(parentT[mpath], l_cmp)
             end
-            t.parentAA   = parentT[mpath]
-            t.mpath      = vv.mpath
-            t.fullName   = fullName
-            t.hidden     = not mrc:isVisible{fullName=fullName, sn=sn, fn=vv.fn}
-            if (not vv.dot_version) then
+            t.parentAA    = parentT[mpath]
+            t.mpath       = vv.mpath
+            t.fullName    = fullName
+            local resultT = mrc:isVisible{fullName=fullName, sn=sn, fn=vv.fn, mpathA=mpathA, mpath = vv.mpath}
+            t.hidden      = not resultT.isVisible
+            kind          = resultT.moduleKindT.kind 
+            if (not vv.dot_version and (kind ~= "hard")) then
                T[vv.fn]  = t
             end
          end
@@ -688,11 +678,11 @@ end
 function M.buildProvideByT(self, dbT, providedByT)
    dbg.start{"Spider:buildProvideByT(dbT, providedByT)"}
 
-   local show_hidden = optionTbl().show_hidden
    local mrc = MRC:singleton()
    for sn, vv in pairs(dbT) do
       for fullPath, v in pairs(vv) do
-         local hidden = not (show_hidden or mrc:isVisible{fullName=v.fullName, sn=sn, fn=fullPath})
+         local resultT = mrc:isVisible{fullName=v.fullName, sn=sn, fn=fullPath, mpath=v.mpath}
+         local hidden  = not resultT.isVisible
          if (v.provides ~= nil) then
             local providesA = v.provides
             for i = 1, #providesA do
@@ -742,21 +732,23 @@ end
 function M.Level0_terse(self,dbT, providedByT)
    dbg.start{"Spider:Level0_terse()"}
    local mrc         = MRC:singleton()
-   local optionTbl   = optionTbl()
-   local show_hidden = optionTbl.show_hidden
    local t           = {}
    local a           = {}
+
+   mrc:set_display_mode("spider")
+
    for sn, vv in pairs(dbT) do
       for fn, v in pairs(vv) do
-         local isActive, version = isActiveMFile(mrc, v.fullName, sn, fn)
-         if (show_hidden or isActive) then
+         local resultT = mrc:isVisible{fullName=v.fullName,sn=sn,fn=fn, mpath = v.mpath}
+         if (resultT.isVisible) then
+            local forbiddenT = mrc:isForbidden{fullName=v.fullName, sn=sn, fn=fn, mpath = v.mpath} 
             if (sn == v.fullName) then
-               t[sn] = sn
+               t[sn] = decorateModule(sn, resultT, forbiddenT)
             else
                -- print out directory name (e.g. gcc) for tab completion.
                t[sn]     = sn .. "/"
                local key = sn .. "/" .. v.pV
-               t[key]    = v.fullName
+               t[key]    = decorateModule(v.fullName, resultT, forbiddenT)
             end
          end
       end
@@ -814,22 +806,42 @@ local function l_case_independent_cmp_by_name(a,b)
    end
 end
 
+local function l_computeColor(resultT, forbiddenT)
+   local fT = forbiddenT 
+   if (not fT or next(fT) == nil) then
+      fT = {forbiddenState = "normal"}
+   end
+   if (fT.forbiddenState ~= "normal") then
+      return fT.forbiddenState 
+   end
+   if (resultT.moduleKindT.kind ~= "normal") then
+      return "hidden"
+   end
+   return false
+end
+
 function M.Level0Helper(self, dbT, providedByT, a)
    local t           = {}
    local optionTbl   = optionTbl()
-   local show_hidden = optionTbl.show_hidden
+   local mrc         = MRC:singleton()
+   local show_hidden = mrc:show_hidden()
    local term_width  = TermWidth() - 4
    local banner      = Banner:singleton()
-   local mrc         = MRC:singleton()
 
    for sn, vv in pairs(dbT) do
       for fn,v in pairsByKeys(vv) do
-         local isActive, version = isActiveMFile(mrc, v.fullName, sn, fn)
-         if (show_hidden or isActive) then
+         local resultT = mrc:isVisible{fullName=v.fullName,sn=sn,fn=fn, mpath = v.mpath}
+         if (resultT.isVisible) then
+            local forbiddenT = mrc:isForbidden{fullName=v.fullName,sn=sn,fn=fn, mpath = v.mpath}
             if (t[sn] == nil) then
                t[sn] = { Description = v.Description, versionA = { }, name = sn}
             end
-            t[sn].versionA[v.pV] = v.fullName
+            local color = l_computeColor(resultT, forbiddenT)
+            dbg.print{"fullName: ",v.fullName,", color: ",color,"\n"}
+            dbg.printT("resultT",resultT)
+            dbg.printT("forbiddenT",forbiddenT or {})
+            
+            t[sn].versionA[v.pV] = colorize(color, v.fullName)
          end
       end
    end
@@ -903,9 +915,12 @@ end
 
 function M.spiderSearch(self, dbT, providedByT, userSearchPat, helpFlg)
    dbg.start{"Spider:spiderSearch(dbT,providedByT,\"",userSearchPat,"\",",helpFlg,")"}
-   local optionTbl   = optionTbl()
-   local show_hidden = optionTbl.show_hidden
    local mrc         = MRC:singleton()
+   local optionTbl   = optionTbl()
+
+   mrc:set_display_mode("spider")
+
+   local show_hidden = mrc:show_hidden()
 
    dbg.print{"show_hidden: ",show_hidden,"\n"}
 
@@ -947,7 +962,8 @@ function M.spiderSearch(self, dbT, providedByT, userSearchPat, helpFlg)
          if (T) then
             dbg.print{"Have T\n"}
             for fn, v in pairs(T) do
-               if (mrc:isVisible{fullName=v.fullName,fn=fn,sn=origUserSearchPat}) then
+               local resultT = mrc:isVisible{fullName=v.fullName,fn=fn,sn=origUserSearchPat, mpath=v.mpath}
+               if (resultT.isVisible) then
                   found = true
                   break
                end
@@ -981,7 +997,8 @@ function M.spiderSearch(self, dbT, providedByT, userSearchPat, helpFlg)
       local fullA = {}
       for sn, vv in pairs(dbT) do
          for fn, v in pairs(vv) do
-            if (show_hidden or mrc:isVisible{fullName=v.fullName,sn=sn,fn=fn}) then
+            local resultT = mrc:isVisible{fullName=v.fullName,sn=sn,fn=fn, mpath = v.mpath}
+            if (resultT.isVisible) then
                 fullA[#fullA+1] = {sn=sn, fullName=v.fullName}
             end
          end
@@ -1029,7 +1046,6 @@ function M.spiderSearch(self, dbT, providedByT, userSearchPat, helpFlg)
    end
 
    if (next(matchT) == nil) then
-      setWarningFlag()
       LmodSystemError{msg="e_Failed_2_Find", name=origUserSearchPat}
    end
 
@@ -1063,9 +1079,9 @@ end
 function M._Level1(self, dbT, providedByT, possibleA, sn, key, helpFlg)
    dbg.start{"Spider:_Level1(dbT, providedByT, possibleA, sn: \"",sn,"\", key: \"",key,"\")"}
    local optionTbl   = optionTbl()
-   local show_hidden = optionTbl.show_hidden
    local term_width  = TermWidth() - 4
    local mrc         = MRC:singleton()
+   local show_hidden = mrc:show_hidden()
    local T           = dbT[sn]
    local TT          = providedByT[sn]
    local tailMsg     = nil
@@ -1092,7 +1108,8 @@ function M._Level1(self, dbT, providedByT, possibleA, sn, key, helpFlg)
          dbg.print{"Have T in l_countEntries\n"}
          dbg.print{"key: ",key,"\n"}
          for fn, v in pairs(T) do
-            if (show_hidden or mrc:isVisible{fullName=v.fullName,sn=sn,fn=fn}) then
+            local resultT = mrc:isVisible{fullName=v.fullName,sn=sn,fn=fn, mpath = v.mpath}
+            if (resultT.isVisible) then
                v.fn=fn
                if (v.fullName == key) then
                   aa[#aa + 1] = v
@@ -1211,8 +1228,9 @@ function M._Level1(self, dbT, providedByT, possibleA, sn, key, helpFlg)
    if (T) then
       dbg.print{"Have T\n"}
       for fn, v in pairsByKeys(T) do
-         local isActive, version = isActiveMFile(mrc, v.fullName, sn, fn)
-         if (show_hidden or isActive) then
+         local resultT = mrc:isVisible{fullName=v.fullName, sn=sn, fn=fn, mpath = v.mpath}
+         if (resultT.isVisible) then
+            local version  = extractVersion(v.fullName, sn)
             local kk       = sn .. "/" .. parseVersion(version)
             if (fullVT[kk] == nil) then
                key         = sn
@@ -1322,7 +1340,10 @@ function M._Level2(self, sn, fullName, entryA, entryPA, possibleA, tailMsg)
    dbg.start{"Spider:_Level2(\"",sn,"\", \"",fullName,"\", entryA, entryPA, possibleA, tailMsg)"}
    --dbg.printT("entryA",entryA)
 
-   local show_hidden  = optionTbl().show_hidden
+   local optionTbl    = optionTbl()
+   local mrc          = MRC:singleton();    mrc:set_display_mode("spider")
+   local show_hidden  = mrc:show_hidden()
+   local terse        = optionTbl.terse
    local a            = {}
    local ia           = 0
    local b            = {}
@@ -1428,13 +1449,19 @@ function M._Level2(self, sn, fullName, entryA, entryPA, possibleA, tailMsg)
                d[k] = 1
             end
          end
+         local x = {}
          c = {}
          for k in pairs(d) do
             c[#c+1] = k
+            x[#x+1] = k:gsub("^ *","")
          end
          table.sort(c)
          c[#c+1] = " "
          ia = ia + 1; a[ia] = concatTbl(c,"\n")
+         if (terse) then
+            table.sort(x)
+            return concatTbl(x,"\n")
+         end
       end
    end
 
