@@ -250,6 +250,8 @@ function main()
    setenv("MODULESHOME",            "")
    setenv("MODULEPATH",             "")
    setenv("MODULEPATH_ROOT",        "")
+   setenv("LMOD_MODULERCFILE",      "")   
+   setenv("LMOD_MODULERC",          "")   
    setenv("MODULERCFILE",           "")   
    setenv("TARG_TITLE_BAR_PAREN",   "")   
    setenv("__Init_Default_Modules", "")
