@@ -1,4 +1,4 @@
---------------------------------------------------------------------------
+-------------------------------------------------------------------------
 -- The all the user sub-commands are implemented here.
 -- @module cmdfuncs
 
@@ -53,6 +53,7 @@ local ColumnTable  = require('ColumnTable')
 local FrameStk     = require('FrameStk')
 local Hub          = require('Hub')
 local MName        = require("MName")
+local MRC          = require("MRC")
 local Spider       = require("Spider")
 local Version      = require("Version")
 local concatTbl    = table.concat
@@ -65,9 +66,7 @@ local lfs          = require("lfs")
 local sort         = table.sort
 local pack         = (_VERSION == "Lua 5.1") and argsPack or table.pack  -- luacheck: compat
 local unpack       = (_VERSION == "Lua 5.1") and unpack or table.unpack  -- luacheck: compat
-local s_purgeFlg   = false
 
-local system_name  = cosmic:value("LMOD_SYSTEM_NAME")
 
 --------------------------------------------------------------------------
 -- Both Help and Whatis functions funnel their actions through
@@ -76,7 +75,7 @@ local system_name  = cosmic:value("LMOD_SYSTEM_NAME")
 -- depending on what mode Access is called with.
 -- @param mode Whether this function has be called via *Help* or *Whatis*.
 local function l_Access(mode, ...)
-   local hub    = Hub:singleton()
+   local hub       = Hub:singleton()
    local shell     = _G.Shell
    local optionTbl = optionTbl()
    dbg.start{"l_Access(", concatTbl({...},", "),")"}
@@ -102,7 +101,8 @@ end
 -- @param a An array containing the results.
 -- @param path The Lmod.d directory path.
 local function l_findNamedCollections(a,pathA)
-   local t = {}
+   local system_name  = cosmic:value("LMOD_SYSTEM_NAME")
+   local t            = {}
    for i = 1,#pathA do
       repeat 
          local path = pathA[i]
@@ -152,7 +152,9 @@ end
 function Category(...)
    dbg.start{"Category(", concatTbl({...},", "),")"}
    local shell = _G.Shell
+   local mrc   = MRC:singleton()
 
+   mrc:set_display_mode("spider")
 
    local cache = Cache:singleton{buildCache = true}
    local moduleT, dbT = cache:build()
@@ -304,22 +306,28 @@ function Help(...)
 end
 
 function IsAvail(...)
+   local mrc  = MRC:singleton()
+   mrc:set_display_mode("avail")
+
    local argA = pack(...)
    for i = 1, argA.n do
       local mname = MName:new("load", argA[i])
       if (not mname:valid()) then
-         setWarningFlag()
+         setStatusFlag()
          break
       end
    end
 end
 
 function IsLoaded(...)
+   local mrc  = MRC:singleton()
+   mrc:set_display_mode("avail")
+
    local argA = pack(...)
    for i = 1, argA.n do
       local mname = MName:new("mt", argA[i])
       if (not mname:isloaded()) then
-         setWarningFlag()
+         setStatusFlag()
          break
       end
    end
@@ -331,6 +339,7 @@ end
 function Keyword(...)
    dbg.start{"Keyword(",concatTbl({...},","),")"}
 
+   local mrc                    = MRC:singleton(); mrc:set_display_mode("spider")
    local banner                 = Banner:singleton()
    local border                 = banner:border(0)
    local shell                  = _G.Shell
@@ -369,16 +378,18 @@ function List(...)
    local shell     = _G.Shell
    local frameStk  = FrameStk:singleton()
    local mt        = frameStk:mt()
-
    local kind      = optionTbl.brief and "fullName_Meta" or "fullName"
-
    local activeA   = mt:list(kind,"active")
    local inactiveA = mt:list(kind,"inactive")
    local total     = #activeA + #inactiveA
    local cwidth    = optionTbl.rt and LMOD_COLUMN_TABLE_WIDTH or TermWidth()
+   local mrc       = MRC:singleton()
+
+   mrc:set_display_mode("list")
 
    dbg.print{"#activeA:   ",#activeA,"\n"}
    dbg.print{"#inactiveA: ",#inactiveA,"\n"}
+   dbg.print{"kind:       ",kind,"\n"}
 
    activeA = hook.apply("listHook",activeA) or activeA
 
@@ -408,12 +419,23 @@ function List(...)
    end
 
    if (optionTbl.terse) then
+      dbg.printT("activeA",activeA)
       for i = 1,#activeA do
-         local fullName = activeA[i].fullName
+         local entry = activeA[i]
+         local s = entry.fullName
+         if (entry.origUserName) then
+            s = s .. "\n" .. entry.origUserName
+         end
+         dbg.print{"fullName: ",entry.fullName, ", orig: ",entry.origUserName,", s: ",s,"\n"}
+         
          for j = 1, wanted.n do
             local p = wanted[j]
-            if (fullName:find(p)) then
-               shell:echo(fullName.."\n")
+            if (s:find(p)) then
+               local aa = {}
+               dbg.printT("entry",entry)
+               aa[#aa+1] = decorateModule(s, entry, entry.forbiddenT)
+               aa[#aa+1] = "\n"
+               shell:echo(concatTbl(aa,""))
             end
          end
       end
@@ -421,21 +443,29 @@ function List(...)
       return
    end
 
-   b[#b+1]       = "\n"
-   b[#b+1]       = msg
-   b[#b+1]       = msg2
-   b[#b+1]       = "\n"
-   local kk      = 0
-   local legendT = {}
+   b[#b+1]            = "\n"
+   b[#b+1]            = msg
+   b[#b+1]            = msg2
+   b[#b+1]            = "\n"
+   local kk           = 0
+   local legendT      = {}
+   local show_hidden  = mrc:show_hidden()
+   local have_hiddenL = false
 
    for i = 1, #activeA do
       local entry    = activeA[i]
       local fullName = entry.fullName
+      local origName = entry.origUserName or ""
+      local showMe   = true
+      if (entry.moduleKindT.hidden_loaded) then
+         showMe       = show_hidden
+         have_hiddenL = not show_hidden
+      end
       for j = 1, wanted.n do
          local p = wanted[j]
-         if (fullName:find(p)) then
+         if (showMe and (fullName:find(p) or origName:find(p))) then
             kk = kk + 1
-            a[#a + 1] = mt:list_property(kk, entry.sn, "short", legendT)
+            a[#a + 1] = mt:list_w_property(kk, entry.sn, "short", legendT)
          end
       end
    end
@@ -461,6 +491,10 @@ function List(...)
       b[#b+1] = bt:build_tbl()
       b[#b+1] = "\n"
    end
+   if (have_hiddenL) then
+      b[#b+1] = i18n("m_Hidden_loaded")
+   end
+
    a = {}
    kk = 0
 
@@ -509,8 +543,10 @@ end
 -- it won't get the swap message.
 
 local function l_usrLoad(argA, check_must_load)
-   local frameStk = FrameStk:singleton()
    dbg.start{"l_usrLoad(argA, check_must_load: ",check_must_load,")"}
+   local frameStk = FrameStk:singleton()
+   local mrc      = MRC:singleton()
+   mrc:set_display_mode("all")
    local uA   = {}
    local lA   = {}
    for i = 1, argA.n do
@@ -593,26 +629,8 @@ end
 -- Unload all loaded modules.
 -- @param force If true then sticky modules are unloaded as well.
 function Purge(force)
-   local frameStk = FrameStk:singleton()
-   local mt       = frameStk:mt()
-   local totalA   = mt:list("short","any")
-
-   if (#totalA < 1) then
-      return
-   end
-
-   local mA = {}
-   for i = #totalA,1,-1 do
-      mA[#mA+1] = MName:new("mt",totalA[i])
-   end
-   dbg.start{"Purge(",concatTbl(totalA,", "),")"}
-   s_purgeFlg = true
-   unload_usr_internal(mA, force)
-   s_purgeFlg = false
-
-   -- A purge should not set the warning flag.
-   clearWarningFlag()
-   dbg.print{"warningFlag: ", getWarningFlag(),"\n"}
+   dbg.start{"Purge(force = ",force,")"}
+   mcp:purge{force=force}
    dbg.fini("Purge")
 end
 
@@ -756,14 +774,15 @@ end
 function CollectionLst(collection)
    collection  = collection or "default"
    dbg.start{"CollectionLst(",collection,")"}
-   local optionTbl = optionTbl()
-   local sname     = (not system_name) and "" or "." .. system_name
-   local mt        = FrameStk:singleton():mt()
-   local shell     = _G.Shell
-   local cwidth    = optionTbl.rt and LMOD_COLUMN_TABLE_WIDTH or TermWidth()
-   local path      = l_find_a_collection(collection)
+   local system_name = cosmic:value("LMOD_SYSTEM_NAME")
+   local optionTbl   = optionTbl()
+   local sname       = (not system_name) and "" or "." .. system_name
+   local mt          = FrameStk:singleton():mt()
+   local shell       = _G.Shell
+   local cwidth      = optionTbl.rt and LMOD_COLUMN_TABLE_WIDTH or TermWidth()
+   local path        = l_find_a_collection(collection)
 
-   local a         = mt:reportContents{fn=path, name=collection}
+   local a           = mt:reportContents{fn=path, name=collection}
    if (optionTbl.terse) then
       for i = 1,#a do
          shell:echo(a[i].."\n")
@@ -793,8 +812,9 @@ end
 -- and will be removed.  Use restore instead.
 -- @param collection The collection name (default="default")
 function GetDefault(collection)
-   collection  = collection or "default"
+   collection        = collection or "default"
    dbg.start{"GetDefault(",collection,")"}
+   local system_name = cosmic:value("LMOD_SYSTEM_NAME")
 
    local sname = (not system_name) and "" or "." .. system_name
    local path  = l_find_a_collection(collection .. sname)
@@ -818,9 +838,10 @@ function Restore(collection)
 
    local msg
    local path
-   local myName  = "default"
-   local sname   = system_name
-   local msgTail = ""
+   local system_name = cosmic:value("LMOD_SYSTEM_NAME")
+   local myName      = "default"
+   local sname       = system_name
+   local msgTail     = ""
    if (not sname) then
       sname   = ""
       myName  = "(empty)"
@@ -866,6 +887,9 @@ function Restore(collection)
    if (collection == "system" ) then
       Reset(msg)
    else
+      local mrc     = MRC:singleton()
+      mrc:set_display_mode("all")
+
       local mt      = FrameStk:singleton():mt()
       local results = mt:getMTfromFile{fn=path, name=myName, msg=msg}
       if (not results and collection == "default") then
@@ -900,8 +924,9 @@ function Save(...)
    local pathA     = l_collectionDir("write")
    dbg.start{"Save(",concatTbl({...},", "),")"}
 
-   local msgTail = ""
-   local sname   = system_name
+   local system_name = cosmic:value("LMOD_SYSTEM_NAME")
+   local msgTail     = ""
+   local sname       = system_name
    if (not sname) then
       sname   = ""
    else
@@ -966,14 +991,15 @@ end
 --------------------------------------------------------------------------
 -- Report to the user all the named collections he/she has.
 function SaveList(...)
-   local mt        = FrameStk:singleton():mt()
-   local optionTbl = optionTbl()
-   local a         = {}
-   local b         = {}
-   local shell     = _G.Shell
-   local cwidth    = optionTbl.rt and LMOD_COLUMN_TABLE_WIDTH or TermWidth()
-   local home      = os.getenv("HOME")
-   local pathA     = l_collectionDir("read")
+   local mt          = FrameStk:singleton():mt()
+   local optionTbl   = optionTbl()
+   local a           = {}
+   local b           = {}
+   local shell       = _G.Shell
+   local cwidth      = optionTbl.rt and LMOD_COLUMN_TABLE_WIDTH or TermWidth()
+   local home        = os.getenv("HOME")
+   local pathA       = l_collectionDir("read")
+   local system_name = cosmic:value("LMOD_SYSTEM_NAME")
 
    l_findNamedCollections(b,pathA)
    if (optionTbl.terse) then
@@ -1048,6 +1074,7 @@ end
 -- level 1 or level 2 report on particular modules.
 function SpiderCmd(...)
    dbg.start{"SpiderCmd(", concatTbl({...},", "),")"}
+   local mrc                    = MRC:singleton() ; mrc:set_display_mode("spider")
    local cache                  = Cache:singleton{buildCache=true}
    local shell                  = _G.Shell
    local optionTbl              = optionTbl()
@@ -1150,10 +1177,11 @@ end
 --------------------------------------------------------------------------
 -- Disable a collection
 function Disable(...)
-   local shell = _G.Shell
-   local pathA = l_collectionDir("read")
-   local argA  = pack(...)
-   local sname = (not system_name) and "" or "." .. system_name
+   local shell       = _G.Shell
+   local pathA       = l_collectionDir("read")
+   local argA        = pack(...)
+   local system_name = cosmic:value("LMOD_SYSTEM_NAME")
+   local sname       = (not system_name) and "" or "." .. system_name
 
    if (argA.n == 0) then
       argA[1] = "default"
@@ -1187,16 +1215,19 @@ end
 --  possibly reloaded if a module.
 function Use(...)
    dbg.start{"Use(", concatTbl({...},", "),")"}
-   local mt  = FrameStk:singleton():mt()
-   local a = {}
-   local mcp_old = mcp
-   local mcp     = MCP
-   local op = mcp.prepend_path
+   local mt       = FrameStk:singleton():mt()
+   local a        = {}
+   local mcp_old  = mcp
+   local mcp      = MCP
+   local op       = mcp.prepend_path
 
    local argA     = pack(...)
    local iarg     = 1
    local priority = 0
 
+   local mrc      = MRC:singleton()
+   mrc:set_display_mode("all")
+
    dbg.print{"using mcp: ",mcp:name(), "\n"}
 
    while (iarg <= argA.n) do
@@ -1213,10 +1244,15 @@ function Use(...)
       iarg = iarg + 1
    end
    for _,v in ipairs(a) do
+      -- Produce warning if leading minus sign(s) are found.
+      if (v:find("^-+")) then
+         LmodWarning{msg="w_Possible_Bad_Dir",dir=v}
+      end
+
       if (v:sub(1,1) ~= '/') then
          local old = v
          -- If relative convert to try to convert to absolute path
-         v          = abspath(v)
+         v         = realpath(v)
          -- If it doesn't exist then build path with current directory and relative path.
          if (not v) then
             v = pathJoin(posix.getcwd(), old)
@@ -1248,6 +1284,7 @@ function UnUse(...)
       MCP:remove_path{ModulePath,  v, delim=":", nodups = true, force = true}
    end
    if (mt:changeMPATH()) then
+      local mrc = MRC:singleton(); mrc:set_display_mode("all")
       mt:reset_MPATH_change_flag()
       hub.reloadAll()
    end
