@@ -133,42 +133,43 @@ function main()
    --           number of arguments the option requires
 
    local lmodOptT = {
-      ["--old_style"] = 0,
       ["-D"]=0,
       ["-?"] = 0, ["-h"] = 0, ["--help"] = 0, ["-H"] = 0,
-      ["--default"]=0,  ["-d"]=0,
       ["--brief"]=0,  ["-b"]=0,
+      ["--config"] = 0,
+      ["--config_json"] = 0, ["--config-json"] = 0,
+      ["--default"]=0,  ["-d"]=0,
       ["--dumpversion"] = 0,
+      ["--dumpname"] = 0,
       ["--expert"]=0,   ["--novice"]=0,
-      ["--force"] = 0,
+      ["--force"] = 0,  ["-f"] = 0,
       ["--gitversion"] = 0,
       ["--ignore_cache"] = 0,  ["--ignore-cache"] = 0, ["-I"] = 0,
       ["--initial_load"] = 0,  ["--initial-load"] = 0,
       ["--latest"] = 0,
       ["--localvar"]=1,
       ["--location"]=0,  ["--loc"] = 0,
-      ["--pin_versions"]=0, ["--pin-versions"]=0,
+      ["--miniConfig"] = 0,
       ["--mt"] = 0,
+      ["--old_style"] = 0,
+      ["--pin_versions"]=0, ["--pin-versions"]=0,
+      ["--pod"] = 0, 
       ["--quiet"] = 0,  ["-q"] = 0,
+      ["--raw"] = 0,
       ["--redirect"] = 0, ["--no_redirect"] = 0, ["--no-redirect"] = 0,
+      ["--regexp"] = 0, ["-r"] = 0,
+      ["--show_hidden"] = 0, ["--show-hidden"] = 0, ["--all"] = 0, ["-A"] = 0,
       ["--spider_timeout"] = 1,       ["--spider-timeout"] = 1,
+      ["--style"] = 1,  ["-s"] = 1,
       ["--terse"] = 0,  ["-t"] = 0,
+      ["--terse_show_extensions"] = 0, 
       ["--timer"] = 0,
       ["--trace"] = 0,  ["-T"] = 0,
       ["--version"]=0,  ["--versoin"]=0, ["--ver"]=0, ["--v"]=0, ["-v"]=0,
-      ['--config'] = 0,
-      ['--miniConfig'] = 0,
-      ['--config_json'] = 0, ['--config-json'] = 0,
-      ['--raw'] = 0,
-      ['--regexp'] = 0, ['-r'] = 0,
-      ['--show_hidden'] = 0, ['--show-hidden'] = 0,
-      ['--style'] = 1,  ['-s'] = 1,
-      ['--width'] = 1,  ['-w'] = 1,
+      ["--width"] = 1,  ["-w"] = 1,
    }
 
    local translateT = {
-      ["-I"]="--ignore_cache",
-      ["--loc"]="--location",
       ["--versoin"]="--version",
       ["--ver"]="--version",
       ["--v"]="--version",
@@ -292,8 +293,9 @@ function main()
 
          local cmd = lmodCmdT[v]
          if (cmd and not cmdFound) then
-            cmdA[#cmdA + 1] = cmd
-            cmdFound        = true
+            cmdA[#cmdA + 1]      = cmd
+            cmdFound             = true
+            stop_processing_opts = true
             break
          end
 
