@@ -62,6 +62,7 @@ require("declare")
 
 local BeautifulTbl = require("BeautifulTbl")
 local MName        = require("MName")
+local MRC          = require("MRC")
 local Version      = require("Version")
 local dbg          = require("Dbg"):dbg()
 local hook         = require("Hook")
@@ -163,7 +164,7 @@ end
 -- @param cmdName The command which is getting its arguments validated.
 local function l_validateModules(cmdName, ...)
    local argA = pack(...)
-   dbg.print{"cmd: ",cmdName, " argA.n: ",argA.n,"\n"}
+   --dbg.print{"l_validateModules: cmd: ",cmdName, " argA.n: ",argA.n,"\n"}
    local allGood = true
    local fn      = false
    for i = 1, argA.n do
@@ -215,6 +216,8 @@ function load_any(...)
    return b
 end   
 
+local s_cleanupDirT = { PATH = true, LD_LIBRARY_PATH = true, LIBRARY_PATH = true, MODULEPATH = true }
+
 --- PATH functions ---
 --------------------------------------------------------------------------
 -- convert arguments into a table if necessary.
@@ -224,16 +227,31 @@ local function l_convert2table(...)
 
    if (argA.n == 1 and type(argA[1]) == "table" ) then
       t = argA[1]
-      t[1] = t[1]:trim()
+      t[1] = t[1]
    else
-      t[1]    = argA[1]:trim()
+      t[1]    = argA[1]
       t[2]    = argA[2]
       t.delim = argA[3]
    end
+   
    t.priority = tonumber(t.priority or "0")
    return t
 end
 
+local function l_cleanupPathArgs(t)
+   local name = t[1]:trim()
+   local path = t[2]:trim()
+   
+   if (s_cleanupDirT[name]) then
+      path = path:gsub(":+$",""):gsub("^:+",""):gsub(":+",":")
+      if (path == "") then path = false end
+      t[2] = path
+   end
+
+   return
+end
+
+
 --------------------------------------------------------------------------
 -- Prepend a value to a path like variable.
 function prepend_path(...)
@@ -241,7 +259,8 @@ function prepend_path(...)
    dbg.start{"prepend_path(",l_concatTbl(t,", "),")"}
    if (not l_validateStringTable(2, "prepend_path",t)) then return end
 
-   mcp:prepend_path(t)
+   l_cleanupPathArgs(t)
+   if (t[2]) then mcp:prepend_path(t) end
    dbg.fini("prepend_path")
 end
 
@@ -251,8 +270,8 @@ function append_path(...)
    local t = l_convert2table(...)
    dbg.start{"append_path(",l_concatTbl(t,", "),")"}
    if (not l_validateStringTable(2, "append_path",t)) then return end
-
-   mcp:append_path(t)
+   l_cleanupPathArgs(t)
+   if (t[2]) then mcp:append_path(t) end
    dbg.fini("append_path")
 end
 
@@ -262,8 +281,8 @@ function remove_path(...)
    local t = l_convert2table(...)
    dbg.start{"remove_path(",l_concatTbl(t,", "),")"}
    if (not l_validateStringTable(2, "remove_path",t)) then return end
-
-   mcp:remove_path(t)
+   l_cleanupPathArgs(t)
+   if (t[2]) then mcp:remove_path(t) end
    dbg.fini("remove_path")
 end
 
@@ -509,6 +528,7 @@ userInGroup = userInGroups
 --------------------------------------------------------------------------
 -- Convert version to canonical so that it can be used in a comparison.
 function convertToCanonical(s)
+   if (not l_validateStringArgs("convertToCanonical",s)) then return end
    return parseVersion(s)
 end
 
@@ -522,7 +542,7 @@ function prereq(...)
    dbg.start{"prereq(",l_concatTbl({...},", "),")"}
    if (not l_validateModules("prereq", ...)) then return end
 
-   mcp:prereq(MName:buildA("mt", ...))
+   mcp:prereq(MName:buildA(mcp:MNamePrereqType(), ...))
    dbg.fini("prereq")
 end
 
@@ -533,22 +553,22 @@ function prereq_any(...)
    dbg.start{"prereq_any(",l_concatTbl({...},", "),")"}
    if (not l_validateModules("prereq_any",...)) then return end
 
-   mcp:prereq_any(MName:buildA("mt",...))
-   dbg.fini("conflict")
+   mcp:prereq_any(MName:buildA(mcp:MNamePrereqType(),...))
+   dbg.fini("prereq_any")
 end
 
 --------------------------------------------------------------------------
 -- Test to see if a conflict module is loaded.  Fail if it is loaded.
 function conflict(...)
    dbg.start{"conflict(",l_concatTbl({...},", "),")"}
-   if (not l_validateStringArgs("conflict",...)) then return end
+   if (not l_validateModules("conflict",...)) then return end
 
    mcp:conflict(MName:buildA("mt",...))
-   dbg.fini()
+   dbg.fini("conflict")
 end
 
 --------------------------------------------------------------------------
--- A load and prereq modifier.  It is used like this:
+-- A load, prereq and conflict modifier.  It is used like this:
 -- load(atleast("gcc","4.8"))
 -- @param m module name
 -- @param is the starting version
@@ -561,6 +581,20 @@ function atleast(m, is)
    return mname
 end
 
+--------------------------------------------------------------------------
+-- A load, prereq and conflict modifier.  It is used like this:
+-- load(atleast("gcc","4.8"))
+-- @param m module name
+-- @param is the starting version
+function atmost(m, ie)
+   dbg.start{"atmost(",m,", ",ie,")"}
+
+   local mname = MName:new("load", m, "atmost", false, ie)
+
+   dbg.fini("atmost")
+   return mname
+end
+
 --------------------------------------------------------------------------
 -- A load and prereq modifier.  It is used like this:
 -- load(between("gcc","4.8","4.9"))
@@ -765,6 +799,9 @@ function unload_usr_internal(mA, force)
       local s = mAList(mA)
       dbg.start{"unload_usr_internal(mA={"..s.."},force=",force,")"}
    end
+   local mrc = MRC:singleton()
+   mrc:set_display_mode("all")
+
    local mcp_old = mcp
    mcp = MainControl.build("unload")
    local b = MainControl.unload_usr(mcp, mA, force)
@@ -779,6 +816,9 @@ function unload_internal(mA)
       local s = mAList(mA)
       dbg.start{"unload_internal(mA={"..s.."})"}
    end
+   local mrc = MRC:singleton()
+   mrc:set_display_mode("all")
+
    local mcp_old = mcp
    mcp = mcp:build_unload()
    local b = mcp:unload(mA)
@@ -811,9 +851,6 @@ function always_load(...)
    return b
 end
 
-
-
-
 --------------------------------------------------------------------------
 -- This function always unloads and never loads. The reverse of this
 -- function is a no-op.
@@ -832,6 +869,14 @@ function depends_on(...)
    dbg.fini("depends_on")
 end
 
+function depends_on_any(...)
+   dbg.start{"depends_on_any(",l_concatTbl({...},", "),")"}
+   if (not l_validateModules("depends_on_any",...)) then return {} end
+
+   local b = mcp:depends_on_any(MName:buildA(mcp:MNameType(),...))
+   dbg.fini("depends_on_any")
+end
+
 function extensions(...)
    dbg.start{"extensions(",l_concatTbl({...},", "),")"}
    if (not l_validateStringArgs("extensions",...)) then return {} end
@@ -868,11 +913,22 @@ end
 
 function LmodBreak(msg)
    dbg.start{"LmodBreak(",msg,")"}
-   if (not l_validateStringArgs("LmodBreak", msg)) then return end
+   if (not (msg == nil or l_validateStringArgs("LmodBreak", msg))) then return end
    mcp:LmodBreak(msg)
    dbg.fini("LmodBreak")
 end
 
+function purge(t)
+   t = t or {}
+   dbg.start{"purge{force=",t.force,"}"}
+   mcp:purge(t)
+   dbg.fini("purge")
+end
+
+function dofile_not_supported()
+   mcp:report{msg="e_Dofile_not_supported"}
+end
+
 
 --- subprocess function ---
 
