@@ -4,7 +4,6 @@
 
 _G._DEBUG          = false               -- Required by the new lua posix
 local posix        = require("posix")
-
 require("strict")
 
 --------------------------------------------------------------------------
@@ -44,9 +43,11 @@ require("strict")
 require("declare")
 require("fileOps")
 
+local Version      = require("Version")
 local cosmic       = require("Cosmic"):singleton()
 local lfs          = require("lfs")
 local getenv       = os.getenv
+local access       = posix.access
 local setenv_posix = posix.setenv
 
 if (isNotDefined("cmdDir")) then
@@ -59,6 +60,14 @@ end
 
 LuaV = (_VERSION:gsub("Lua ",""))
 
+------------------------------------------------------------------------
+-- Lmod branch
+------------------------------------------------------------------------
+cosmic:init{name    = "LMOD_BRANCH",
+            default = "main",
+            assignV = Version.branch()}
+            
+
 ------------------------------------------------------------------------
 -- Lmod ExitHookArray Object:
 ------------------------------------------------------------------------
@@ -72,14 +81,28 @@ ExitHookA = require("HookArray")
 setenv_posix("LC_ALL","C",true)
 
 ------------------------------------------------------------------------
--- MODULEPATH_INIT: Name of the file that can be used to initialize
---                  MODULEPATH in the startup scripts
+-- LMOD_MODULEPATH_INIT: Name of the file that can be used to initialize
+--                       MODULEPATH in the startup scripts
 ------------------------------------------------------------------------
+local mpath_init = getenv("LMOD_MODULEPATH_INIT")
+local default_mpath_init = "@PKG@/init/.modulespath"
+local global_mpath_init  = "/etc/lmod/.modulespath"
+if (not mpath_init) then
+   if (access(global_mpath_init, "r")) then
+      mpath_init = global_mpath_init
+   else
+      mpath_init = "@modulepath_init@"
+      if (mpath_init:sub(1,1) == "@") then
+         mpath_init = default_mpath_init
+      end
+   end
+   
+end
 
 cosmic:init{name    = "LMOD_MODULEPATH_INIT",
             sedV    = "@modulepath_init@",
-            no_env  = true,
-            default = "@PKG@/init/.modulespath"}
+            default = default_mpath_init,
+            assignV = mpath_init}
 
 ------------------------------------------------------------------------
 -- SITE_CONTROLLED_PREFIX: If a site configured lmod with direct prefix
@@ -156,6 +179,17 @@ cosmic:init{name = "LMOD_REDIRECT",
             yn   = "no"}
 
 ------------------------------------------------------------------------
+-- LMOD_DOWNSTREAM_CONFLICTS: Module confiicts are remember for later
+--                            module loads
+--                            Note: this variable can only be set
+--                            at config time or via cosmic:assign() at
+--                            startup
+------------------------------------------------------------------------
+cosmic:init{name    = "LMOD_DOWNSTREAM_CONFLICTS",
+            sedV    = "@lmod_downstream_conflicts@",
+            default = "no",
+            assignV = "no"}
+------------------------------------------------------------------------
 -- LMOD_RC:  Lmod RC list of colon separated files 
 ------------------------------------------------------------------------
 local rcfiles = getenv("LMOD_RC")
@@ -192,7 +226,7 @@ cosmic:init{name    = "LMOD_SITEPACKAGE_LOCATION",
 
 
 ------------------------------------------------------------------------
--- LMOD_CFG:  lmod_config.lua locatoin
+-- LMOD_CFG:  lmod_config.lua location
 ------------------------------------------------------------------------
 cosmic:init{name    = "LMOD_CONFIG_LOCATION",
             default = "no"}
@@ -212,6 +246,13 @@ cosmic:init{name    = "LMOD_CONFIG_DIR",
             sedV    = "@lmod_config_dir@",
             default = "/etc/lmod"}
 
+------------------------------------------------------------------------
+-- LMOD_TERSE_DECORATIONS: Allow terse listing to be decorated
+------------------------------------------------------------------------
+cosmic:init{name    = "LMOD_TERSE_DECORATIONS",
+            sedV    = "@terse_decorations@",
+            default = "yes"}
+
 ------------------------------------------------------------------------
 -- LMOD_PACKAGE_PATH: Colon separated list of directories to search for
 --                    SitePackage.lua
@@ -328,16 +369,11 @@ cosmic:init{name = "LMOD_DUPLICATE_PATHS",
             yn   = "no"}
 
 
-if (cosmic:value("LMOD_TMOD_PATH_RULE") == "yes") then
-   cosmic:assign("LMOD_DUPLICATE_PATHS", "no")
-end
-
 ------------------------------------------------------------------------
 -- LMOD_IGNORE_CACHE:  Ignore user and system caches and rebuild if needed
 ------------------------------------------------------------------------
-cosmic:init{name    = "LMOD_IGNORE_CACHE",
-            lower   = true,
-            default = false}
+cosmic:init{name = "LMOD_IGNORE_CACHE",
+            yn   = "no"}
 
 ------------------------------------------------------------------------
 -- LMOD_CACHED_LOADS: Use spider cache on loads
@@ -346,11 +382,6 @@ cosmic:init{name = "LMOD_CACHED_LOADS",
             sedV = "@cached_loads@",
             yn   = "no"}
 
-local ignore_cache = cosmic:value("LMOD_IGNORE_CACHE")
-local cached_loads = cosmic:value("LMOD_CACHED_LOADS")
-
-cosmic:assign("LMOD_CACHED_LOADS",ignore_cache and "no" or cached_loads)
-
 ------------------------------------------------------------------------
 -- LMOD_PAGER: Lmod will use this value of pager if set.
 ------------------------------------------------------------------------
@@ -384,8 +415,10 @@ local rc_dflt    = pathJoin(etcDir,"rc.lua")
 if (not isFile(rc_dflt)) then
    rc_dflt   = pathJoin(etcDir,"rc")
 end
-local rc        = getenv("LMOD_MODULERCFILE") or getenv("MODULERCFILE")
-cosmic:init{name    = "LMOD_MODULERCFILE",
+local rc        = getenv("LMOD_MODULERC") or 
+                  getenv("LMOD_MODULERCFILE") or
+                  getenv("MODULERCFILE")
+cosmic:init{name    = "LMOD_MODULERC",
             default = rc_dflt,
             envV    = rc,
             assignV = rc,
@@ -415,6 +448,20 @@ LMOD_QUARANTINE_VARS = getenv("LMOD_QUARANTINE_VARS")
 ------------------------------------------------------------------------
 LMOD_RTM_TESTING = getenv("LMOD_RTM_TESTING")
 
+
+------------------------------------------------------------------------
+-- LMOD_SHOW_HIDDEN: Controlling when hidden modules are shown.
+--                   Possible values are all, avail, list, spider
+--                   Default value is none, meaning that all hidden modules
+--                   are not shown
+------------------------------------------------------------------------
+local show_hiddenStr = getenv("LMOD_SHOW_HIDDEN")
+
+cosmic:init{name    = "LMOD_SHOW_HIDDEN",
+            default = "no",
+            envV    = show_hiddenStr,
+            assignV = show_hiddenStr}
+
 ------------------------------------------------------------------------
 -- LMOD_ADMIN_FILE: The Nag file.
 ------------------------------------------------------------------------
@@ -555,6 +602,11 @@ parseVersion  = false
 ------------------------------------------------------------------------
 s_warning     = false
 
+------------------------------------------------------------------------
+-- s_status:   When set return a non-zero status
+------------------------------------------------------------------------
+s_status      = false
+
 ------------------------------------------------------------------------
 -- s_haveWarnings:  if warning are allowed (or ignored).  For example
 --                  a try-load command turns off warnings.
@@ -599,6 +651,21 @@ cosmic:init{name = "LMOD_SETTARG_FULL_SUPPORT",
             sedV = "@lmod_settarg_full_support@",
             yn   = "no"}
 
+------------------------------------------------------------------------
+-- LMOD_NEARLY_FORBIDDEN_DAYS : The number of days where the nearly
+--                              forbidden applies.
+------------------------------------------------------------------------
+local nFD_DFLT = 14
+local nFD_Env  = getenv("LMOD_NEARLY_FORBIDDEN_DAYS")
+local nFD_SedV = "@nearly_forbidden_days@"
+local nFD      = tonumber(nFD_Env) or tonumber(nFD_SedV)
+
+cosmic:init{name    = "LMOD_NEARLY_FORBIDDEN_DAYS",
+            default = nFD_DFLT,
+            envV    = nFD_Env,
+            sedV    = nFD_SedV,
+            assignV = nFD}
+
 ------------------------------------------------------------------------
 -- LMOD_ANCIENT_TIME:  the time in seconds when the cache file is considered old
 ------------------------------------------------------------------------
@@ -611,7 +678,12 @@ cosmic:init{name    = "LMOD_ANCIENT_TIME",
             envV    = ancientEnv,
             sedV    = ancientSedV,
             assignV = ancient}
-ancient             = cosmic:value("LMOD_ANCIENT_TIME")
+ancient = cosmic:value("LMOD_ANCIENT_TIME")
+------------------------------------------------------------------------
+-- shortLifeCache: If building the cache file is fast then shorten the
+--                 ancient to this time.
+------------------------------------------------------------------------
+shortLifeCache = ancient/12
 
 ------------------------------------------------------------------------
 -- LMOD_SHORT_TIME: the time in seconds when building the cache file is quick
@@ -628,6 +700,8 @@ cosmic:init{name     = "LMOD_SHORT_TIME",
             sedV     = shortTimeSedV,
             assignV  = shortTime}
 
+
+
 ------------------------------------------------------------------------
 -- Threshold:  The amount of time to wait before printing the cache
 --             rebuild message.  (It has to be 1 second or greater).
@@ -641,12 +715,6 @@ cosmic:init{name     = "LMOD_THRESHOLD",
             assignV  = threshold}
 
 
-------------------------------------------------------------------------
--- shortLifeCache: If building the cache file is fast then shorten the
---                 ancient to this time.
-------------------------------------------------------------------------
-shortLifeCache = ancient/12
-
 ------------------------------------------------------------------------
 -- LMOD_ALLOW_ROOT_USE
 ------------------------------------------------------------------------
@@ -661,6 +729,8 @@ cosmic:init{name = "LMOD_HAVE_LUA_TERM",
             sedV = "@have_lua_term@",
             yn   = "no"}
 
+
+
 ------------------------------------------------------------------------
 -- MODULEPATH_ROOT
 ------------------------------------------------------------------------
@@ -688,6 +758,15 @@ cosmic:init{name    = "LMOD_HASHSUM_PATH",
             sedV    = "@hashsum@",
             default = HashSum}
 
+------------------------------------------------------------------------
+-- MODULES_AUTO_HANDLING: If true the make prereq -> depends_on
+--                        prereq_any -> depends_on_any
+------------------------------------------------------------------------
+
+cosmic:init{name    = "MODULES_AUTO_HANDLING",
+            sedV    = "@modules_auto_handling@",
+            default = "no"}
+
 ------------------------------------------------------------------------
 -- PATH_TO_LUA
 ------------------------------------------------------------------------
@@ -731,6 +810,14 @@ cosmic:init{name = "LMOD_HIDDEN_ITALIC",
             sedV = "@LMOD_HIDDEN_ITALIC@",
             yn   = "no"}
 
+------------------------------------------------------------------------
+-- LMOD_FILE_IGNORE_PATTERNS
+-----------------------------------------------------------------------
+local patternA = {"%.version[-._].*",  "%.modulerc[-._].*"}
+cosmic:init{name    = "LMOD_FILE_IGNORE_PATTERNS",
+            assignV = patternA,
+            default = patternA}
+
 ------------------------------------------------------------------------
 -- GIT_VERSION: The exact git version of Lmod
 ------------------------------------------------------------------------
