@@ -102,6 +102,7 @@ require("modfuncs")
 require("cmdfuncs")
 require("deepcopy")
 require("parseVersion")
+require("declare")
 MainControl         = require("MainControl")
 Cache               = require("Cache")
 MRC                 = require("MRC")
@@ -110,6 +111,7 @@ BaseShell           = require("BaseShell")
 Shell               = false
 local Optiks        = require("Optiks")
 local Spider        = require("Spider")
+local Version       = require("Version")
 local concatTbl     = table.concat
 local cosmic        = require("Cosmic"):singleton()
 local dbg           = require("Dbg"):dbg()
@@ -210,7 +212,7 @@ local function l_add2map(entry, tbl, dirA, moduleFn, kind, rmapT)
          end
          dbg.print{"assigning rmapT for path: ",path,"\n"}
          rmapT[path] = t
-         local p2 = abspath(path)
+         local p2 = realpath(path)
          if (p2 and p2 ~= path) then
             dbg.print{"assigning rmapT for path: ",p2,"\n"}
             rmapT[p2] = deepcopy(t)
@@ -220,6 +222,14 @@ local function l_add2map(entry, tbl, dirA, moduleFn, kind, rmapT)
    dbg.fini("l_add2map")
 end
 
+--------------------------------------------------------------------------
+-- Build the version string.
+function l_version()
+   local v = {}
+   v[#v + 1] = "\nModules based on Lua: Version " .. Version.name().."\n"
+   v[#v + 1] = "    by Robert McLay mclay@tacc.utexas.edu\n\n"
+   return concatTbl(v,"")
+end
 --------------------------------------------------------------------------
 --
 -- @param moduleDirA
@@ -342,6 +352,11 @@ end
 
 local function l_rptReverseMapTJson(mpathMapT, spiderT, timestampFn, dbT, providedByT)
    dbg.start{ "l_rptReverseMapTJson(mpathMapT, spiderT, timestampFn, dbT, providedByT)"}
+   if (_VERSION ~= "Lua 5.1") then
+      require("declare")
+      declare("loadstring")
+      loadstring = load
+   end
    local json        = require("json")
    local reverseMapT = l_buildReverseMapT(dbT)
    local libA        = l_buildLibMapA(reverseMapT)
@@ -354,6 +369,11 @@ end
 
 local function l_rptXALTRmapTJson(mpathMapT, spiderT, timestampFn, dbT, providedByT)
    dbg.start{ "l_rptXALTRmapTJson(mpathMapT, spiderT, timestampFn, dbT, providedByT)"}
+   if (_VERSION ~= "Lua 5.1") then
+      require("declare")
+      declare("loadstring")
+      loadstring = load
+   end
    local json        = require("json")
    local reverseMapT = l_buildReverseMapT(dbT)
    local libA        = l_buildLibMapA(reverseMapT)
@@ -366,6 +386,11 @@ end
 
 local function l_rptSoftwarePageJson(mpathMapT, spiderT, timestampFn, dbT, providedByT)
    dbg.start{ "l_rptSoftwarePageJson(mpathMapT, spiderT, timestampFn, dbT, providedByT)"}
+   if (_VERSION ~= "Lua 5.1") then
+      require("declare")
+      declare("loadstring")
+      loadstring = load
+   end
    local json = require("json")
    local spA  = softwarePage(dbT)
    print(json.encode(spA))
@@ -399,6 +424,11 @@ end
 
 local function l_rptDbTJson(mpathMapT, spiderT, timestampFn, dbT, providedByT)
    dbg.start{ "l_rptDbTJson(mpathMapT, spiderT, timestampFn, dbT, providedByT)"}
+   if (_VERSION ~= "Lua 5.1") then
+      require("declare")
+      declare("loadstring")
+      loadstring = load
+   end
    local json = require("json")
    print(json.encode(dbT))
    dbg.fini("l_rptDbTJson")
@@ -411,11 +441,15 @@ function main()
    local optionTbl  = optionTbl()
    local pargs      = optionTbl.pargs
    local mpathA     = {}
-
    Shell            = BaseShell:build("bash")
 
    local hub        = Hub:singleton(false)
 
+   if (optionTbl.version) then
+      io.stderr:write(l_version())
+      os.exit(0)
+   end
+
    for _, v in ipairs(pargs) do
       for path in v:split(":") do
          local my_path     = path_regularize(path)
@@ -442,18 +476,20 @@ function main()
    dbg.print{"LMOD_TRACING: ",cosmic:value("LMOD_TRACING"),"\n"}
 
 
-   ------------------------------------------------------------------------
-   -- do not colorize output from spider
-   colorize = plain
-
    ------------------------------------------------------------------
    -- initialize lmod with SitePackage and /etc/lmod/lmod_config.lua
    initialize_lmod()
 
+   ------------------------------------------------------------------------
+   -- do not colorize output from spider
+   colorize = plain
 
-   -- Make sure that MRC uses $MODULERCFILE and ignores ~/.modulerc when building the cache
+   local mrc        = MRC:singleton()
+
+   mrc:set_display_mode("spider")
+
+   -- Make sure that MRC uses $LMOD_MODULERC and ignores ~/.modulerc when building the cache
    local remove_MRC_home         = true
-   local mrc                     = MRC:singleton(getModuleRCT(remove_MRC_home))
    local cache                   = Cache:singleton{dontWrite = true, quiet = true, buildCache = true,
                                                    buildFresh = true, noMRC=true}
    local spider                  = Spider:new()
@@ -611,6 +647,13 @@ function options()
                                     prt     = l_prt,
    }
 
+   cmdlineParser:add_option{
+      name   = {'-v','--version'},
+      dest   = 'version',
+      action = 'store_true',
+      help   = "Version info",
+   }
+
    cmdlineParser:add_option{
       name   = {'-D'},
       dest   = 'debug',
