@@ -55,6 +55,7 @@ local decode64     = base64.decode64
 local encode64     = base64.encode64
 local floor        = math.floor
 local getenv       = os.getenv
+local hook         = require("Hook")
 local huge         = math.huge
 local load         = (_VERSION == "Lua 5.1") and loadstring or load
 local min          = math.min
@@ -65,6 +66,10 @@ local stat         = posix.stat
 local strfmt       = string.format
 local s_envT       = {}
 local s_clrEnvT    = {}
+local blank          = " "
+local s_indentLevel  = 0
+local s_indentString = ""
+
 --------------------------------------------------------------------------
 -- This is 5.1 Lua function to cover the table.pack function
 -- that is in Lua 5.2 and later.
@@ -72,7 +77,7 @@ function argsPack(...)
    local argA = { n = select("#", ...), ...}
    return argA
 end
-pack     = (_VERSION == "Lua 5.1") and argsPack or table.pack
+local pack     = (_VERSION == "Lua 5.1") and argsPack or table.pack
 
 function __FILE__()
    return debug.getinfo(2,'S').source
@@ -96,8 +101,7 @@ end
 --------------------------------------------------------------------------
 -- Generate a message that will fix the available terminal width.
 -- @param width The terminal width
-function buildMsg(width, ... )
-   local argA = pack(...)
+function buildMsg(width, argA)
    local a    = {}
    local len  = 0
 
@@ -254,23 +258,51 @@ end
 -- @param propT The property table
 -- @param legendT The legend table.  A key-value pairing of keys to descriptions.
 -- @return An array of colorized strings
-function colorizePropA(style, modT, mrc, propT, legendT)
+function colorizePropA(style, mt, modT, mrc, propT, legendT, forbiddenT)
+   dbg.start{"colorizePropA(style, mt, modT, mrc, propT, legendT)"}
    local readLmodRC   = require("ReadLmodRC"):singleton()
    local propDisplayT = readLmodRC:propT()
    local iprop        = 0
    local pA           = {}
-   local moduleName   = modT.fullName
+   local moduleName   = mt:name_w_possible_alias(modT, "full")
    propT              = propT or {}
+   forbiddenT         = forbiddenT or {}
 
-   if (not mrc:isVisible(modT)) then
+   local resultT = mrc:isVisible(modT)
+   if (resultT.moduleKindT.kind ~= "normal") then
       local i18n = require("i18n")
-      local H    = 'H'
-      moduleName = colorize("hidden",moduleName)
+      local H    = "H"
+      local msg  = "HiddenM"
+      if (resultT.moduleKindT.kind == "soft") then
+         H = "s"
+         msg = "Hidden_softM"
+      end
+      moduleName = colorize("hidden",modT.fullName)
       pA[#pA+1]  = H
-      legendT[H] = i18n("HiddenM")
+      legendT[H] = i18n(msg)
+      if (resultT.moduleKindT.hidden_loaded) then
+         pA[#pA+1]     = "HL"
+         legendT["HL"] = i18n("HiddenLoadM")
+      end
    end
 
+   if (forbiddenT.forbiddenState == "forbid") then
+      local i18n = require("i18n")
+      local F    = "F"
+      local msg  = "ForbiddenM"
+      moduleName = colorize("forbid",modT.fullName)
+      pA[#pA+1]  = F
+      legendT[F] = i18n(msg)
+   end      
 
+   if (forbiddenT.forbiddenState == "nearly") then
+      local i18n  = require("i18n")
+      local NF    = "NF"
+      local msg   = "NearlyForbiddenM"
+      moduleName  = colorize("nearly",modT.fullName)
+      pA[#pA+1]   = NF
+      legendT[NF] = i18n(msg)
+   end      
 
    local resultA      = { moduleName }
    for kk,vv in pairsByKeys(propDisplayT) do
@@ -311,6 +343,7 @@ function colorizePropA(style, modT, mrc, propT, legendT)
    if (#pA > 0) then
       resultA[#resultA+1] = concatTbl(pA,",")
    end
+   dbg.fini("colorizePropA")
    return resultA
 end
 
@@ -342,11 +375,8 @@ end
 --------------------------------------------------------------------------
 -- Find the admin file (or nag message file).
 function findAdminFn()
-   local readable    = "no"
-   local adminFn     = cosmic:value("LMOD_ADMIN_FILE")
-   if (posix.access(adminFn, 'r')) then
-      readable = "yes"
-   end
+   local adminFn  = cosmic:value("LMOD_ADMIN_FILE")
+   local readable = posix.access(adminFn, 'r') and "yes" or "no"
    return adminFn, readable
 end
 
@@ -388,9 +418,8 @@ function readAdmin()
 
       for v in whole:split("\n") do
          repeat
-            v = v:gsub("%s+$","")
-
-            if (v:sub(1,1) == "#") then
+            v = v:gsub("^%s+$","")
+            if (v:find("^%s*#")) then
                -- ignore this comment line
                break
 
@@ -446,7 +475,6 @@ function getMT()
       local envNm = strfmt(piece,i)
       local v     = getenv(envNm)
       if (v == nil) then break end
-      dbg.print{"getMT: nm:",envNm,", v: ",v,"\n"}
 
       a[#a+1]    = v
    end
@@ -460,18 +488,39 @@ end
 ------------------------------------------------------------
 -- Get the table of modulerc files with proper weights
 
+local s_fnIgnorePatternsA = { "^.*~", "^#.*", "^%.#.*", "^%..*%.swp"}
+
+
 function getModuleRCT(remove_MRC_home)
-   --dbg.start{"getModuleRCT(remove_MRC_home)"}
+   dbg.start{"getModuleRCT(remove_MRC_home: ",remove_MRC_home,")"}
    local A            = {}
-   local MRC_system   = cosmic:value("LMOD_MODULERCFILE")
+   local MRC_system   = cosmic:value("LMOD_MODULERC")
    local MRC_home     = pathJoin(getenv("HOME"), ".modulerc")
    local MRC_home_lua = pathJoin(getenv("HOME"), ".modulerc.lua")
+   dbg.print{"MRC_system: ",MRC_system,"\n"}
 
    if (MRC_system) then
       local a = {}
-      for file in MRC_system:split(":") do
-         if (isFile(file) and access(file,"r")) then
-            a[#a+1] = file
+      for n in MRC_system:split(":") do
+         if (isDir(n) and access(n,"rx")) then
+            for f in lfs.dir(n) do
+               local valid = true
+               for i = 1,#s_fnIgnorePatternsA do
+                  local patt = s_fnIgnorePatternsA[i]
+                  if (f:find(patt)) then
+                     valid = false
+                     break
+                  end
+               end
+               if (valid) then
+                  local fullPath = pathJoin(n, f)
+                  if (isFile(fullPath) and access(fullPath,"r")) then
+                     a[#a+1] = fullPath
+                  end
+               end
+            end
+         elseif (isFile(n) and access(n,"r")) then
+            a[#a+1] = n
          end
       end
       for i = #a, 1, -1 do
@@ -485,16 +534,11 @@ function getModuleRCT(remove_MRC_home)
          A[#A+1] = { MRC_home, "u"}
       end
    end
-   --dbg.printT("fnA",A)
-   --dbg.fini("getModuleRCT")
+   dbg.printT("fnA",A)
+   dbg.fini("getModuleRCT")
    return A
 end
 
-function isActiveMFile(mrc, full, sn, fn)
-   local version = extractVersion(full, sn) or ""
-   return mrc:isVisible{fullName=full, sn=sn, fn=fn}, version
-end
-
 -----------------------------------------------------------------------
 -- This function decides if the modulefile is a marked default
 -- The rule is that a marked default is given in the first character
@@ -575,10 +619,15 @@ function path2pathA(path, delim, clearDoubleSlash)
    if (path == '') then
       return { '' }
    end
-   local delimPatt = delim .. "+";
 
+   -- If path is /sw1/man::/sw2/man then
+   -- keep the double delim's
+   -- However convert /sw1/man:::/sw2/man to
+   --                 /sw1/man::/sw2/man 
 
-   path = path:gsub(delimPatt,delim)
+   local delimPatt = delim .. delim .. "+";
+   local delimStr  = delim .. delim 
+   path = path:gsub(delimPatt,delimStr)
 
    local pathA = {}
    for v  in path:split(delim) do
@@ -618,15 +667,12 @@ function regular_cmp(x,y)
 end
 
 
-
-
-
-function sanizatizeTbl(rplmntA, inT, outT)
+local function l_sanizatizeTbl(replaceA, inT, outT)
    for k, v in pairs(inT) do
       local key = k
       if (type(k) == "string") then
-         for i = 1, #rplmntA do
-            local p  = rplmntA[i]
+         for i = 1, #replaceA do
+            local p  = replaceA[i]
             local s1 = p[1]
             local s2 = p[2]
             key = key:gsub(s1,s2)
@@ -637,11 +683,11 @@ function sanizatizeTbl(rplmntA, inT, outT)
          outT[key] = nil
       elseif (type(v) == "table") then
          outT[key] = {}
-         sanizatizeTbl(rplmntA, v, outT[key])
+         l_sanizatizeTbl(replaceA, v, outT[key])
          v = outT[key]
       elseif (type(v) == "string") then
-         for i = 1,#rplmntA do
-            local p  = rplmntA[i]
+         for i = 1,#replaceA do
+            local p  = replaceA[i]
             local s1 = p[1]
             local s2 = p[2]
             v = v:gsub(s1,s2)
@@ -654,6 +700,17 @@ function sanizatizeTbl(rplmntA, inT, outT)
    end
 end
 
+function sanizatizeTbl(rplmntA, inT, outT)
+   local replaceA = {}
+   for i = 1,#rplmntA do
+      local p = rplmntA[i]
+      replaceA[i] = { p[1]:escape(), p[2]}
+   end
+   
+   l_sanizatizeTbl(replaceA, inT, outT)
+end
+
+
 --------------------------------------------------------------------------
 -- Push the Lmod Version into the environment
 function setenv_lmod_version()
@@ -717,6 +774,15 @@ local s_defaultsT = {
    priority = "0",
 }
 
+--------------------------------------------------------------------------
+-- Modify output indentation by +/- 2 ' '
+-- MC_Show
+
+function s_indent(i)
+   s_indentLevel  = s_indentLevel + i
+   s_indentString = blank:rep(s_indentLevel*2)
+end
+
 --------------------------------------------------------------------------
 -- This routine converts a command into a string.  This is used by MC_Show
 -- @param name Input command name.
@@ -761,6 +827,7 @@ function ShowCmdStr(name, ...)
    end
 
    local b = {}
+   b[#b+1] = s_indentString
    b[#b+1] = name
    b[#b+1] = left
    b[#b+1] = concatTbl(a,",")
@@ -769,6 +836,67 @@ function ShowCmdStr(name, ...)
    return concatTbl(b,"")
 end
 
+--------------------------------------------------------------------------
+-- This routine prints a help message.  This is used by MC_Show
+function ShowHelpStr(...)
+   dbg.start{"ShowHelpStr(...)"}
+
+   local argA    = pack(...)
+   local a       = {}
+   a[1]          = s_indentString .. "help([["
+   for i = 1,argA.n do
+      for line in argA[i]:split("\n") do
+         a[#a + 1] = line
+         a[#a + 1] = "\n" .. s_indentString
+      end
+      a[#a] = "]],[[\n" .. s_indentString 
+   end
+   a[#a] = "]])\n"
+   dbg.fini("ShowHelpStr")
+   return concatTbl(a,"")
+
+end
+
+local s_decoyT    = false
+local s_decorateT = false
+
+function decorateModule(name, resultT, forbiddenT)
+   if (not s_decorateT) then
+      s_decoyT = {
+         forbiddenState = "normal"
+      }
+      local decorate = cosmic:value("LMOD_TERSE_DECORATIONS")
+      if (decorate == "yes") then
+         s_decorateT = {
+            normal  = "",
+            hidden  = " <H>",
+            soft    = " <H>",
+            forbid  = " <F>",
+            nearly  = " <NF>",
+         }
+      else
+         s_decorateT = {
+            normal  = "",
+            hidden  = "",
+            soft    = "",
+            forbid  = "",
+            nearly  = "",
+         }
+      end
+   end
+   
+   local fT = forbiddenT
+   if (not forbiddenT or next(forbiddenT) == nil) then
+      fT = s_decoyT
+   end
+
+   local a = {}
+   a[#a+1] = name
+   a[#a+1] = s_decorateT[resultT.moduleKindT.kind or "normal"]
+   a[#a+1] = s_decorateT[fT.forbiddenState]
+   return concatTbl(a,"")
+end
+   
 --------------------------------------------------------------------------
 -- Unique string that combines the current time/date
 -- with a uuid id string.
@@ -833,23 +961,17 @@ function haveWarnings()
    return s_haveWarnings
 end
 
---------------------------------------------------------------------------
--- Reset warning flag to false.
-function clearWarningFlag()
-   s_warning = false
-end
-
 --------------------------------------------------------------------------
 -- Set warning flags to true.
-function setWarningFlag()
-   dbg.print{"setting warning flag\n"}
-   s_warning = true
+function setStatusFlag()
+   dbg.print{"setting status flag\n"}
+   s_status = true
 end
 
 --------------------------------------------------------------------------
 -- Get warning flag value.
-function getWarningFlag()
-   return s_warning
+function getStatusFlag()
+   return s_status
 end
 
 local function l_restoreEnv(oldEnvT, newEnvT)
@@ -1100,6 +1222,20 @@ function initialize_lmod()
    cosmic:set_key("SitePkg")
    require("SitePackage")
    cosmic:set_key("Other")
+   colorize_init()
+   if (cosmic:value("LMOD_TMOD_PATH_RULE") == "yes") then
+      cosmic:assign("LMOD_DUPLICATE_PATHS", "no")
+   end
+   local ignore_cache = cosmic:value("LMOD_IGNORE_CACHE") == "yes"
+   local cached_loads = cosmic:value("LMOD_CACHED_LOADS")
+   cosmic:assign("LMOD_CACHED_LOADS", ignore_cache and "no" or cached_loads)
+   local ancient = cosmic:value("LMOD_ANCIENT_TIME")
+   ------------------------------------------------------------------------
+   -- shortLifeCache: If building the cache file is fast then shorten the
+   --                 ancient to this time.
+   ------------------------------------------------------------------------
+   shortLifeCache = ancient/12
+
 end
 
 function tracing_msg(msgA)
@@ -1170,10 +1306,17 @@ function locatePkg(pkg)
    for path in package.path:split(";") do
       local s = path:gsub("?",pkg)
       if (isFile(s)) then
-         result = abspath(s)
+         result = realpath(s)
          break
       end
    end
    return result
 end
 
+function wrap_complete(name)
+   return "complete<" .. name .. ">"
+end
+function unwrap_complete(name)
+   local i,j,n = name:find("complete<([^<]*)>")
+   return n
+end
