@@ -47,7 +47,6 @@ require("strict")
 require("fileOps")
 require("pairsByKeys")
 require("TermWidth")
-require("lmod_system_execute")
 
 --------------------------------------------------------------------------
 -- Convert the string value into a quoted string of some kind and boolean
@@ -107,11 +106,12 @@ local keywordT = {
    ['nil']    = true,  ['not']    = true,    ['or']       = true,
    ['repeat'] = true,  ['return'] = true,    ['then']     = true,
    ['true']   = true,  ['until']  = true,    ['while']    = true,
+   ['']       = true,  [' ']      = true,
 }
 
 local function wrap_name(indent, name)
    local str
-   if (name:find("[^0-9A-Za-z_]") or keywordT[name] or name == " " or
+   if (name:find("[^0-9A-Za-z_]") or keywordT[name] or
        name:sub(1,1):find("[0-9]") ) then
       str = indent .. "[\"" .. name .. "\"]"
    else
@@ -264,7 +264,7 @@ function serializeTbl(options)
    local fn = options.fn
    local d  = dirname(fn)
    if (not isDir(d)) then
-      lmod_system_execute('mkdir -p ' .. d)
+      os.execute('mkdir -p ' .. d)
    end
    local f  = assert(io.open(fn, "w"))
 
